Imports Aricie.DNN.Configuration
Imports Aricie.DNN.Diagnostics
Imports Aricie.DNN.Modules.LuceneSearch.Configuration
Imports Aricie.DNN.Services
Imports Aricie.Services

Imports DotNetNuke.Common
Imports DotNetNuke.Services.Localization

Imports Lucene.Net.Index
Imports Lucene.Net.QueryParsers
Imports Lucene.Net.Search

Imports System.Globalization
Imports Aricie.DNN.Modules.LuceneSearch.Settings
Imports Aricie.DNN.Modules.LuceneSearch.SiteMap
Imports DotNetNuke.Entities.Modules
Imports DotNetNuke.Services.Search

Imports System.Linq
Imports Lucene.Net.Search.Highlight

Namespace Aricie.DNN.Modules.LuceneSearch.Business
    Public Enum LuceneLogType
        IndexRetrieval
        IndexProcessing
        Storage
        Upgrade
        Retrieval
        Indexation
        Infrastructure
    End Enum

    ''' <summary>
    ''' Business Controller Class
    ''' </summary>
    Public Class BusinessController

#Region "Private members"

        Private Shared _IsLuceneProviderInstalled As Nullable(Of Boolean)
        Private Shared _LuceneSearchResultsDefId As Integer = -1

#End Region

#Region "Properties"


        Public Shared ReadOnly Property LuceneSearchResultsDefId() As Integer
            Get
                If _LuceneSearchResultsDefId = -1 Then
                    _LuceneSearchResultsDefId = _
                        GetModuleDefIdByModuleName(Aricie.DNN.Modules.LuceneSearch.Common.Constants.ModuleNameLuceneSearchResults)
                End If
                Return _LuceneSearchResultsDefId
            End Get
        End Property



#End Region


#Region "Methods"


#Region "Config methods"

        Public Shared Function IsLuceneProviderInstalled() As Boolean
            If Not _IsLuceneProviderInstalled.HasValue Then
                _IsLuceneProviderInstalled = ConfigHelper.IsInstalled(New LuceneSearchConfigUpdate(), True)
            End If
            Return _IsLuceneProviderInstalled.Value
        End Function



        Public Shared Function GetSearchModuleInstances(ByVal portalId As Integer) As List(Of ModuleInfo)
            Dim modules As ArrayList = NukeHelper.ModuleController.GetModulesByDefinition(portalId, Aricie.DNN.Modules.LuceneSearch.Common.Constants.ModuleNameLuceneSearchResults)
            Dim toReturn As New List(Of ModuleInfo)(modules.Count)
            For Each objModule As ModuleInfo In modules
                toReturn.Add(objModule)
            Next
            Return toReturn
        End Function

#End Region

#Region "String methods"

        Public Shared Function GetSearchUrl(ByVal resultsTabId As Integer, ByVal SearchFilterQuery As LuceneFilterQuery) As String
            Dim Filter As String = String.Empty
            If SearchFilterQuery.FilterQuery IsNot Nothing Then
                Filter = SearchFilterQuery.FilterQuery.ToString
            End If
            Return GetSearchUrl(resultsTabId, SearchFilterQuery.SearchQuery, Filter)
        End Function


        Public Shared Function GetSearchUrl(ByVal resultsTabId As Integer, ByVal searchQuery As String) As String
            Return GetSearchUrl(resultsTabId, searchQuery, "")
        End Function

        Public Shared Function GetSearchUrl(ByVal resultsTabId As Integer, ByVal searchQuery As String, _
                                             ByVal filter As String) As String

            Dim searchCriteria As String = GetSearchQueryString(searchQuery)
            Dim filterCriteria As String = GetFilterQueryString(filter)

            Dim p As New List(Of String)
            If (Not String.IsNullOrEmpty(searchCriteria)) Then
                p.Add(searchCriteria)
            End If
            If (Not String.IsNullOrEmpty(filterCriteria)) Then
                p.Add(filterCriteria)
            End If
            Return NavigateURL(resultsTabId, String.Empty, p.ToArray)


        End Function

        Private Shared Function GetFilterQueryString(ByVal filter As String) As String
            Return GetEncodedQueryString(Common.Constants.Http.FilterCriteria, filter)
        End Function

        Private Shared Function GetSearchQueryString(ByVal searchQuery As String) As String
            Return GetEncodedQueryString(Common.Constants.Http.SearchCriteria, searchQuery)
        End Function

        Private Shared Function GetEncodedQueryString(key As String, value As String) As String
            If String.IsNullOrEmpty(value) Then
                Return String.Empty
            End If
            Return String.Format("{0}={1}", key, HttpUtility.UrlEncode(LuceneEscape(value)))
        End Function

        Public Shared Function FormatSearchCriteria(ByVal inputText As String) As String
            Dim toReturn As String = inputText.Trim(" "c)
            toReturn = LuceneEscape(toReturn)
            Return toReturn
        End Function

        Public Shared Function GetRefinedQuery(ByVal baseQuery As Query, ByVal additionalFilter As Query) As Query
            Dim toReturn As Query
            If Not baseQuery Is Nothing Then
                Dim boolQuery As BooleanQuery
                If baseQuery Is GetType(BooleanQuery) Then
                    boolQuery = DirectCast(baseQuery, BooleanQuery)
                Else
                    boolQuery = New BooleanQuery()
                    boolQuery.Add(baseQuery, Occur.MUST)
                End If
                boolQuery.Add(additionalFilter, Occur.MUST)
                toReturn = boolQuery
            Else
                toReturn = additionalFilter
            End If
            Return toReturn
        End Function


        Public Shared Function GetRefinedSearchUrl(ByVal resultsTabId As Integer, ByVal additionalFilter As Query) _
            As String


            Dim currentQuery As Query = GetHttpFilterQuery()
            Dim newFilter As Query = GetRefinedQuery(currentQuery, additionalFilter)
            Dim searchQuery As String = HttpContext.Current.Request.QueryString(Common.Constants.Http.SearchCriteria)
            If String.IsNullOrEmpty(searchQuery) Then
                searchQuery = ""
            End If

            Return GetSearchUrl(resultsTabId, searchQuery, newFilter.ToString)

        End Function

        Public Shared Function LuceneEscape(ByVal criteria As String) As String
            Return IUrlParametersProcessorFactory.Instance.Encode(criteria)
        End Function

        Public Shared Function LuceneUnEscape(ByVal criteria As String) As String
            Return IUrlParametersProcessorFactory.Instance.Decode(criteria)
        End Function


#End Region

#Region "Http Methods"

        Public Shared Property HttpFilterString() As String
            Get
                Dim _
                    toReturn As String = _
                        DirectCast(HttpContext.Current.Items(Common.Constants.Http.FilterCriteria), String)
                If String.IsNullOrEmpty(toReturn) Then
                    toReturn = HttpContext.Current.Request.Params(Common.Constants.Http.FilterCriteria)

                    If toReturn IsNot Nothing Then
                        toReturn = LuceneUnEscape(LuceneUnEscape(toReturn))
                        HttpContext.Current.Items(Common.Constants.Http.FilterCriteria) = toReturn
                    End If
                End If
                Return DirectCast(IIf(toReturn Is Nothing, "", toReturn), String)
            End Get
            Set(ByVal value As String)
                HttpContext.Current.Items(Common.Constants.Http.FilterCriteria) = value
                ResetHttpQueries()
            End Set
        End Property


        Public Shared Property HttpCriteriaSring() As String
            Get
                Dim _
                    toReturn As String = _
                        DirectCast(HttpContext.Current.Items(Common.Constants.Http.SearchCriteria), String)
                If String.IsNullOrEmpty(toReturn) Then

                    toReturn = HttpContext.Current.Request.Params(Common.Constants.Http.SearchCriteria)
                    If toReturn IsNot Nothing Then
                        toReturn = LuceneUnEscape(LuceneUnEscape(toReturn))
                        HttpContext.Current.Items(Common.Constants.Http.SearchCriteria) = toReturn
                    End If

                End If
                Return DirectCast(IIf(toReturn Is Nothing, "", toReturn), String)
            End Get
            Set(ByVal value As String)
                HttpContext.Current.Items(Common.Constants.Http.SearchCriteria) = value
                ResetHttpQueries()
            End Set
        End Property


        Private Shared Sub ResetHttpQueries()
            HttpContext.Current.Items(Common.Constants.Http.FilterQuery) = Nothing
            HttpContext.Current.Items(Common.Constants.Http.SearchQuery) = Nothing
        End Sub


        Public Shared Function GetHttpFilterQuery() As Query
            Dim toReturn As Query = DirectCast(HttpContext.Current.Items(Common.Constants.Http.FilterQuery), Query)

            If toReturn Is Nothing Then

                Dim strquery As String = HttpFilterString
                If Not String.IsNullOrEmpty(strquery) Then
                    toReturn = GetQuery(NukeHelper.PortalId, strquery)
                Else
                    Return Nothing
                End If

                HttpContext.Current.Items(Common.Constants.Http.FilterQuery) = toReturn
            End If
            Return toReturn
        End Function



#End Region

#Region "Lucene Methods"


        Public Shared Function GetCurrentLocaleQuery() As Query
            Dim criteria As String = New Localization().CurrentCulture
            Dim toReturn As New PhraseQuery()
            toReturn.Add(New Term(LuceneGeneralField.Language, criteria))
            Return toReturn
        End Function


        'just to fix a bug since the Text function and text members seem to interfere with one another
        Public Shared Function GetTermText(ByVal term As Term) As String
            Return term.[Text]
        End Function

        Public Shared Function GetTerms(ByVal portalId As Integer, ByVal strFilter As String) As Dictionary(Of String, WeightedTerm)
            Dim toReturn As Dictionary(Of String, WeightedTerm) = CacheHelper.GetGlobal(Of Dictionary(Of String, WeightedTerm))("LuceneTerms", portalId.ToString(CultureInfo.InvariantCulture), strFilter)
            If toReturn Is Nothing Then
                toReturn = ExtractTerms(GetQuery(portalId, strFilter))
                CacheHelper.SetCacheDependant(Of Dictionary(Of String, WeightedTerm))(toReturn, Common.Constants.Cache.DependencyKey, TimeSpan.FromMinutes(5), "LuceneTerms", portalId.ToString(CultureInfo.InvariantCulture), strFilter)
            End If
            Return toReturn
        End Function

        Public Shared Function GetQuery(ByVal portalId As Integer, ByVal strFilter As String) As Query
            If String.IsNullOrEmpty(strFilter) Then
                Return Nothing
            End If
            Dim queryingDriver As PortalQueryingDriver = PortalQueryingDriver.GetByPortalId(portalId)
            Dim objParser As QueryParser = queryingDriver.GetDefaultQueryParser("")
            Return objParser.Parse(strFilter)
        End Function



        Public Shared Function ExtractTerms(ByVal objQuery As Query) As Dictionary(Of String, WeightedTerm)
            Dim toReturn As New Dictionary(Of String, WeightedTerm)
            ExtractTerms(objQuery, toReturn)
            Return toReturn
        End Function

        Public Shared Function ExtractAllTerms(ByVal objQuery As Query) As ISet(Of Term)
            Dim nativeExtract As New HashSet(Of Term)
            'Rewrites Must be Mandatory to ExtractTerms
            objQuery = objQuery.Rewrite(PortalQueryingDriver.GetByPortalId(PortalId).GetIndexReader(False))
            objQuery.ExtractTerms(nativeExtract)

            Return nativeExtract
        End Function

        Public Shared Function ExtractAllTermsGetFirst(ByVal objQuery As Query) As Term
            Dim toReturn As Term = Nothing
            Dim nativeExtract As ISet(Of Term) = ExtractAllTerms(objQuery)

            If nativeExtract.Count > 0 Then
                'TODO : Watch to do better
                For Each keyTerm As Term In nativeExtract
                    toReturn = keyTerm
                    Exit For
                Next
            End If
            Return toReturn
        End Function

        Private Shared Sub ExtractTerms(ByVal objQuery As Query, ByRef extract As Dictionary(Of String, WeightedTerm))

            Dim nativeExtract As ISet(Of Term) = ExtractAllTerms(objQuery)

            Dim objTerm As Term
            For Each objTerm In nativeExtract
                If Not extract.ContainsKey(objTerm.Field.ToLowerInvariant) Then
                    extract(objTerm.Field.ToLowerInvariant) = New WeightedTerm(objQuery.Boost, GetTermText(objTerm))
                Else
                    ' Util pour la rcriture du filtre par mots-cls, on concatne les terms si ils ont le mme champs
                    Dim myPreviousTerms As String = extract(objTerm.Field.ToLowerInvariant).Term
                    myPreviousTerms += "/"
                    extract(objTerm.Field.ToLowerInvariant) = New WeightedTerm(objQuery.Boost, myPreviousTerms & GetTermText(objTerm))
                End If
            Next

        End Sub



        Public Shared Function GetUid(ByVal item As SearchItemInfo) As String
            Return DirectCast(item, LuceneSearchItemInfo).UniqueId
        End Function


#End Region

#Region "Sitemap Methods"


        Public Shared Function GetSitemap(ByVal portalId As Integer) As UrlCollection

            Dim urlCollection As New UrlCollection

            Dim contentUri As Uri
            Dim frequency As ChangeFrequency

            Dim priority As Decimal
            Dim AllUrlAddedCollection As New StringCollection

            Dim nbResultsFound As Integer = 0
            Dim portalDocs As IList(Of LuceneSearch.Business.LuceneSearchResultsInfo) = SearchController.GetPortalDocuments(portalId, nbResultsFound)
            For Each result As LuceneSearchResultsInfo In portalDocs
                contentUri = New Uri(DotNetNuke.Common.Globals.NavigateURL(result.TabId, "", result.Guid))

                If Not AllUrlAddedCollection.Contains(contentUri.AbsoluteUri) Then
                    AllUrlAddedCollection.Add(contentUri.AbsoluteUri)
                    frequency = CType(result.SiteMapFrequency, ChangeFrequency)
                    priority = result.SiteMapPriority
                    urlCollection.Add(New Url(contentUri, result.PubDate, frequency, priority))
                End If

            Next

            Return urlCollection
        End Function

#End Region

#Region "Log"





        Public Shared Sub LogIndexingStep(ByVal logType As LuceneLogType, ByVal portalId As Integer, _
                                           ByVal stepStatus As String, ByVal lastStep As Boolean, _
                                           Optional ByVal statPropertyName As String = "", _
                                           Optional ByVal statPropertyValue As Integer = 0)

            Dim objStep As StepInfo
            If Not String.IsNullOrEmpty(statPropertyName) Then
                Dim objPair As New KeyValuePair(Of String, String)(statPropertyName, statPropertyValue.ToString(CultureInfo.InvariantCulture))
                objStep = New StepInfo(logType.ToString, stepStatus, WorkingPhase.InProgress, lastStep, True, portalId, "", objPair)

            Else
                objStep = New StepInfo(logType.ToString, stepStatus, WorkingPhase.InProgress, lastStep, True, portalId, "")
            End If

            Global.LuceneSearch.Monitoring.Logging.DirectToAsyncLogger.Instance.AddDebugInfo(objStep)

        End Sub


#End Region

#End Region


    End Class
End Namespace


