Imports DotNetNuke.Services.Search
Imports System.Threading
Imports Aricie.Services
Imports DotNetNuke.Services.Exceptions
Imports DotNetNuke.Entities.Users
Imports DotNetNuke.Entities.Tabs
Imports Aricie.DNN.Services
Imports DotNetNuke.Security
Imports DotNetNuke.Entities.Modules
Imports DotNetNuke.Entities.Portals
Imports Aricie.DNN.Modules.LuceneSearch.Settings
Imports Aricie.DNN.Settings
Imports Aricie.DNN.Modules.LuceneSearch.Logging
Imports System.Linq


Namespace Aricie.DNN.Modules.LuceneSearch.Business
    ''' <summary>
    ''' SearchDataStore Provider For DotNetNuke using Lucene.Net
    ''' </summary>
    Public Class LuceneSearchDataStore
        Inherits SearchDataStoreProvider


#Region "Overrides"

        Public Overrides Function GetSearchItems(ByVal PortalID As Integer, ByVal TabID As Integer, _
                                                ByVal ModuleID As Integer) As SearchResultsInfoCollection
            Return _
                GetSearchResults(PortalID, _
                                  DnnSearchItemField.ModuleId & ":" & ModuleID.ToString & " AND " & _
                                  DnnSearchItemField.TabId & ":" & TabID.ToString)

        End Function



        ''' <summary>
        ''' Store the SearchItemInfos in the indexer
        ''' </summary>
        ''' <param name="SearchItems">SearchItems to store</param>
        Public Overrides Sub StoreSearchItems(ByVal searchItems As SearchItemInfoCollection)
            ' on ne stocke les donnes que si le thread qui arrive ici est celui qui a procd  l'indexation du contenu
            If (LuceneIndexer.ThisThreadHandlesSearchIndexationAndStorage) Then
                Try
                    'Cre une structure de donnes qui a pour clefs chacune des portalid de l'instance et pour valeur une collection de searchiteminfo
                    Dim myDicoOfPortalIdListOfSearchItemInfo As New Dictionary(Of Integer, SearchItemInfoCollection)
                    Dim objPortalController As New PortalController
                    Dim arrPortals As ArrayList = objPortalController.GetPortals()
                    For Each myPortalInfo As PortalInfo In arrPortals
                        myDicoOfPortalIdListOfSearchItemInfo.Add(myPortalInfo.PortalID, New SearchItemInfoCollection)
                    Next
                    'mettre chaque searchitem dans la liste de searchiteminfos correspondant  son portalid
                    For Each mySearchItemInfo As SearchItemInfo In searchItems

                        Dim portalId As Integer = -1
                        If TypeOf mySearchItemInfo Is LuceneSearchItemInfo Then
                            portalId = DirectCast(mySearchItemInfo, LuceneSearchItemInfo).PortalId
                        End If

                        If (portalId = -1) Then
                            portalId = NukeHelper.GetPortalIdByModuleId(mySearchItemInfo.ModuleId)
                        End If

                        myDicoOfPortalIdListOfSearchItemInfo(portalId).Add(mySearchItemInfo)
                    Next
                    'Store chacune des listes dans un index de portail

                    For Each portalId As Integer In myDicoOfPortalIdListOfSearchItemInfo.Keys
                        Dim settings As LucenePortalSettings = GetModuleSettings(Of LucenePortalSettings)(SettingsScope.PortalSettings, portalId)

                        If Not String.IsNullOrEmpty(settings.IndexFolderMapPath) Then
                            Dim mySearchItemInfoCollection As SearchItemInfoCollection = myDicoOfPortalIdListOfSearchItemInfo(portalId)
                            If settings.LogIndexingProcess Then
                                BusinessController.LogIndexingStep(LuceneLogType.Storage, portalId, "Starting Storing documents for portal " & portalId.ToString, False)
                            End If
                            
                            PortalIndexingDriver.GetByPortalId(portalId).StoreSearchItems(portalId, mySearchItemInfoCollection, settings.LogIndexingProcess)

                            GC.Collect(2, GCCollectionMode.Forced)
                            GC.GetTotalMemory(True)
                            If settings.LogIndexingProcess Then
                                BusinessController.LogIndexingStep(LuceneLogType.Storage, portalId, "Finished storing documents for portal " & portalId.ToString, True)
                            End If
                        End If
                    Next
                    myDicoOfPortalIdListOfSearchItemInfo.Clear()
                    CacheHelper.RemoveCache(Common.Constants.Cache.DependencyKey)
                Catch ex As Exception
                    ' on relance l'exception, elle n'tait pas traite initialement
                    Throw
                Finally
                    ' les oprations sont termines, on vide tout
                    LuceneIndexer.ThisThreadHandlesSearchIndexationAndStorage = False
                    LuceneIndexer.ThreadPickedUpForRunningIndexation = False
                End Try

            End If




        End Sub


#End Region

#Region "Public methods"

        ''' <summary>
        ''' Search a specified String in the index on a given portal
        ''' </summary>
        ''' <param name="PortalID">PortalID where to search</param>
        ''' <param name="Criteria">Query String to search in the index</param>
        ''' <returns>SearchResultsInfoCollection containing the results of the query</returns>
        Public Overrides Function GetSearchResults(ByVal portalID As Integer, ByVal criteria As String) _
            As SearchResultsInfoCollection
            Dim nbResultsFound As Integer = 0
            Dim objLuceneFilter As New LuceneFilterQuery
            objLuceneFilter.SearchQuery = criteria
            Dim objSearchParams As New SearchParams
            objSearchParams.LuceneFilterQuery = objLuceneFilter
            Return GetSearchResults(portalID, -1, objSearchParams, nbResultsFound)

        End Function


        ''' <summary>
        ''' 
        ''' </summary>
        ''' <param name="portalID">ID of the portal</param>
        ''' <param name="moduleId">ID of the module</param>
        ''' <param name="objSearchParams">Class for all search params (Query, SortFields, PageSize and PageIndex)</param>
        ''' <param name="nbResultsFound">Fill by the number of result found</param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Overloads Function GetSearchResults(ByVal portalID As Integer, ByVal moduleId As Integer, ByVal objSearchParams As SearchParams, _
                         ByRef nbResultsFound As Integer) As SearchResultsInfoCollection

            Dim tempResult As LuceneQueryResults = GetSearchResults(portalID, moduleId, objSearchParams)
            Dim toReturn As SearchResultsInfoCollection = tempResult.AsSearchResultsCollection(moduleId, True)
            nbResultsFound = tempResult.NbResults
            Return toReturn
        End Function

        Public Overloads Function GetSearchResults(ByVal portalID As Integer, ByVal moduleId As Integer, ByVal objSearchParams As SearchParams) As LuceneQueryResults

            Dim toReturn As LuceneQueryResults = Nothing

            Try
                Dim queryDriver As PortalQueryingDriver = PortalQueryingDriver.GetByPortalId(portalID)

                toReturn = queryDriver.RunQuery(portalID, moduleId, objSearchParams.LuceneFilterQuery, objSearchParams.SortField, objSearchParams.PageIndex, objSearchParams.PageSize)

            Catch ex As Exception
                ExceptionHelper.LogException(ex)
                toReturn = LuceneQueryResults.Empty
            End Try

            Return toReturn

        End Function


        Public Function GetCurrentIndex(ByVal portalId As Integer) As LuceneQueryResults

            Dim objFilterQuery As New LuceneFilterQuery

            objFilterQuery.FilterQuery = Nothing
            objFilterQuery.SearchQuery = ""
            Dim objSearchParams As New SearchParams
            objSearchParams.LuceneFilterQuery = objFilterQuery
            objSearchParams.SortField = LuceneGeneralField.IndexedDate
            Return Me.GetSearchResults(portalId, -1, objSearchParams)


        End Function



#End Region

#Region "Obsolete methods"

        <Obsolete("Please use the LuceneFilterQuery Structure rather than 'criteria' and 'additionnalFilter' string params")> _
        Public Overloads Function GetSearchResults(ByVal portalID As Integer, ByVal criteria As String, _
                                                ByVal additionalFilter As String, ByRef nbResultsFound As Integer) As SearchResultsInfoCollection
            Dim objFilterQuery As New LuceneFilterQuery
            If Not String.IsNullOrEmpty(additionalFilter) Then
                objFilterQuery.FilterQuery = BusinessController.GetQuery(portalID, additionalFilter)
            End If
            objFilterQuery.SearchQuery = criteria
            Dim objSearchParams As New SearchParams
            objSearchParams.LuceneFilterQuery = objFilterQuery

            Return GetSearchResults(portalID, -1, objSearchParams, nbResultsFound)
        End Function

        <Obsolete("Please use the LuceneFilterQuery Structure rather than criteria and additionnalFilter params")> _
        Public Overloads Function GetSearchResults(ByVal portalID As Integer, ByVal moduleId As Integer, ByVal criteria As String, ByVal additionalFilter As String, _
                         ByVal sorters As String, ByRef nbResultsFound As Integer) As SearchResultsInfoCollection
            Dim objFilterQuery As New LuceneFilterQuery

            If Not String.IsNullOrEmpty(additionalFilter) Then
                objFilterQuery.FilterQuery = BusinessController.GetQuery(portalID, additionalFilter)
            End If
            objFilterQuery.SearchQuery = criteria

            Dim objSearchParams As New SearchParams
            objSearchParams.LuceneFilterQuery = objFilterQuery
            objSearchParams.SortField = sorters

            Dim tempResult As LuceneQueryResults = GetSearchResults(portalID, moduleId, objSearchParams)
            Dim toReturn As SearchResultsInfoCollection = tempResult.AsSearchResultsCollection(moduleId, True)
            nbResultsFound = tempResult.NbResults
            Return toReturn
        End Function


        <Obsolete("Please use the LuceneFilterQuery Structure rather than criteria and additionnalFilter params")> _
        Public Overloads Function GetSearchResults(ByVal portalID As Integer, ByVal moduleId As Integer, ByVal criteria As String, ByVal additionalFilter As String, _
                        ByVal sorters As String) As LuceneQueryResults
            Dim objFilterQuery As New LuceneFilterQuery
            If Not String.IsNullOrEmpty(additionalFilter) Then
                objFilterQuery.FilterQuery = BusinessController.GetQuery(portalID, additionalFilter)
            End If
            objFilterQuery.SearchQuery = criteria

            Dim objSearchParams As New SearchParams
            objSearchParams.LuceneFilterQuery = objFilterQuery
            objSearchParams.SortField = sorters

            Return GetSearchResults(portalID, moduleId, objSearchParams)

        End Function



        <Obsolete("Please use the LuceneFilterQuery Structure rather than criteria and additionnalFilter params")> _
        Public Overloads Function GetSearchResults(ByVal portalID As Integer, ByVal moduleId As Integer, ByVal criteria As String, ByVal additionalFilter As String, _
                        ByVal sorters As String, ByVal resultIndex As Integer, ByVal pageSize As Integer) As LuceneQueryResults
            Dim objFilterQuery As New LuceneFilterQuery
            If Not String.IsNullOrEmpty(additionalFilter) Then
                objFilterQuery.FilterQuery = BusinessController.GetQuery(portalID, additionalFilter)
            End If
            objFilterQuery.SearchQuery = criteria

            Dim objSearchParams As New SearchParams
            objSearchParams.LuceneFilterQuery = objFilterQuery
            objSearchParams.SortField = sorters
            objSearchParams.PageIndex = resultIndex
            objSearchParams.PageSize = pageSize

            Return GetSearchResults(portalID, moduleId, objSearchParams)

        End Function



#End Region

    End Class
End Namespace

