Imports System.Globalization
Imports Lucene.Net.Documents
Imports DotNetNuke.Services.Search
Imports DotNetNuke.Entities.Modules
Imports DotNetNuke.Entities.Tabs
Imports System.Xml.Serialization
Imports Aricie.DNN.Services
Imports Aricie.DNN.Settings
Imports DotNetNuke.Entities.Users
Imports Aricie.DNN.Modules.LuceneSearch.SiteMap
Imports Aricie.DNN.Modules.LuceneSearch.Settings
Imports DotNetNuke.Security.Permissions
Imports Aricie.Services

Namespace Aricie.DNN.Modules.LuceneSearch.Business
    Public Class LuceneSearchItemInfo
        Inherits SearchItemInfo

        Public Const NO_PROVIDER As String = "Native"

        Private _ModuleName As String = ""
        Private _PortalId As Integer = -1
        Private _Tabid As Integer = -1
        Private _ModuleInfo As ModuleInfo
        Private _TabInfo As TabInfo
        'Private _CategoryId As Integer = -1
        Private _SiteMapFrequency As ChangeFrequency
        Private _SiteMapPriority As Decimal
        Private _SiteMapFrequencyIsSet As Boolean = False
        Private _SiteMapPriorityIsSet As Boolean = False

        Private _IconPathSetTriggered As Boolean = False
        Private _IconPath As String = String.Empty
        Private _ImagePath As String = String.Empty

        Private _ItemType As String = String.Empty

        Private _ItemGroup As String = String.Empty

        Private TaxonomyInitialized As Boolean = False


#Region "cTors"

        Private Sub New(ByVal searchItem As SearchItemInfo, portalId As Integer)
            MyBase.New(searchItem.Title, searchItem.Description, searchItem.Author, searchItem.PubDate, _
                        searchItem.ModuleId, searchItem.SearchKey, searchItem.Content, searchItem.GUID, _
                        searchItem.ImageFileId)
            Me.PortalId = portalId
            ' rcuprons en une seule fois les donnes
            RetrieveInfoFromModuleId()

            If TypeOf searchItem Is LuceneSearchItemInfo Then
                Me.AdditionalFields = CType(searchItem, LuceneSearchItemInfo).AdditionalFields
            Else
                Try
                    Dim lucenePortalSets As LucenePortalSettings = Aricie.DNN.Settings.SettingsController.GetModuleSettings(Of LucenePortalSettings)(SettingsScope.PortalSettings, Me.PortalId)
                    _SiteMapFrequency = lucenePortalSets.DefaultChangeFrequency
                    _SiteMapPriority = lucenePortalSets.DefaultPriority
                Catch ex As Exception

                End Try

            End If
        End Sub

        Private Sub New(ByVal searchItem As SearchItemInfo, portalId As Integer, ByVal providerName As String)
            Me.New(searchItem, portalId)
            Me.ProviderName = providerName
        End Sub

        Public Sub New(ByVal luceneDocument As Document)
            MyBase.New()
            Me.ParseDocument(luceneDocument)
            Dim lucenePortalSets As LucenePortalSettings = Aricie.DNN.Settings.SettingsController.GetModuleSettings(Of LucenePortalSettings)(SettingsScope.PortalSettings, PortalId)
            _SiteMapFrequency = lucenePortalSets.DefaultChangeFrequency
            _SiteMapPriority = lucenePortalSets.DefaultPriority
        End Sub

        Public Sub New(portalId As Integer)
            MyBase.New()
            Me.PortalId = portalId
            Me.ModuleId = Aricie.DNN.Modules.LuceneSearch.Common.Constants.SearchAndResultItemInfos.ModuleIdCannotBeDetermined
        End Sub

        Public Shared Function GetLuceneSearchItem(ByVal searchItem As SearchItemInfo, portalId As Integer, ByVal providerName As String) As LuceneSearchItemInfo
            If TypeOf searchItem Is LuceneSearchItemInfo Then
                Return DirectCast(searchItem, LuceneSearchItemInfo)
            Else
                Return New LuceneSearchItemInfo(searchItem, portalId, providerName)
            End If
        End Function

#End Region

        Public Property AdditionalFields() As new Document
          
        <XmlIgnore()> _
        Public ReadOnly Property FieldValue(ByVal fieldName As String) As String
            Get
                Dim toReturn As String = Me._AdditionalFields.Get(fieldName)
                If toReturn Is Nothing Then
                    Return ""
                End If
                Return toReturn
            End Get
        End Property


        Friend Property ProviderName() As String = NO_PROVIDER

        Public Property UpgradeProviderName As String

        Public Property ItemId() As Integer = -1

        <XmlIgnore()> _
        Public ReadOnly Property IsModule() As Boolean
            Get
                Return Me.ModuleId > 0
            End Get
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property ModuleName() As String
            Get
                RetrieveInfoFromModuleId()
                Return Me._ModuleName
            End Get
        End Property

        <XmlIgnore()> _
        Public Property PortalId() As Integer
            Get
                If Me.IsModule AndAlso Me._PortalId = -1 Then
                    Me._PortalId = NukeHelper.GetPortalIdByModuleId(Me.ModuleId)
                End If
                Return Me._PortalId
            End Get
            Set(ByVal value As Integer)
                Me._PortalId = value
            End Set
        End Property


        <XmlIgnore()> _
        Public Property TabId() As Integer
            Get
                If Me._Tabid = -1 Then
                    If Me.IsModule Then
                        Me._Tabid = GetTabIdByModuleId(Me.ModuleId)
                    Else
                        Dim searchModules As List(Of ModuleInfo) = BusinessController.GetSearchModuleInstances(Me.PortalId)
                        If searchModules.Count > 0 Then
                            Me._Tabid = searchModules(0).TabID
                        End If
                    End If
                End If
                Return Me._Tabid
            End Get
            Set(ByVal value As Integer)
                Me._Tabid = value
            End Set
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property TabInfo() As TabInfo
            Get
                If Me._TabInfo Is Nothing AndAlso Me._Tabid <> -1 Then
                    Me._TabInfo = NukeHelper.TabController.GetTab(Me.TabId, Me.PortalId, False)
                End If
                Return Me._TabInfo
            End Get
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property ModuleInfo() As ModuleInfo
            Get
                If Me.IsModule AndAlso Me._ModuleInfo Is Nothing Then
                    Me._ModuleInfo = NukeHelper.ModuleController.GetModule(Me.ModuleId, Me.TabId, False)
                End If
                Return Me._ModuleInfo
            End Get
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property ModuleTitle() As String
            Get
                If Me.IsModule Then
                    Return Me.ModuleInfo.ModuleTitle
                End If
                Return String.Empty
            End Get
        End Property

        <XmlIgnore()> _
        Private BackingUniqueId As String = Nothing
        <XmlIgnore()> _
        Public ReadOnly Property UniqueId() As String
            Get
                If (BackingUniqueId Is Nothing) Then
                    BackingUniqueId = "U" & ModuleId & "-" & BusinessController.LuceneEscape(SearchKey) & "-" & BusinessController.LuceneEscape(GUID)
                End If
                Return BackingUniqueId
            End Get
        End Property

        Public Property Category() As String

        Public Property ForcedUrl() As String

        Public Property InheritModuleViewPermissions() As Boolean = True

        Public Property ItemViewRoleIds() As New List(Of Integer)

        Public Property ItemUserRoleIds() As new List(Of Integer)
            



        Public Property CategoryId() As Integer = -1

        <XmlIgnore()> _
        Public Property Language() As String

        <XmlIgnore()> _
        Public Property SiteMapFrequency() As ChangeFrequency
            Get
                Return Me._SiteMapFrequency
            End Get
            Set(ByVal value As ChangeFrequency)
                Me._SiteMapFrequency = value
                Me._SiteMapFrequencyIsSet = True
            End Set
        End Property

        <XmlIgnore()> _
        Public Property SiteMapPriority() As Decimal
            Get
                Return Me._SiteMapPriority
            End Get
            Set(ByVal value As Decimal)
                Me._SiteMapPriority = value
                Me._SiteMapPriorityIsSet = True
            End Set
        End Property

        <XmlIgnore()> _
        Public Property IndexingStatus() As IndexingItemStatus = Business.IndexingItemStatus.Ok

        Public Property TabTaxonomy As List(Of String) = New List(Of String)
        Public Property ModuleTaxonomy As List(Of String) = New List(Of String)
        Public Property ContentTaxonomy As List(Of String) = New List(Of String)

        Public Property IconPath As String
            Get
                Return _IconPath
            End Get
            Set(ByVal value As String)
                _IconPath = value
                _IconPathSetTriggered = True
            End Set
        End Property

        Public Property ImagePath As String
            Get
                Return _ImagePath
            End Get
            Set(ByVal value As String)
                _ImagePath = value
            End Set
        End Property

        Public Property ItemType As String
            Get
                Return Me._ItemType
            End Get
            Set(ByVal value As String)
                Me._ItemType = value
            End Set
        End Property

        Public Property ItemGroup As String
            Get
                Return Me._ItemGroup
            End Get
            Set(ByVal value As String)
                Me._ItemGroup = value
            End Set
        End Property

        Public Property FriendlyModuleName As String = String.Empty

        Public Property CanBeReturnedBySearch As Boolean = True
        ' Hack pour grer des fentres de publication gnriques
        Public Property ReturnedBySearchWindowStart As DateTime = New Date(1, 1, 1)
        Public Property ReturnedBySearchWindowEnd As DateTime = New Date(9999, 12, 31)

#Region "Public Methods"

        Public Overridable Function ToDocument() As Document
            Dim d As New Document

            If Me.GUID Is Nothing Then
                Me.GUID = ""
            End If

            If Me.Description Is Nothing Then
                Me.Description = ""
            End If

            If Me.Language = "" AndAlso _
                PortalIndexingDriver.GetByPortalId(Me.PortalId).LucenePortalSettings.SetPortalLanguageAsDefault Then
                Me.Language = NukeHelper.PortalInfo(Me.PortalId).DefaultLanguage
            End If

            Dim factory As FieldFactory = FieldFactory.Instance

            'basic id fields

            factory.AddFieldToDocument(d, LuceneGeneralField.ProviderName, Me.ProviderName)
            If (Not String.IsNullOrEmpty(UpgradeProviderName)) Then
                factory.AddFieldToDocument(d, LuceneGeneralField.UpgradeProviderName, Me.UpgradeProviderName)
            End If

            If Me.ItemId <> -1 Then
                factory.AddFieldToDocument(d, LuceneGeneralField.ItemId, Me.ItemId.ToString(CultureInfo.InvariantCulture))
            End If
            factory.AddFieldToDocument(d, LuceneGeneralField.Uid, Me.UniqueId)
            factory.AddFieldToDocument(d, DnnSearchItemField.PortalId, Me.PortalId.ToString(CultureInfo.InvariantCulture))
            factory.AddFieldToDocument(d, DnnSearchItemField.TabId, Me.TabId.ToString(CultureInfo.InvariantCulture))
            factory.AddFieldToDocument(d, DnnSearchItemField.ModuleId, Me.ModuleId.ToString(CultureInfo.InvariantCulture))
            factory.AddFieldToDocument(d, DnnSearchItemField.ModuleName, Me.ModuleName)
            factory.AddFieldToDocument(d, DnnSearchItemField.Guid, Me.GUID)


            'author field
            factory.AddFieldToDocument(d, DnnSearchItemField.Author, Me.Author.ToString)
            If Me.Author <> -1 Then
                Dim author As UserInfo = UserController.GetUser(PortalId, Me.Author, True)
                If author IsNot Nothing AndAlso author.DisplayName <> "" Then
                    factory.AddFieldToDocument(d, LuceneGeneralField.AuthorName, author.DisplayName)
                End If
            End If

            'content fields
            If Not String.IsNullOrEmpty(Me.SearchKey) Then
                factory.AddFieldToDocument(d, DnnSearchItemField.SearchKey, Me.SearchKey)
            End If
            If Not String.IsNullOrEmpty(Me.Title) Then
                factory.AddFieldToDocument(d, DnnSearchItemField.Title, Me.Title)
            End If
            If Not String.IsNullOrEmpty(Me.Description) Then
                factory.AddFieldToDocument(d, DnnSearchItemField.Description, Me.Description)
            End If
            If Not String.IsNullOrEmpty(Me.Content) Then
                factory.AddFieldToDocument(d, DnnSearchItemField.Content, Me.Content)
            End If

            If Me.Category <> "" Then
                factory.AddFieldToDocument(d, LuceneGeneralField.Category, Me.Category)
            End If
            If Me.CategoryId <> -1 Then
                factory.AddFieldToDocument(d, LuceneGeneralField.CategoryId, Me.CategoryId.ToString(CultureInfo.InvariantCulture))
            End If
            If Me.ForcedUrl <> "" Then
                factory.AddFieldToDocument(d, LuceneGeneralField.ForcedUrl, Me.ForcedUrl)
            End If
            If Not String.IsNullOrEmpty(Me.Language) Then
                factory.AddFieldToDocument(d, LuceneGeneralField.Language, Me.Language)
            End If

            ' gestion de la publication
            If Me.PubDate <> Date.MinValue Then
                factory.AddFieldToDocument(d, DnnSearchItemField.PubDate, DateTools.DateToString(Me.PubDate, DateTools.Resolution.SECOND))
            End If

            ' gestion de la visibilit
            factory.AddFieldToDocument(d, LuceneGeneralField.CanBeReturnedBySearch, CanBeReturnedBySearch.ToString())
            factory.AddFieldToDocument(d, LuceneGeneralField.ReturnedBySearchWindowStart, DateTools.DateToString(ReturnedBySearchWindowStart, DateTools.Resolution.SECOND))
            factory.AddFieldToDocument(d, LuceneGeneralField.ReturnedBySearchWindowEnd, DateTools.DateToString(ReturnedBySearchWindowEnd, DateTools.Resolution.SECOND))

            'additional dnn related fields
            factory.AddFieldToDocument(d, DnnSearchItemField.ImageFileId, Me.ImageFileId.ToString(CultureInfo.InvariantCulture))
            If Me.TabId <> -1 Then
                factory.AddFieldToDocument(d, DnnSearchItemField.TabName, Me.TabInfo.TabName)
                factory.AddFieldToDocument(d, DnnSearchItemField.TabTitle, Me.TabInfo.Title)
                factory.AddFieldToDocument(d, DnnSearchItemField.TabDescription, Me.TabInfo.Description)
                factory.AddFieldToDocument(d, DnnSearchItemField.ParentTabId, Me.TabInfo.ParentId.ToString(CultureInfo.InvariantCulture))
                If Me.TabInfo.ParentId <> -1 Then
                    Dim parentTab As TabInfo = NukeHelper.TabController.GetTab(Me.TabInfo.ParentId, Me.PortalId, False)
                    factory.AddFieldToDocument(d, DnnSearchItemField.ParentTabName, parentTab.TabName)
                    factory.AddFieldToDocument(d, DnnSearchItemField.ParentTabTitle, parentTab.Title)
                End If

                For Each objPermission As TabPermissionInfo In Me.TabInfo.TabPermissions
                    If objPermission.AllowAccess AndAlso objPermission.PermissionKey = "VIEW" Then
                        If objPermission.UserID <> -1 Then
                            factory.AddFieldToDocument(d, DnnSearchItemField.TabViewUserIds, objPermission.UserID.ToString(CultureInfo.InvariantCulture))
                        Else
                            factory.AddFieldToDocument(d, DnnSearchItemField.TabViewRoleIds, objPermission.RoleID.ToString(CultureInfo.InvariantCulture))
                        End If
                    End If
                Next


            End If

            If Me.IsModule Then
                factory.AddFieldToDocument(d, DnnSearchItemField.ModuleTitle, Me.ModuleInfo.ModuleTitle)

                factory.AddFieldToDocument(d, DnnSearchItemField.InheritTabViewPermissions, Me.ModuleInfo.InheritViewPermissions.ToString(CultureInfo.InvariantCulture).ToLower())

                If Not Me.ModuleInfo.InheritViewPermissions Then ' si on hrite pas des permissions de la page, on rajoute les informations d'accs pour le module ici

                    For Each objPermission As ModulePermissionInfo In Me.ModuleInfo.ModulePermissions
                        If objPermission.AllowAccess AndAlso objPermission.PermissionKey = "VIEW" Then
                            If objPermission.UserID <> -1 Then
                                factory.AddFieldToDocument(d, DnnSearchItemField.ModuleViewUserIds, objPermission.UserID.ToString(CultureInfo.InvariantCulture))
                            Else
                                factory.AddFieldToDocument(d, DnnSearchItemField.ModuleViewRoleIds, objPermission.RoleID.ToString(CultureInfo.InvariantCulture))
                            End If
                        End If
                    Next
                End If
            Else
                factory.AddFieldToDocument(d, DnnSearchItemField.InheritTabViewPermissions, (True).ToString(CultureInfo.InvariantCulture).ToLower())
            End If

            factory.AddFieldToDocument(d, DnnSearchItemField.InheritModuleViewPermissions, Me.InheritModuleViewPermissions.ToString(CultureInfo.InvariantCulture).ToLower())

            If Not Me.InheritModuleViewPermissions Then
                For Each roleId As Integer In Me.ItemViewRoleIds
                    factory.AddFieldToDocument(d, DnnSearchItemField.ItemViewRoleIds, roleId.ToString(CultureInfo.InvariantCulture))
                Next
                For Each userId As Integer In Me.ItemUserRoleIds
                    factory.AddFieldToDocument(d, DnnSearchItemField.ItemViewUserIds, userId.ToString(CultureInfo.InvariantCulture))
                Next
            End If


            'additional fields (module specific)
            For Each objField As Field In Me._AdditionalFields.GetFields
                Dim field = factory.GetDefinition(objField.Name())
                factory.AddFieldToDocument(d, field.Id, objField.StringValue())
            Next

            Dim objLucenePortalSets As LucenePortalSettings = Aricie.DNN.Settings.SettingsController.GetModuleSettings(Of LucenePortalSettings)(SettingsScope.PortalSettings, Me.PortalId)

            If Me._SiteMapPriorityIsSet OrElse Not objLucenePortalSets.ComputePriority Then
                factory.AddFieldToDocument(d, LuceneGeneralField.SiteMapPriority, Me.SiteMapPriority.ToString(CultureInfo.InvariantCulture))
            End If

            If Me._SiteMapFrequencyIsSet OrElse Not objLucenePortalSets.ComputeChangeFrequency Then
                factory.AddFieldToDocument(d, LuceneGeneralField.SiteMapFrequency, Me.SiteMapFrequency.ToString())
            End If

            factory.AddFieldToDocument(d, LuceneGeneralField.HashCode, d.ToString.GetHashCode().ToString(CultureInfo.InvariantCulture))

            ' IconPath rajout globalement 01/12/11
            factory.AddFieldToDocument(d, LuceneGeneralField.IconPath, IconPath)
            ' ImagePath rajout globalement 26/04/13
            factory.AddFieldToDocument(d, LuceneGeneralField.ImagePath, ImagePath)
            ' tags
            For Each tag In TabTaxonomy
                tag = tag.ToLowerInvariant().Trim()
                factory.AddFieldToDocument(d, DnnSearchItemField.TabTaxonomy, tag)
                factory.AddFieldToDocument(d, DnnSearchItemField.CompoundedTaxonomy, tag)
            Next
            For Each tag In ModuleTaxonomy
                tag = tag.ToLowerInvariant().Trim()
                factory.AddFieldToDocument(d, DnnSearchItemField.ModuleTaxonomy, tag)
                factory.AddFieldToDocument(d, DnnSearchItemField.CompoundedTaxonomy, tag)
            Next
            For Each tag In ContentTaxonomy
                tag = tag.ToLowerInvariant().Trim()
                factory.AddFieldToDocument(d, DnnSearchItemField.ContentTaxonomy, tag)
                factory.AddFieldToDocument(d, DnnSearchItemField.CompoundedTaxonomy, tag)
            Next

            ' par dfaut
            factory.AddFieldToDocument(d, LuceneGeneralField.ItemType, ItemType)
            factory.AddFieldToDocument(d, LuceneGeneralField.ItemGroup, ItemGroup)

            Return d

        End Function

        Public Sub ParseDocument(ByVal doc As Document)

            For Each objField As Field In doc.GetFields

                Select Case objField.Name
                    Case LuceneGeneralField.ProviderName
                        Me._ProviderName = objField.StringValue
                    Case LuceneGeneralField.UpgradeProviderName
                        Me.UpgradeProviderName = objField.StringValue
                    Case DnnSearchItemField.ModuleId
                        ModuleId = Integer.Parse(objField.StringValue)
                    Case DnnSearchItemField.Guid
                        GUID = objField.StringValue
                    Case DnnSearchItemField.SearchKey
                        SearchKey = objField.StringValue
                    Case DnnSearchItemField.Title
                        Title = objField.StringValue
                    Case DnnSearchItemField.PubDate
                        PubDate = DateTools.StringToDate(objField.StringValue)
                    Case LuceneGeneralField.CanBeReturnedBySearch
                        CanBeReturnedBySearch = Boolean.Parse(objField.StringValue)
                    Case LuceneGeneralField.ReturnedBySearchWindowStart
                        ReturnedBySearchWindowStart = DateTools.StringToDate(objField.StringValue)
                    Case LuceneGeneralField.ReturnedBySearchWindowEnd
                        ReturnedBySearchWindowEnd = DateTools.StringToDate(objField.StringValue)
                    Case DnnSearchItemField.Author
                        Author = Integer.Parse(objField.StringValue)
                    Case DnnSearchItemField.Description
                        Description = objField.StringValue
                    Case DnnSearchItemField.Content
                        Content = objField.StringValue
                    Case DnnSearchItemField.ImageFileId
                        ImageFileId = Integer.Parse(objField.StringValue)
                    Case LuceneGeneralField.Language
                        Language = objField.StringValue
                    Case LuceneGeneralField.ForcedUrl
                        ForcedUrl = objField.StringValue
                    Case DnnSearchItemField.ItemViewRoleIds
                        Me.ItemViewRoleIds.AddRange(ParseIds(objField.StringValue))
                    Case DnnSearchItemField.ItemViewUserIds
                        Me.ItemUserRoleIds.AddRange(ParseIds(objField.StringValue))
                    Case DnnSearchItemField.InheritModuleViewPermissions
                        Me.InheritModuleViewPermissions = Boolean.Parse(objField.StringValue)
                    Case LuceneGeneralField.IconPath ' Ajout du 01/12/11
                        IconPath = objField.StringValue
                    Case LuceneGeneralField.ImagePath ' Ajout du 26/04/13
                        ImagePath = objField.StringValue
                    Case DnnSearchItemField.ModuleName
                        Me._ModuleName = objField.StringValue
                    Case DnnSearchItemField.ContentTaxonomy
                        ContentTaxonomy.Add(objField.StringValue)
                    Case DnnSearchItemField.ModuleTaxonomy
                        ModuleTaxonomy.Add(objField.StringValue)
                    Case DnnSearchItemField.TabTaxonomy
                        TabTaxonomy.Add(objField.StringValue)
                    Case LuceneGeneralField.ItemType
                        ItemType = objField.StringValue
                    Case LuceneGeneralField.ItemGroup
                        ItemGroup = objField.StringValue
                    Case Else
                        Me.AdditionalFields.Add(objField)
                End Select

            Next

        End Sub

#End Region

        Public Function GetUid() As String
            Return UniqueId
            'Return
        End Function

        Public Shared Function ParseIds(ByVal field As String) As List(Of Integer)
            Dim toReturn As New List(Of Integer)
            Dim strIds As String() = field.Split(" "c)
            For Each strId As String In strIds
                toReturn.Add(Integer.Parse(strId, CultureInfo.InvariantCulture))
            Next
            Return toReturn
        End Function

        Private Sub RetrieveInfoFromModuleId()
            If IsModule AndAlso String.IsNullOrEmpty(_ModuleName) Then
                _ModuleName = GetModuleNameByModuleId(ModuleId)
                FriendlyModuleName = GetModuleFriendlyNameByModuleId(ModuleId)
                ' icon path
                If Not _IconPathSetTriggered Then
                    _IconPath = String.Format("{0}.png", ModuleName)
                End If

                ItemType = FriendlyModuleName
                ItemGroup = FriendlyModuleName

            End If

            Dim TaxonomyEnabledDotNetNukeVersion As New Version(5, 3, 0)

            If DnnVersion >= TaxonomyEnabledDotNetNukeVersion AndAlso Not TaxonomyInitialized Then

                GetTagsFromContentItemEntity(TabInfo, TabTaxonomy)
                GetTagsFromContentItemEntity(ModuleInfo, ModuleTaxonomy)
                'We don't pick up content taxonomy automatically, so far no content seems to use the taxonomy, and it doesn't look like something will

                TaxonomyInitialized = True
            End If
        End Sub

        Private Sub GetTagsFromContentItemEntity(ByVal entity As Object, ByVal Output As List(Of String))
            Dim typeContentItem = ReflectionHelper.CreateType("DotNetNuke.Entities.Content.ContentItem") ' cache gr par DNN
            Dim typeTerm = ReflectionHelper.CreateType("DotNetNuke.Entities.Content.Taxonomy.Term") ' cache gr par DNN

            If (entity IsNot Nothing) Then
                Dim Terms As IEnumerable = DirectCast(ReflectionHelper.GetProperty(typeContentItem, "Terms", entity), IEnumerable)
                If (Terms IsNot Nothing) Then
                    For Each t In Terms
                        Output.Add(DirectCast(ReflectionHelper.GetProperty(typeTerm, "Name", t), String))
                    Next
                End If
            End If
        End Sub

    End Class

End Namespace


