Imports System.ComponentModel
Imports System.Globalization
Imports Lucene.Net.Documents
Imports DotNetNuke.Services.Search
Imports Lucene.Net.Search
Imports System.Xml.Serialization
Imports DotNetNuke.Entities.Modules
Imports DotNetNuke.Entities.Tabs
Imports Aricie.DNN.Services
Imports Aricie.Services
Imports Aricie.DNN.Modules.LuceneSearch.SiteMap
Imports Aricie.DNN.Modules.LuceneSearch.Settings
Imports Aricie.DNN.Settings
Imports Lucene.Net.Index
Imports Aricie.DNN.Modules.LuceneSearch.ModuleProviders
Imports LSCommon = Aricie.DNN.Modules.LuceneSearch.Common
Imports System.Linq
Imports System.Reflection
Imports Lucene.Net.Search.Similar

Namespace Aricie.DNN.Modules.LuceneSearch.Business
    Public Class LuceneSearchResultsInfo
        Inherits SearchResultsInfo
        Implements ICustomTypeDescriptor



#Region "Private members"

        Private _CurrentQueryingDriver As PortalQueryingDriver
        Private _AdditionalFields As New Document
        Private _SearchModuleId As Integer
        Private _Document As Document
        Private _ScoreDoc As ScoreDoc
        Private _Query As Query
        Private _Highlights As String
        Private _GoToUrl As String

        Private _ItemType As String
        Private _ItemGroup As String

#End Region

#Region "cTors"

        Public Sub New()
            MyBase.New()
            ModuleId = Aricie.DNN.Modules.LuceneSearch.Common.Constants.SearchAndResultItemInfos.ModuleIdCannotBeDetermined
        End Sub

        Public Sub New(ByVal searchModuleId As Integer)
            MyBase.New()
            Me._SearchModuleId = searchModuleId
        End Sub

        Public Sub New(ByVal searchModuleId As Integer, ByVal searchItem As SearchResultsInfo)
            Me.New(searchModuleId)
            Me.Title = searchItem.Title
            Me.Description = searchItem.Description
            Me.Author = searchItem.Author
            Me.PubDate = searchItem.PubDate
            Me.ModuleId = searchItem.ModuleId
            Me.SearchKey = searchItem.SearchKey
            Me.Delete = searchItem.Delete
            Me.Guid = searchItem.Guid
            Me.Image = searchItem.Image
            Me.Occurrences = searchItem.Occurrences
            Me.AuthorName = searchItem.AuthorName
            Me.PortalId = searchItem.PortalId
            Me.Relevance = searchItem.Relevance
            Me.SearchItemID = searchItem.SearchItemID
            Me.TabId = searchItem.TabId

            If TypeOf searchItem Is LuceneSearchResultsInfo Then
                Me.AdditionalFields = CType(searchItem, LuceneSearchResultsInfo).AdditionalFields
            End If
        End Sub

        Public Sub New(ByVal searchModuleId As Integer, ByVal query As Query, ByVal objDoc As Document, ByVal objScoreDoc As ScoreDoc)
            Me.New(searchModuleId)
            Me._Query = query
            Me._Document = objDoc
            Me._ScoreDoc = objScoreDoc
            Me.ParseData()

        End Sub

#End Region

#Region "Public properties"

        Public Property AdditionalFields() As Document
            Get
                Return _AdditionalFields
            End Get
            Set(ByVal value As Document)
                _AdditionalFields = value
            End Set
        End Property

        Public ReadOnly Property ForcedUrl() As String
            Get
                Return Me.Field(LuceneGeneralField.ForcedUrl)
            End Get
        End Property

        Public ReadOnly Property Uid() As String
            Get
                Return Me.Field(LuceneGeneralField.Uid)
            End Get
        End Property

        Public ReadOnly Property Query() As Query
            Get
                Return Me._Query
            End Get
        End Property

        <XmlIgnore()> _
        Public Overridable ReadOnly Property GoToUrl() As String
            Get
                If _GoToUrl = "" Then
                    If ForcedUrl <> "" Then
                        _GoToUrl = ForcedUrl
                    Else
                        _GoToUrl = Me.FormatURL(Me.TabId, Me.Guid)
                    End If
                End If
                Return _GoToUrl
            End Get
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property GoToFullUrl() As String
            Get
                Dim myUri As New Uri(GoToUrl, UriKind.RelativeOrAbsolute)

                If myUri.IsAbsoluteUri Then
                    Return myUri.AbsoluteUri
                Else
                    Return New Uri(HttpContext.Current.Request.Url, GoToUrl).AbsoluteUri
                End If

            End Get
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property CurrentQueryingDriver() As PortalQueryingDriver
            Get
                If _CurrentQueryingDriver Is Nothing Then
                    Me._CurrentQueryingDriver = PortalQueryingDriver.GetByPortalId(Me.PortalId)
                End If
                Return _CurrentQueryingDriver
            End Get
        End Property

        Private _ResultIndex As Integer
        Public Property ResultIndex() As Integer
            Get
                Return _ResultIndex
            End Get
            Set(ByVal value As Integer)
                _ResultIndex = value
            End Set
        End Property

#Region "Hit properties"

        <XmlIgnore()> _
        Public ReadOnly Property Score() As Single
            Get
                If Me._ScoreDoc IsNot Nothing Then
                    Return Me._ScoreDoc.Score
                End If
                Return 0
            End Get
        End Property

        Public ReadOnly Property ScoreDoc() As ScoreDoc
            Get
                Return Me._ScoreDoc
            End Get
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property Boost() As Single
            Get
                If Me.Document IsNot Nothing Then
                    Return Me.Document.Boost
                End If
                Return 1
            End Get
        End Property

        Public ReadOnly Property Document() As Document
            Get
                If Me._Document IsNot Nothing Then
                    Return Me._Document
                End If
                Return Me._AdditionalFields
            End Get
        End Property

        Public ReadOnly Property Highlights() As String
            Get
                If String.IsNullOrEmpty(Me._Highlights) Then
                    If Me.CurrentQueryingDriver.LuceneResultsSettings(Me._SearchModuleId).EnableHighlighter Then
                        Dim fragments As Dictionary(Of String, String) = _
                                Me.CurrentQueryingDriver.CreateHighlightsNode(Me.Document, Me.Query, Me._SearchModuleId)
                        Dim highlightLayout As New StringBuilder
                        Dim separator As String = Me.CurrentQueryingDriver.LuceneResultsSettings(Me._SearchModuleId). _
                                HighlighterFragmentsSeparator
                        Dim fragmentFormat As String = Me.CurrentQueryingDriver.LuceneResultsSettings(Me._SearchModuleId). _
                                HighlighterFragmentFormat

                        For Each fieldName As String In fragments.Keys
                            highlightLayout.Append(fragmentFormat.Replace("[FieldName]", FieldFactory.Instance.GetDefinition(fieldName).FriendlyName).Replace( _
                                                    "[Fragment]", fragments(fieldName)))
                            highlightLayout.Append(separator)
                        Next
                        Me._Highlights = highlightLayout.ToString
                        If Me._Highlights.EndsWith(separator) Then
                            Me._Highlights = Me._Highlights.Substring(0, Me._Highlights.Length - separator.Length)
                        End If
                    End If
                End If
                Return Me._Highlights
            End Get
        End Property

#End Region

#Region "Field properties"

        <XmlIgnore()> _
        Public ReadOnly Property Field(ByVal fieldName As String) As String
            Get
                Dim toReturn As String = Me.Document.Get(fieldName)
                If toReturn Is Nothing Then
                    Return ""
                End If
                Return toReturn
            End Get
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property LocaleCode() As String
            Get
                Return Me.Field(LuceneGeneralField.Language)
            End Get
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property ProviderName() As String
            Get
                Return Me.Field(LuceneGeneralField.ProviderName)
            End Get
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property ModuleName() As String
            Get
                Return Me.Field(DnnSearchItemField.ModuleName)
            End Get
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property SiteMapFrequency() As ChangeFrequency
            Get
                Dim toReturn As ChangeFrequency
                Dim strField As String = Me.Field(LuceneGeneralField.SiteMapFrequency)
                If Not String.IsNullOrEmpty(strField) Then
                    toReturn = DirectCast(System.Enum.Parse(GetType(ChangeFrequency), strField), ChangeFrequency)
                Else
                    toReturn = Me.ComputeFrequency()
                End If
                Return toReturn
            End Get
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property SiteMapPriority() As Decimal
            Get
                Dim toReturn As Decimal
                Dim strField As String = Me.Field(LuceneGeneralField.SiteMapPriority)
                If Not String.IsNullOrEmpty(strField) Then
                    toReturn = Decimal.Parse(strField, CultureInfo.InvariantCulture)
                Else
                    Dim LucenePortalSets As LucenePortalSettings = Aricie.DNN.Settings.SettingsController.GetModuleSettings(Of LucenePortalSettings)(SettingsScope.PortalSettings, PortalId)
                    If LucenePortalSets.ComputePriority Then
                        toReturn = Me.ComputePriority()
                    Else
                        toReturn = LucenePortalSets.DefaultPriority
                    End If
                End If
                Return toReturn
            End Get
        End Property

        Public Property IconPath As String
        Public Property ImagePath As String

        Public Property Category As String
        Public Property ContentTaxonomy As List(Of String) = New List(Of String)
        Public Property ModuleTaxonomy As List(Of String) = New List(Of String)
        Public Property TabTaxonomy As List(Of String) = New List(Of String)
        Public Property CompoundedTaxonomy As List(Of String) = New List(Of String)

        Public Property ItemType As String
            Get
                Return _ItemType
            End Get
            Private Set(ByVal value As String)
                _ItemType = GetLocalizedValueInstance(LuceneGeneralField.ItemType, value)
                RawItemType = value
            End Set
        End Property

        Public Property ItemGroup As String
            Get
                Return _ItemGroup
            End Get
            Private Set(ByVal value As String)
                _ItemGroup = GetLocalizedValueInstance(LuceneGeneralField.ItemGroup, value)
                RawItemGroup = value
            End Set
        End Property

        Public Property RawItemType As String
        Public Property RawItemGroup As String
#End Region

#Region "Comfort properties"

        <XmlIgnore()> _
        Public ReadOnly Property PubHistory() As List(Of Date)
            Get
                Dim toreturn As List(Of Date)
                Dim strPubHistory As String = Me.Field(LuceneGeneralField.PubHistory)
                If Not String.IsNullOrEmpty(strPubHistory) Then
                    Dim stringDates As New List(Of String)(strPubHistory.Split(","c))
                    toreturn = stringDates.ConvertAll(New Converter(Of String, Date)(AddressOf DateTools.StringToDate))
                Else
                    toreturn = New List(Of Date)
                End If
                toreturn.Add(Me.PubDate)
                Return toreturn
            End Get
        End Property


        <XmlIgnore()> _
        Public ReadOnly Property FormattedDate() As String
            Get
                Return Me.PubDate.ToString
            End Get
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property IndexedDate() As Date
            Get
                Dim dateToReturn As Date = DateTools.StringToDate(Me.Field(LuceneGeneralField.IndexedDate))
                dateToReturn = CDate(dateToReturn.ToShortDateString)
                Return dateToReturn
            End Get
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property Culture() As CultureInfo
            Get
                Dim localeCode As String = Me.LocaleCode
                If localeCode <> "" Then
                    Return New CultureInfo(localeCode)
                End If
                Return CultureInfo.InvariantCulture
            End Get
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property ModuleInfo() As ModuleInfo
            Get
                Return NukeHelper.ModuleController.GetModule(Me.ModuleId, Me.TabId, False)
            End Get
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property FriendlyModuleName() As String
            Get
                Return GetModuleFriendlyNameByModuleId(Me.ModuleId)
            End Get
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property ModuleTitle() As String
            Get
                If ModuleInfo IsNot Nothing Then
                    Return ModuleInfo.ModuleTitle
                End If
                Return String.Empty
            End Get
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property ModuleViewRoles() As String
            Get
                ''Return ""
                ''par reflection:
                'Dim viewRolesProp As System.Reflection.PropertyInfo = Nothing
                'Dim strViewRolesProp As String = "AuthorizedViewRoles"
                'If Aricie.DNN.Services.NukeHelper.DnnVersion.Major < 5 Then
                '    strViewRolesProp = "AuthorizedRoles"
                'End If
                'If Aricie.Services.ReflectionHelper.GetPropertiesDictionary(Of ModuleInfo).TryGetValue(strViewRolesProp, viewRolesProp) Then
                '    Return DirectCast(viewRolesProp.GetValue(Me.ModuleInfo, Nothing), String)
                'End If
                Return ""
                'Return Me.ModuleInfo.AuthorizedViewRoles
                'objModulePermissionController.GetModulePermissions(objModuleInfo.ModulePermissions, "VIEW")


                'Return Me.ModuleInfo.AuthorizedRoles
            End Get
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property TabInfo() As TabInfo
            Get
                Return LuceneContext.Instance.CurrentTabs(Me.TabId)
            End Get
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property TabName() As String
            Get
                Return Me.TabInfo.TabName
            End Get
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property TabTitle() As String
            Get
                Return Me.TabInfo.Title
            End Get
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property TabDescription() As String
            Get
                Return Me.TabInfo.Description
            End Get
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property TabViewRoles() As String
            Get
                Return Me.TabInfo.AuthorizedRoles
            End Get
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property ParentTab() As TabInfo
            Get
                If Me.TabInfo.ParentId <> -1 Then
                    Return LuceneContext.Instance.CurrentTabs(Me.TabInfo.ParentId)
                End If
                Return New TabInfo
            End Get
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property ParentTabId() As Integer
            Get
                Return Me.ParentTab.TabID
            End Get
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property ParentTabName() As String
            Get
                Return Me.ParentTab.TabName
            End Get
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property ParentTabTitle() As String
            Get
                Return Me.ParentTab.Title
            End Get
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property ParentTabDescription() As String
            Get
                Return Me.ParentTab.Description
            End Get
        End Property

        <XmlIgnore()> _
        Public ReadOnly Property CompoundedTaxonomyList() As String
            Get
                Return String.Join(", ", CompoundedTaxonomy.ToArray())
            End Get
        End Property

#End Region

#End Region

#Region "Public Methods"

        Public Function HasField(ByVal name As String) As Boolean
            Return Me.Document.GetField(name) IsNot Nothing
        End Function

        ''' <summary>
        ''' Get similar Results Query
        ''' </summary>
        ''' <returns>Return a Query</returns>
        ''' <remarks>Unused at this day - 15/03/2011 </remarks>
        Public Function GetSimilarResultsQuery() As Query
            Return Me.CurrentQueryingDriver.GetSimilarResultsQuery(Me.ModuleId, Me.Document)

        End Function

        Public Function GetMoreLikeThis() As Query
            Dim reader As IndexReader
            If Me.CurrentQueryingDriver.LucenePortalSettings.UseSharedReader Then
                reader = Me.CurrentQueryingDriver.SharedIndexReader
            Else
                reader = Me.CurrentQueryingDriver.GetIndexReader(True)
            End If
            Dim more As New MoreLikeThis(reader)
            Dim toReturn As Query = more.Like(Me.ScoreDoc.Doc)
            If Not Me.CurrentQueryingDriver.LucenePortalSettings.UseSharedReader Then
                reader.Dispose()
            End If

            Return toReturn
        End Function

        Private Sub ParseData()


            For Each objField As Field In Me.Document.GetFields

                Dim fieldStringValue As String = objField.StringValue

                Select Case objField.Name
                    Case DnnSearchItemField.ModuleId
                        Me.ModuleId = Integer.Parse(fieldStringValue)
                    Case DnnSearchItemField.TabId
                        Me.TabId = Integer.Parse(fieldStringValue)
                    Case DnnSearchItemField.Guid
                        Me.Guid = fieldStringValue
                    Case DnnSearchItemField.SearchKey
                        Me.SearchKey = fieldStringValue
                    Case DnnSearchItemField.Title
                        Me.Title = fieldStringValue
                    Case DnnSearchItemField.PubDate
                        Me.PubDate = DateTools.StringToDate(fieldStringValue)
                    Case DnnSearchItemField.Author
                        Me.Author = fieldStringValue
                    Case DnnSearchItemField.Description
                        Me.Description = fieldStringValue
                    Case DnnSearchItemField.ImageFileId
                        Me.Image = Integer.Parse(fieldStringValue)
                    Case LuceneGeneralField.AuthorName
                        Me.AuthorName = fieldStringValue
                    Case LuceneGeneralField.IconPath
                        Me.IconPath = fieldStringValue
                    Case LuceneGeneralField.ImagePath
                        Me.ImagePath = fieldStringValue
                    Case LuceneGeneralField.Category
                        Category = fieldStringValue
                    Case DnnSearchItemField.ContentTaxonomy
                        AddToListIfDoesntExistWithAnotherCasing(CompoundedTaxonomy, fieldStringValue)
                        ContentTaxonomy.Add(fieldStringValue)
                    Case DnnSearchItemField.ModuleTaxonomy
                        AddToListIfDoesntExistWithAnotherCasing(CompoundedTaxonomy, fieldStringValue)
                        ModuleTaxonomy.Add(fieldStringValue)
                    Case DnnSearchItemField.TabTaxonomy
                        AddToListIfDoesntExistWithAnotherCasing(CompoundedTaxonomy, fieldStringValue)
                        TabTaxonomy.Add(fieldStringValue)
                    Case LuceneGeneralField.ItemType
                        ItemType = fieldStringValue
                    Case LuceneGeneralField.ItemGroup
                        ItemGroup = fieldStringValue
                    Case Else
                        Me.AdditionalFields.Add(objField)
                End Select

            Next

            Me.Occurrences = Convert.ToInt32(Math.Round(Me.Boost))
            Me.Relevance = Convert.ToInt32(Math.Round(Me.Score() * 100))
            Me.SearchItemID = Me.ScoreDoc.Doc

            'if standalone
            If Me.ModuleId = Aricie.DNN.Modules.LuceneSearch.Common.Constants.SearchAndResultItemInfos.ModuleIdCannotBeDetermined Then
                SetStandaloneProperties()
            End If

        End Sub

        Private Sub AddToListIfDoesntExistWithAnotherCasing(ByVal l As List(Of String), ByVal Value As String)
            If Not (l.Any(Function(s) s.Equals(Value, StringComparison.OrdinalIgnoreCase))) Then
                l.Add(Value)
            End If
        End Sub

        Private Sub SetStandaloneProperties()

            Me.ModuleId = _SearchModuleId
            Dim resultProvider As LuceneProviderConfig = ProviderFactory.Instance.GetProviderByName(Me.ProviderName)
            If TypeOf resultProvider Is LuceneStandaloneProviderConfig Then
                Dim standAloneProvider As LuceneStandaloneProviderConfig = DirectCast(resultProvider, LuceneStandaloneProviderConfig)
                If standAloneProvider.LinkToDetails Then

                    Dim modules As ArrayList = NukeHelper.ModuleController.GetModuleTabs(Me._SearchModuleId)
                    If modules.Count > 0 Then
                        Me.TabId = CType(modules(0), ModuleInfo).TabID
                    End If

                    If Me.Guid IsNot Nothing Then
                        Dim encryptedGuid As String = DotNetNuke.Common.Utilities.UrlUtils.EncryptParameter(Me.Guid)
                        Me.Guid = String.Format("mid={0}&ctl=ResultDetails&guid={1}", Me.ModuleId, encryptedGuid)
                    End If

                End If
            End If

        End Sub

        Private Function FormatURL(ByVal tabID As Integer, ByVal link As String) As String

            Dim strURL As String
            If Me.CurrentQueryingDriver.LuceneResultsSettings(Me._SearchModuleId).DisplayBackButton Then
                link += "&" & LSCommon.Constants.UrlParameters.LuceneSearchModeParameter & "=detail"
            End If
            If link = "" Then
                strURL = DotNetNuke.Common.NavigateURL(tabID)
            Else
                strURL = DotNetNuke.Common.NavigateURL(tabID, "", link.Split("&"c))
            End If

            Return strURL

        End Function

#End Region

        Private Function ComputeFrequency() As ChangeFrequency
            Dim pubHistory As List(Of Date) = Me.PubHistory
            If pubHistory.Count < 1 Then
                Throw New ArgumentException("Publication history should contain at least last published date", "result")
            End If
            pubHistory.Add(Now)
            Dim totalSpan As TimeSpan = TimeSpan.Zero
            Dim currentSpan As TimeSpan
            For i As Integer = pubHistory.Count - 1 To 1 Step -1
                currentSpan = pubHistory(i).Subtract(pubHistory(i - 1))
                totalSpan = totalSpan + currentSpan
            Next
            Dim meanSpanTicks As Long = Convert.ToInt64(totalSpan.Ticks / (pubHistory.Count - 1))
            Dim meanSpan As TimeSpan = TimeSpan.FromTicks(meanSpanTicks)
            Dim LucenePortalSets As LucenePortalSettings = Aricie.DNN.Settings.SettingsController.GetModuleSettings(Of LucenePortalSettings)(SettingsScope.PortalSettings, PortalId)
            Return TimeSpanToChangeFrequency(meanSpan, LucenePortalSets.DefaultChangeFrequency)

        End Function

        Private Function ComputePriority() As Decimal
            Return 0.7D 'TODO: rparer ce code Decimal.Round(Convert.ToDecimal(Me.Score * 0.7), 1)
        End Function

        Private Function TimeSpanToChangeFrequency(ByVal span As TimeSpan, ByVal defaultFreq As ChangeFrequency) As ChangeFrequency
            Dim frequency As ChangeFrequency = defaultFreq
            If span > TimeSpan.FromDays(365) Then
                Return ChangeFrequency.Yearly
            End If
            If span > TimeSpan.FromDays(31) Then
                Return ChangeFrequency.Monthly
            End If
            If span > TimeSpan.FromDays(7) Then
                Return ChangeFrequency.Weekly
            End If
            If span > TimeSpan.FromDays(1) Then
                frequency = ChangeFrequency.Daily
            End If
            Return frequency
        End Function

        Public Class LuceneFieldPropertyDescriptor
            Inherits PropertyDescriptor

            Public Sub New(ByVal propertyName As String)
                MyBase.New(propertyName, Nothing)
            End Sub


            Public Overrides Function CanResetValue(ByVal component As Object) As Boolean
                Return False
            End Function

            Public Overrides ReadOnly Property ComponentType() As Type
                Get
                    Return GetType(LuceneSearchResultsInfo)
                End Get
            End Property

            Public Overrides Function GetValue(ByVal component As Object) As Object
                Dim result As LuceneSearchResultsInfo = DirectCast(component, LuceneSearchResultsInfo)
                Dim toReturn As Object

                If result.HasField(Me.Name) Then
                    toReturn = result.Field(Me.Name)
                Else
                    toReturn = ReflectionHelper.GetPropertiesDictionary(result.GetType())(Me.Name).GetValue(result, Nothing)
                End If

                If toReturn Is Nothing Then
                    Return String.Empty
                End If

                Return toReturn

            End Function

            Public Overrides ReadOnly Property IsReadOnly() As Boolean
                Get
                    Return True
                End Get
            End Property

            Public Overrides ReadOnly Property PropertyType() As Type
                Get
                    Return GetType(String)
                End Get
            End Property

            Public Overrides Sub ResetValue(ByVal component As Object)

            End Sub

            Public Overrides Sub SetValue(ByVal component As Object, ByVal value As Object)

            End Sub

            Public Overrides Function ShouldSerializeValue(ByVal component As Object) As Boolean
                Return False
            End Function
        End Class

#Region "ICustomTypeDescriptor implementation"

        Public Function GetAttributes() As AttributeCollection Implements ICustomTypeDescriptor.GetAttributes
            Return TypeDescriptor.GetAttributes(Me, True)
        End Function

        Public Function GetClassName() As String Implements ICustomTypeDescriptor.GetClassName
            Return TypeDescriptor.GetClassName(Me, True)
        End Function

        Public Function GetComponentName() As String Implements ICustomTypeDescriptor.GetComponentName
            Return TypeDescriptor.GetComponentName(Me, True)
        End Function

        Public Function GetConverter() As TypeConverter Implements ICustomTypeDescriptor.GetConverter
            Return TypeDescriptor.GetConverter(Me, True)
        End Function

        Public Function GetDefaultEvent() As EventDescriptor Implements ICustomTypeDescriptor.GetDefaultEvent
            Return TypeDescriptor.GetDefaultEvent(Me, True)
        End Function

        Public Function GetDefaultProperty() As PropertyDescriptor Implements ICustomTypeDescriptor.GetDefaultProperty
            Return TypeDescriptor.GetDefaultProperty(Me, True)
        End Function

        Public Function GetEditor(ByVal editorBaseType As Type) As Object Implements ICustomTypeDescriptor.GetEditor
            Return TypeDescriptor.GetEditor(Me, editorBaseType, True)
        End Function

        Public Function GetEvents() As EventDescriptorCollection Implements ICustomTypeDescriptor.GetEvents
            Return TypeDescriptor.GetEvents(Me, True)
        End Function

        Public Function GetEvents(ByVal attributes() As Attribute) As EventDescriptorCollection _
            Implements ICustomTypeDescriptor.GetEvents
            Return TypeDescriptor.GetEvents(Me, attributes, True)
        End Function

        Public Function GetProperties() As PropertyDescriptorCollection Implements ICustomTypeDescriptor.GetProperties
            Return Me.GetProperties(Nothing)
        End Function

        Public Function GetProperties(ByVal attributes() As Attribute) As PropertyDescriptorCollection _
            Implements ICustomTypeDescriptor.GetProperties

            Dim tempDescs As PropertyDescriptorCollection = TypeDescriptor.GetProperties(GetType(LuceneSearchResultsInfo), attributes, True)

            Dim toReturn As New List(Of PropertyDescriptor)()
            For Each desc As PropertyDescriptor In tempDescs
                toReturn.Add(desc)
            Next
            For Each objField As FieldDefinition In PortalQueryingDriver.GetByPortalId(Me.PortalId).GetIndexedFields.Where(Function(o) Not String.IsNullOrEmpty(o.Id))
                Dim fieldName As String = objField.Id
                If tempDescs.Find(fieldName, True) Is Nothing Then
                    toReturn.Add(New LuceneFieldPropertyDescriptor(objField.Id))
                End If
            Next

            Return New PropertyDescriptorCollection(toReturn.ToArray(), True)

        End Function

        Public Function GetPropertyOwner(ByVal pd As PropertyDescriptor) As Object _
            Implements ICustomTypeDescriptor.GetPropertyOwner

            If TypeOf pd Is LuceneFieldPropertyDescriptor Then
                Return Me
            End If
            Return Nothing

        End Function

#End Region

#Region "LocalizedValueInstance"
        ''' <summary>
        ''' Permet de grer la traduction au vol d'lments qui ne sont pas ncessairement  afficher tels quels
        ''' </summary>
        ''' <param name="FieldName">Le nom du champ tel qu'il est conserv dans l'index </param>
        ''' <param name="FieldValue">La valeur du champ</param>
        ''' <returns>La chaine transforme si Fieldname.Fieldvalue est trouv dans le SharedResources.resx, FieldValue sinon</returns>
        ''' <remarks>Si on a activ le ShowMissingKeys du site, on aura l'indication de ressource manquante RESX:FieldName.FieldValue retourne</remarks>
        Private Function GetLocalizedValueInstance(ByVal FieldName As String, ByVal FieldValue As String) As String
            Dim FinalValue As String = String.Empty
            Try
                If (FieldValue IsNot Nothing) Then
                    FinalValue = FieldValue

                    Dim FI As FieldDefinition = FieldFactory.Instance().GetDefinition(FieldName)
                    If FI.LocalizeValuedInstance Then
                        ' localis par instance, on cherche la combinaison dans le fichier de ressources
                        If (TypeOf FinalValue Is String) Then
                            Dim ResultValue As String = DotNetNuke.Services.Localization.Localization.GetString(FieldName & "." & FinalValue.ToString, _
                                    NukeHelper.GetModuleSharedResourceFile(Aricie.DNN.Modules.LuceneSearch.Common.Constants.ModuleNameLuceneSearchResults))
                            If (Not String.IsNullOrEmpty(ResultValue)) Then
                                ' si la cl de resource est vide, on garde la version originale, sinon on affiche la resource
                                FinalValue = ResultValue
                            End If
                        End If
                    End If
                End If

            Catch ex As Exception
                FinalValue = String.Format("Error while trying to access the ""{0}"" field", FieldName)
            End Try

            Return FinalValue

        End Function

#End Region

    End Class
End Namespace


