Imports DotNetNuke.Services.Localization
Imports Lucene.Net.Documents
Imports Aricie.DNN.Services
Imports Lucene.Net.Search
Imports Lucene.Net.Analysis

Namespace Aricie.DNN.Modules.LuceneSearch.Business
    Public Class FieldDefinition


        Public Sub New(ByVal id As String)
            Me._Id = id
        End Sub

        Public Sub New(ByVal id As String, ByVal store As Field.Store, ByVal index As Field.Index, _
                        ByVal termVector As Field.TermVector, ByVal localized As Boolean, Optional ByVal useSynonyms As Boolean = False)
            Me.New(id)
            Me._Index = index
            Me._Store = store
            Me._TermVector = termVector
            Me._IsLocalized = localized
            Me._UseSynonyms = useSynonyms
        End Sub


        Public Sub New(ByVal id As String, ByVal store As Field.Store, ByVal index As Field.Index, _
                        ByVal termVector As Field.TermVector, ByVal isLocalized As Boolean, ByVal boost As Single, Optional ByVal useSynonyms As Boolean = False)
            Me.New(id, store, index, termVector, isLocalized, useSynonyms)
            Me._Boost = boost
        End Sub

        Public Sub New(ByVal id As String, ByVal store As Field.Store, ByVal index As Field.Index, _
                ByVal termVector As Field.TermVector, ByVal isLocalized As Boolean, ByVal boost As Single, ByVal type As Integer, Optional ByVal useSynonyms As Boolean = False)
            Me.New(id, store, index, termVector, isLocalized, useSynonyms)
            Me._Boost = boost
            Me._Type = type
        End Sub


#Region "Public properties"

        Public Property Id As String
        Public Property Store As Field.Store = Field.Store.YES
        Public Property Index As Field.Index = Field.Index.ANALYZED
        Public Property TermVector As Field.TermVector = Field.TermVector.NO
        Public Property IsLocalized As Boolean = False
        Public Property UseSynonyms As Boolean = False
        Public Property Boost As Single = 1
        Public Property Type As Integer = -1
        Public Property LocalizeValuedInstance As Boolean = False
        Public Property Displayed As Boolean = True
        ''' <summary>
        ''' Le champ lucene de tri; si le champ courant est un champ analys, il ne pourra pas tre tri. Le BackingSortableField permet de forcer la conservation
        ''' des valeurs non tries pour les besoins des tris.
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Protected Friend Property BackingSortableField As FieldDefinition = Nothing
        ''' <summary>
        ''' Dtermine si un backing field doit tre cr ou non pour ce champ
        ''' </summary>
        ''' <remarks></remarks>
        Private _BackingFieldEnabled As Boolean = True

        Public ReadOnly Property FriendlyName() As String
            Get
                Dim name As String = Localization.GetString("LuceneFieldName." & Me.Id, NukeHelper.GetModuleSharedResourceFile(Common.Constants.ModuleNameLuceneSearchResults))
                Return IIf(String.IsNullOrEmpty(name), Id, name).ToString()
            End Get
        End Property

        ''' <summary>
        ''' Met en place le backing field pour le tri si ce dernier est ncessaire
        ''' </summary>
        ''' <remarks></remarks>
        Protected Friend Sub SetupBackingFields()
            If (_BackingFieldEnabled) Then
                If (Not Index = Field.Index.NO) Then
                    If (Index = Field.Index.ANALYZED OrElse Index = Field.Index.ANALYZED_NO_NORMS) Then
                        BackingSortableField = New FieldDefinition("SORT_" & Id, Field.Store.NO, Field.Index.NOT_ANALYZED, Field.TermVector.NO, False) With {.Displayed = False}
                    End If
                End If
            End If
        End Sub

        Protected Friend ReadOnly Property SortableFieldId As String
            Get
                If BackingSortableField IsNot Nothing Then
                    Return BackingSortableField.Id
                End If
                Return Id
            End Get
        End Property

        Private _DisplayAsSearchable As Boolean = True
        Public Property DisplayAsSearchable As Boolean
            Get
                Return Displayed AndAlso _DisplayAsSearchable
            End Get
            Set(value As Boolean)
                _DisplayAsSearchable = value
            End Set
        End Property

        Private _DisplayAsSortable As Boolean = True
        Public Property DisplayAsSortable As Boolean
            Get
                Return Displayed AndAlso _DisplayAsSortable
            End Get
            Set(value As Boolean)
                _DisplayAsSortable = value
            End Set
        End Property

        Private _DisplayAsGroupable As Boolean = True
        Public Property DisplayAsGroupable As Boolean
            Get
                Return Displayed AndAlso _DisplayAsGroupable
            End Get
            Set(value As Boolean)
                _DisplayAsGroupable = value
            End Set
        End Property


        Public Property AnalyzerProvider() As FieldManagement.Analysis.IAnalyzerProvider

        Public Function GetAnalyzer() As Analyzer
            'TODO: Mauvais, faire en sorte que la culture soit trouve intelligemment
            Return PortalQueryingDriver.GetCurrent().GetFieldAnalyzer(Me, PortalQueryingDriver.GetCurrent.DefaultAnalyzerCulture)
        End Function

#End Region

#Region "Ecrire une valeur dans un document"

        Public Sub AddFieldTo(d As Document, value As String)
            If (Index <> Field.Index.NO OrElse Store <> Field.Store.NO) Then ' modification obligatoire, j'ai t un peu trop gourmand sur le index no, ou pas?
                d.Add(New Field(Id, value, Store, Index, TermVector))
                ' si le champ a un backing pour le tri, on va rajouter un champ pour le tri
                If (BackingSortableField IsNot Nothing) Then
                    BackingSortableField.AddFieldTo(d, value)
                End If
            End If
        End Sub

#End Region

#Region "Backing Sortable Field management"
        Public Sub DisableBackingSortableField()
            BackingSortableField = Nothing
            _BackingFieldEnabled = False
        End Sub
#End Region


    End Class

End Namespace