Imports Lucene.Net.Documents
Imports Aricie.Collections
Imports Aricie.DNN.Modules.LuceneSearch.ModuleProviders
Imports Aricie.DNN.Modules.LuceneSearch.Settings


Namespace Aricie.DNN.Modules.LuceneSearch.Business
    Public Class FieldFactory

        Public Sub New()
            Me.InitDefinitions()
        End Sub

        Public Shared ReadOnly Property Instance() As FieldFactory
            Get
                Return Aricie.Services.ReflectionHelper.GetSingleton(Of FieldFactory)()
            End Get
        End Property


        Private _FieldDefinitions As New SerializableDictionary(Of String, FieldDefinition)

        Public ReadOnly Property RegisteredFieldIds() As IEnumerable(Of String)
            Get
                Return Me._FieldDefinitions.Keys
            End Get
        End Property

        Public Function GetDefinition(ByVal fieldId As String) As FieldDefinition
            If Not Me._FieldDefinitions.ContainsKey(fieldId) Then
                Return New FieldDefinition(fieldId)
            End If
            Return Me._FieldDefinitions(fieldId)
        End Function

       

        Public Sub AddFieldToDocument(d As Document, fieldId As String, fieldValue As String)
            CheckArguments(fieldId, fieldValue)
            Dim fieldDef As FieldDefinition = GetDefinition(fieldId)
            fieldDef.AddFieldTo(d, fieldValue)
        End Sub

        Public Function CreateField(ByVal fieldId As String, ByVal fieldValue As String) As Field
            CheckArguments(fieldId, fieldValue)

            Dim fieldDef As FieldDefinition = GetDefinition(fieldId)
            Return New Field(fieldId, fieldValue, fieldDef.Store, fieldDef.Index, fieldDef.TermVector)
        End Function

        Private Sub CheckArguments(fieldId As String, fieldValue As String)
            If fieldId Is Nothing Then
                Throw New ArgumentNullException("fieldName")
            End If
            If fieldValue Is Nothing Then
                Throw New ArgumentNullException("fieldValue", "Field Name = " & fieldId)
            End If
        End Sub

        Public Sub RegisterGlossary(ByVal glossary As ILuceneFieldGlossary)

            Dim fields As IList(Of FieldDefinition) = glossary.GetFieldDefinitions()
            If fields IsNot Nothing Then
                For Each objField As FieldDefinition In fields
                    If Me._FieldDefinitions.ContainsKey(objField.Id) Then
                        Throw New ArgumentException(String.Format("There are multiple definitions for field {0} in the" _
                                                                  & "set of providers used. Last Provider is {1} This is not allowed yet", objField.Id, glossary.GetType.FullName))
                    End If
                    _FieldDefinitions(objField.Id) = objField
                    ' on cre les backings pour notre champ
                    objField.SetupBackingFields()
                    If (objField.BackingSortableField IsNot Nothing) Then
                        _FieldDefinitions(objField.BackingSortableField.Id) = objField.BackingSortableField
                    End If
                Next

            End If

        End Sub



        Private Sub InitDefinitions()
            Dim glossaries As New List(Of ILuceneFieldGlossary)

            For Each objProvider As LuceneProviderConfig In ProviderFactory.Instance.GetGlossaries
                glossaries.Add(objProvider.AsGlossary)
            Next

            For Each glossary As ILuceneFieldGlossary In glossaries
                Try
                    RegisterGlossary(glossary)
                Catch ex As Exception
                    DotNetNuke.Services.Exceptions.Exceptions.LogException(ex)
                End Try
            Next


        End Sub

        Public Shared Function GetFieldDictionary(ByVal fieldDefs As IEnumerable(Of FieldDefinition)) _
            As Dictionary(Of String, FieldDefinition)
            Dim toReturn As New Dictionary(Of String, FieldDefinition)
            If fieldDefs IsNot Nothing Then
                For Each fieldDef As FieldDefinition In fieldDefs
                    toReturn.Add(fieldDef.Id, fieldDef)
                Next
            End If
            Return toReturn
        End Function

        

    End Class
End Namespace

