Imports System.IO
Imports System.Globalization
Imports Aricie.DNN.Diagnostics
Imports Lucene.Net.Index
Imports Lucene.Net.Analysis
Imports Aricie.DNN.Modules.LuceneSearch.Settings
Imports Lucene.Net.Documents
Imports DotNetNuke.Entities.Portals
Imports Lucene.Net.Search
Imports Aricie.DNN.Settings
Imports Lucene.Net.Analysis.Standard
Imports Aricie.Services
Imports DotNetNuke.Services.Mail
Imports System.Threading
Imports Aricie.DNN.Modules.LuceneSearch.Common.Constants
Imports DotNetNuke.Entities.Modules
Imports System.Linq
Imports System.Collections.Generic
Imports Lucene.Net.Search.Similar

Namespace Aricie.DNN.Modules.LuceneSearch.Business
    ''' <summary>
    ''' SearchDataStore Provider For DotNetNuke using Lucene.Net
    ''' </summary>
    Public MustInherit Class PortalDriverBase
        Implements IDisposable


#Region "members"

        Public Const glbDefaultLanguage As String = "en"
        Public Const InheritPermissionsKeyword As String = "InheritViewPermissions"

        Protected Const WriterLockTimeout As Integer = 100000
        'milliseconds

        Public Shared ReadOnly DEFAULT_SEARCHFIELDS As String() = { _
                                                                      DnnSearchItemField.Title, _
                                                                      DnnSearchItemField.Description, _
                                                                      DnnSearchItemField.Content, _
                                                                      LuceneGeneralField.AuthorName, _
                                                                      LuceneGeneralField.KeyWords, _
                                                                      LuceneGeneralField.Text, _
                                                                      LuceneGeneralField.TextNoStore, _
                                                                      LuceneGeneralField.FileName, _
                                                                      DnnSearchItemField.FriendlyModuleName, _
                                                                      DnnSearchItemField.ModuleTitle, _
                                                                      DnnSearchItemField.PubDate, _
                                                                      DnnSearchItemField.TabName}

        Private _DefaultAnalyzerCulture As String = ""
        Private _LuceneDirectory As Lucene.Net.Store.Directory
        Private _IndexReader As Lucene.Net.Index.IndexReader
        Protected _PortalId As Integer

        Private _IdxLock As New Object

        Protected Friend ReadOnly Property PortalId As Integer
            Get
                Return _PortalId
            End Get
        End Property

        Protected Friend ReadOnly Property DefaultAnalyzerCulture() As String
            Get
                Return _DefaultAnalyzerCulture
            End Get
        End Property

        Public Property LucenePortalSettings() As LucenePortalSettings
            Get
                Return GetModuleSettings(Of LucenePortalSettings)(SettingsScope.PortalSettings, Me._PortalId)
            End Get
            Set(ByVal value As LucenePortalSettings)
                SetModuleSettings(Of LucenePortalSettings)(SettingsScope.PortalSettings, Me._PortalId, value)
            End Set
        End Property

        Public Property LuceneResultsSettings(ByVal moduleId As Integer) As LuceneResultsSettings
            Get
                Return GetModuleSettings(Of LuceneResultsSettings)(SettingsScope.ModuleSettings, moduleId)
            End Get
            Set(ByVal value As LuceneResultsSettings)
                SetModuleSettings(Of LuceneResultsSettings)(SettingsScope.ModuleSettings, moduleId, value)
            End Set
        End Property

        Public ReadOnly Property SharedIndexReader() As IndexReader
            Get
                If Me._IndexReader Is Nothing Then
                    SyncLock _IdxLock
                        If Me._IndexReader Is Nothing Then
                            Me._IndexReader = Me.GetIndexReader(True)
                        End If
                    End SyncLock
                End If
                Return Me._IndexReader
            End Get
        End Property

#End Region

#Region "ctor"

        Protected Sub New(ByVal portalId As Integer)
            Me._PortalId = portalId
            If Me.LucenePortalSettings.SetPortalLanguageAsDefault Then
                Dim pc As New PortalController
                Me._DefaultAnalyzerCulture = pc.GetPortal(portalId).DefaultLanguage
            End If

        End Sub

#End Region

#Region "Public methods"



        Public Function ResetIndex() As Boolean
            Return Me.ResetIndex(Me.GetDirectory)
        End Function

        Public Function GetDirectory() As Lucene.Net.Store.Directory
            If Me._LuceneDirectory Is Nothing Then
                SyncLock _IdxLock
                    If Me._LuceneDirectory Is Nothing Then
                        Me._LuceneDirectory = Me.GetDirectory(Me.LucenePortalSettings.GetIndexFolderMapPath(Me._PortalId))
                    End If
                End SyncLock
            End If
            Return Me._LuceneDirectory
        End Function

        Public Function GetDirectory(ByVal path As String) As Lucene.Net.Store.Directory
            Dim objDir As New DirectoryInfo(path)
            Return Lucene.Net.Store.FSDirectory.Open(objDir)
        End Function

        Protected Function GetIndexModifier() As IndexWriter
            Return New IndexWriter(Me.GetDirectory, Me.GetPortalAnalyzer(), False, IndexWriter.MaxFieldLength.UNLIMITED)
        End Function

        Public Function GetSearcher() As Searcher
            Dim toReturn As IndexSearcher
            Dim reader As IndexReader = Nothing
            If Me.LucenePortalSettings.UseSharedReader Then
                reader = SharedIndexReader
            Else
                reader = GetIndexReader(True)
            End If
            toReturn = New IndexSearcher(reader)
            toReturn.SetDefaultFieldSortScoring(True, True)
            If (Not LucenePortalSettings.UseSharedReader) Then
                reader.Dispose()
            End If

            Return toReturn
        End Function

        Public Function GetDocAnalyzer(ByVal doc As Document) As Analyzer

            Dim docLanguage As String = doc.Get(LuceneGeneralField.Language)
            If String.IsNullOrEmpty(docLanguage) Then
                Return Me.GetPortalAnalyzer
            End If
            Return Me.GetCompleteAnalyzer(docLanguage)

        End Function

        Public Function GetPortalAnalyzer() As Analyzer

            Return GetCompleteAnalyzer(DefaultAnalyzerCulture)

        End Function



        Public Function GetUntokenizedAnalyzer() As Analyzer
            Return New KeywordAnalyzer()
        End Function

        Public Function GetStandardAnalyzer() As Analyzer
            Return New StandardAnalyzer(LuceneCurrentVersion)
        End Function

        Public Function GetSynonymsAnalyzer(ByVal subAnalyzer As Analyzer) As Analyzer
            Return New SynonymAnalyzer(New XmlSynonymEngine(LucenePortalSettings.SynonymeMapPath, subAnalyzer), subAnalyzer)
        End Function

        Public Function GetCompleteAnalyzer(ByVal cultureCode As String) As Analyzer

            If cultureCode = String.Empty Then
                cultureCode = DefaultAnalyzerCulture
            End If
            Dim toReturn As PerFieldAnalyzerWrapper = CacheHelper.GetGlobal(Of PerFieldAnalyzerWrapper)(Me._PortalId.ToString(CultureInfo.InvariantCulture), cultureCode)
            If toReturn Is Nothing Then
                toReturn = New DnnAnalyzer(Me)
                Dim factory As FieldFactory = FieldFactory.Instance
                For Each fieldName As String In factory.RegisteredFieldIds
                    toReturn.AddAnalyzer(fieldName, GetFieldAnalyzer(factory.GetDefinition(fieldName), cultureCode))
                Next
                CacheHelper.SetCacheDependant(Of PerFieldAnalyzerWrapper)(toReturn, Aricie.DNN.Modules.LuceneSearch.Common.Constants.Cache.DependencyKey, _
                                                                          Aricie.Constants.Cache.GlobalExpiration, Me._PortalId.ToString(CultureInfo.InvariantCulture), cultureCode)
            End If
            Return toReturn

        End Function

        Public Function GetIndexedFields() As List(Of FieldDefinition)
            Dim toReturn As New List(Of FieldDefinition)

            Try
                Dim factory As FieldFactory = FieldFactory.Instance
                Dim objSearcher As IndexReader

                If Me.LucenePortalSettings.UseSharedReader Then
                    objSearcher = SharedIndexReader
                Else
                    objSearcher = GetIndexReader(True)
                End If

                Dim fieldNames As ICollection(Of String) = objSearcher.GetFieldNames(IndexReader.FieldOption.INDEXED)
                If Not Me.LucenePortalSettings.UseSharedReader Then
                    objSearcher.Dispose()
                End If

                toReturn.AddRange(fieldNames.Select(Function(s) factory.GetDefinition(s)))
                If toReturn.Count = 0 Then
                    toReturn.AddRange(GetDefaultIndexedFields())
                End If
            Catch ex As Exception
                DotNetNuke.Services.Exceptions.Exceptions.LogException(ex)
            End Try

            Return toReturn
        End Function

        Private Function GetDefaultIndexedFields() As IList(Of FieldDefinition)
            Dim toReturn As New List(Of FieldDefinition)
            Dim factory As FieldFactory = FieldFactory.Instance

            toReturn.Add(factory.GetDefinition(DnnSearchItemField.Title))
            toReturn.Add(factory.GetDefinition(DnnSearchItemField.Content))
            toReturn.Add(factory.GetDefinition(DnnSearchItemField.Description))
            toReturn.Add(factory.GetDefinition(DnnSearchItemField.TabTitle))
            toReturn.Add(factory.GetDefinition(DnnSearchItemField.TabDescription))

            Return toReturn
        End Function

        Public Function GetSearchableFields() As IList(Of FieldDefinition)
            Return GetIndexedFields().Where(Function(f) f.DisplayAsSearchable).ToList()
        End Function


        Public Function GetSearchFields(ByVal moduleId As Integer) As List(Of FieldDefinition)

            Dim toReturn As List(Of FieldDefinition) = GetGlobal(Of List(Of FieldDefinition))(Me._PortalId.ToString(CultureInfo.InvariantCulture), moduleId.ToString(CultureInfo.InvariantCulture), "Search")

            If toReturn Is Nothing Then

                Dim filter As List(Of String) = LuceneResultsSettings(moduleId).SearchFields.Distinct.ToList()
                If filter.Count = 0 Then
                    filter = LucenePortalSettings.DefaultSearchFields.Distinct().ToList()
                    ' si on a pas de champ dfini au niveau du portail,on inclut ceux par dfaut
                    If (filter.Count = 0) Then
                        filter = New List(Of String)(PortalDriverBase.DEFAULT_SEARCHFIELDS).Distinct().ToList()
                    End If
                End If
                toReturn = GetSearchableFields().Where(Function(f) filter.Contains(f.Id)).ToList()

                SetCacheDependant(Of List(Of FieldDefinition))(toReturn, Common.Constants.Cache.DependencyKey, TimeSpan.FromMinutes(10), _
                                                                 Me._PortalId.ToString(CultureInfo.InvariantCulture), _
                                                                 moduleId.ToString(CultureInfo.InvariantCulture), _
                                                                 "Search")
            End If

            Return toReturn

        End Function

        Public Function GetSearchableModules() As List(Of DesktopModuleInfo)
            Dim toReturn As New List(Of DesktopModuleInfo)
            Dim mCtlr As New DesktopModuleController
            For Each moduleName As String In LucenePortalSettings.IndexedModuleNames
                toReturn.Add(mCtlr.GetDesktopModuleByModuleName(moduleName))
            Next
            Return toReturn
        End Function

        Public Function GetTopTerms(ByVal fieldName As String, ByVal maxNb As Integer) As List(Of TermInfo)

            Dim toReturn As List(Of TermInfo) = CacheHelper.GetGlobal(Of List(Of TermInfo))(_PortalId.ToString(), fieldName, maxNb.ToString)
            If toReturn Is Nothing Then
                toReturn = New List(Of TermInfo)

                Dim objReader As IndexReader
                If Me.LucenePortalSettings.UseSharedReader Then
                    objReader = Me.SharedIndexReader
                Else
                    objReader = Me.GetIndexReader(True)

                End If

                Dim objTerms As TermEnum = objReader.Terms(New Term(fieldName, ""))

                Dim docFreq As Integer
                Do
                    Dim objTerm As Term = objTerms.Term
                    docFreq = objTerms.DocFreq
                    If objTerm IsNot Nothing AndAlso objTerm.Field = fieldName Then
                        toReturn.Add(New TermInfo(objTerm, docFreq))
                    End If
                Loop While objTerms.Next AndAlso (toReturn.Count <= maxNb OrElse maxNb = 0)

                If Not Me.LucenePortalSettings.UseSharedReader Then
                    objReader.Dispose()
                End If

                CacheHelper.SetCacheDependant(Of List(Of TermInfo))(toReturn, Common.Constants.Cache.DependencyKey, TimeSpan.FromMinutes(2), _PortalId.ToString(), fieldName, maxNb.ToString)

            End If

            Return toReturn

        End Function

        Public Function GetSimilarResultsQuery(ByVal moduleId As Integer, ByVal document As Document) As Query
            Dim completeQuery As New BooleanQuery

            Dim defaultAnalyzer As Analyzer = Me.GetDocAnalyzer(document)
            Dim searchFields As List(Of FieldDefinition) = Me.GetSearchFields(moduleId)
            Dim _
                searchFieldsAsDico As Dictionary(Of String, FieldDefinition) = _
                    FieldFactory.GetFieldDictionary(searchFields)

            For Each objField As Field In document.GetFields
                If searchFieldsAsDico.ContainsKey(objField.Name) Then
                    Dim _
                        filterQuery As Query = _
                            SimilarityQueries.FormSimilarQuery(objField.StringValue, defaultAnalyzer, objField.Name, _
                                                                Nothing)
                    filterQuery.Boost = objField.Boost
                    completeQuery.Add(filterQuery, Occur.SHOULD)
                End If
            Next

            Return completeQuery

        End Function

#End Region

#Region "private and protected methods"

        Friend Function GetIndexReader(ByVal openReadOnly As Boolean) As IndexReader
            Dim objDirectory As Lucene.Net.Store.Directory = Me.GetDirectory

            If Not IndexReader.IndexExists(objDirectory) Then
                SyncLock _IdxLock
                    If Not IndexReader.IndexExists(objDirectory) Then
                        Me.ResetIndex(objDirectory)
                    End If
                End SyncLock
            End If

            Return IndexReader.Open(objDirectory, openReadOnly)

        End Function


        Protected Friend Function GetFieldAnalyzer(ByVal fieldDef As FieldDefinition, ByVal cultureCode As String) As Analyzer

            Dim toReturn As Analyzer

            ' added to be able to specify the analyzer for each field
            If (fieldDef.AnalyzerProvider IsNot Nothing) Then
                Return fieldDef.AnalyzerProvider.GetAnalyzer()
            End If

            If fieldDef.Index = Field.Index.NOT_ANALYZED Then
                toReturn = GetUntokenizedAnalyzer()
            ElseIf fieldDef.IsLocalized Then
                toReturn = GetLocalizedAnalyzer(cultureCode)
            Else
                toReturn = GetStandardAnalyzer()
            End If
            If fieldDef.UseSynonyms Then
                toReturn = GetSynonymsAnalyzer(toReturn)
            End If

            Return toReturn

        End Function

        Private Function GetLocalizedAnalyzer(ByVal cultureCode As String) As Analyzer

            If Not String.IsNullOrEmpty(cultureCode) AndAlso Me.LucenePortalSettings.UseSnowBallAnalysers Then
                cultureCode = NormalizeCulture(cultureCode)
                Select Case cultureCode
                    Case "en"
                        Return New LocalizedSnowballAnalyzer("English")
                    Case "de"
                        Return New LocalizedSnowballAnalyzer("German")
                    Case "es"
                        Return New LocalizedSnowballAnalyzer("Spanish")
                    Case "fr"
                        Return New LocalizedSnowballAnalyzer("French")
                    Case "it"
                        Return New LocalizedSnowballAnalyzer("Italian")
                    Case "nl-nl"
                        Return New LocalizedSnowballAnalyzer("Dutch")
                    Case "no"
                        Return New LocalizedSnowballAnalyzer("Norwegian")
                    Case "pt"
                        Return New LocalizedSnowballAnalyzer("Portuguese")
                    Case "ru"
                        Return New LocalizedSnowballAnalyzer("Russian")
                    Case "sv"
                        Return New LocalizedSnowballAnalyzer("Swedish")
                End Select
            End If
            Return GetStandardAnalyzer()

        End Function

        Private Function NormalizeCulture(ByVal culture As String) As String

            If Not String.IsNullOrEmpty(culture) Then
                culture = culture.ToLower(CultureInfo.InvariantCulture)
                If ((culture = "en") OrElse culture.StartsWith("en-")) Then
                    Return "en"
                End If
                If ((culture = "de") OrElse culture.StartsWith("de-")) Then
                    Return "de"
                End If
                If ((culture = "da") OrElse culture.StartsWith("da-")) Then
                    Return "da"
                End If
                If ((culture = "es") OrElse culture.StartsWith("es-")) Then
                    Return "es"
                End If
                If ((culture = "fi") OrElse culture.StartsWith("fi-")) Then
                    Return "fi"
                End If
                If ((culture = "fr") OrElse culture.StartsWith("fr-")) Then
                    Return "fr"
                End If
                If ((culture = "it") OrElse culture.StartsWith("it-")) Then
                    Return "it"
                End If
                If ((culture = "ja") OrElse culture.StartsWith("ja-")) Then
                    Return "ja"
                End If
                If ((culture = "ko") OrElse culture.StartsWith("ko-")) Then
                    Return "ko"
                End If
                If (culture = "nl-nl") Then
                    Return culture
                End If
                If (((culture = "no") OrElse culture.StartsWith("nb-")) OrElse culture.StartsWith("nn-")) Then
                    Return "no"
                End If
                If ((culture = "pt") OrElse culture.StartsWith("pt-")) Then
                    Return "pt"
                End If
                If ((culture = "ru") OrElse culture.StartsWith("ru-")) Then
                    Return "ru"
                End If
                If ((culture = "sv") OrElse culture.StartsWith("sv-")) Then
                    Return "sv"
                End If
                If ((culture = "zh") OrElse culture.StartsWith("zh-")) Then
                    Return "zh"
                End If
            End If
            Return glbDefaultLanguage

        End Function




        Private Function ResetIndex(ByVal objDirectory As Lucene.Net.Store.Directory) As Boolean
            Dim IndexResetSuccess As Boolean = False
            Try
                Dim modifier As New IndexWriter(objDirectory, Me.GetPortalAnalyzer(), True, IndexWriter.MaxFieldLength.UNLIMITED)
                modifier.DeleteAll()
                modifier.Dispose()
                If Me.LucenePortalSettings.LogIndexingProcess Then
                    SimpleDebugLogger.Instance.AddDebugInfo(New DebugInfo(LuceneLogType.IndexProcessing.ToString, "Index Reset", "Lucene File Index was emptied", True, Me._PortalId))
                End If
                ' success: so say we all
                IndexResetSuccess = True
            Catch ex As Exception
                DotNetNuke.Services.Exceptions.Exceptions.LogException(ex)
            End Try

            Return IndexResetSuccess
        End Function


        Friend Sub ReleaseState(ByVal shutDownDirectory As Boolean)
            SyncLock Me._IdxLock
                If Me._IndexReader IsNot Nothing Then
                    Me._IndexReader.Dispose()
                    Me._IndexReader = Nothing
                End If
                If shutDownDirectory AndAlso Me._LuceneDirectory IsNot Nothing Then
                    Me._LuceneDirectory.Dispose()
                    Me._LuceneDirectory = Nothing
                End If
            End SyncLock
        End Sub

#End Region

        Private disposedValue As Boolean = False        ' To detect redundant calls

        ' IDisposable
        Protected Overridable Sub Dispose(ByVal disposing As Boolean)
            If Not Me.disposedValue Then
                If disposing Then
                    Me.ReleaseState(True)
                End If

            End If
            Me.disposedValue = True
        End Sub

#Region " IDisposable Support "
        ' This code added by Visual Basic to correctly implement the disposable pattern.
        Public Sub Dispose() Implements IDisposable.Dispose
            ' Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
            Dispose(True)
            GC.SuppressFinalize(Me)
        End Sub
#End Region

    End Class

End Namespace