Imports System.Threading
Imports System.Globalization
Imports Lucene.Net.Analysis
Imports Lucene.Net.Index
Imports Lucene.Net.Documents
Imports Lucene.Net.Search
Imports Lucene.Net.QueryParsers
Imports Aricie.DNN.Services
Imports Aricie.Services
Imports DotNetNuke.Common.Utilities
Imports DotNetNuke.Entities.Users
Imports DotNetNuke.Security.Roles
Imports Aricie.DNN.Modules.LuceneSearch.Settings
Imports Aricie.DNN.Modules.LuceneSearch.Common.Constants
Imports System.Linq
Imports Lucene.Net.Search.Highlight

Namespace Aricie.DNN.Modules.LuceneSearch.Business
    Public Class PortalQueryingDriver
        Inherits PortalDriverBase

#Region "ctor"

        Public Shared Function GetCurrent() As PortalQueryingDriver
            Return GetByPortalId(NukeHelper.PortalId)
        End Function


        Public Shared Function GetByPortalId(ByVal portalId As Integer) As PortalQueryingDriver
            Dim toreturn As PortalQueryingDriver = GetGlobal(Of PortalQueryingDriver)(portalId.ToString)
            If toreturn Is Nothing Then
                toreturn = New PortalQueryingDriver(portalId)
                SetGlobal(Of PortalQueryingDriver)(toreturn, portalId.ToString)
            End If
            Return toreturn

        End Function


        Private Sub New(ByVal portalId As Integer)

            MyBase.New(portalId)

        End Sub


#End Region

#Region "Public methods"




        Public Function GetHighlighter(ByVal objQuery As Query, ByVal searchModuleId As Integer) As Highlighter
            Dim _
                cacheKey As String = _
                    Constants.GetKey(Me._PortalId.ToString(CultureInfo.InvariantCulture), objQuery.ToString)
            Dim toReturn As Highlighter = GetGlobal(Of Highlighter)(cacheKey)
            If toReturn Is Nothing Then

                ' on cre la liste des termes corrects, pas la version concatne avec les / dans ExtractTerms (pourquoi cette ide d'ailleurs?)
                Dim QueryTerms = BusinessController.ExtractAllTerms(objQuery)
                Dim ListWeightedTerms As New List(Of WeightedSpanTerm)
                Dim ListSearchedFields = GetSearchFields(searchModuleId).Select(Function(FD) FD.Id.ToLowerInvariant())
                For Each T As Term In QueryTerms.Where(Function(TCourant) ListSearchedFields.Contains(TCourant.Field().ToLowerInvariant()))
                    ListWeightedTerms.Add(New WeightedSpanTerm(objQuery.Boost, T.Text()))
                Next

                toReturn = New Highlighter(Me.LuceneResultsSettings(searchModuleId).GetFormatter, New SimpleHTMLEncoder(), New QueryScorer(ListWeightedTerms.ToArray()))
                toReturn.TextFragmenter = New SimpleFragmenter(Me.LuceneResultsSettings(searchModuleId).HighlighterFragmentsMaxNbTokens)

                SetCacheDependant(Of Highlighter)(toReturn, Common.Constants.Cache.DependencyKey, TimeSpan.FromMinutes(3), cacheKey)
            End If

            Return toReturn

        End Function

        Public Function CreateHighlightsNode(ByVal currentDocument As Document, ByVal objQuery As Query, _
                                              ByVal searchModuleId As Integer) As Dictionary(Of String, String)


            Dim toReturn As New Dictionary(Of String, String)

            Dim objHighlighter As Highlighter = GetHighlighter(objQuery, searchModuleId)

            'On recupere dans l'index tous les champs sur lesquel on effectue la recherche:
            Dim termList As New HashSet(Of Term)
            objQuery = objQuery.Rewrite(Me.GetIndexReader(True)) 'rewrite pour tester l'ensemble des termes existants dans l'index
            objQuery.ExtractTerms(termList)

            For Each item As Term In termList
                Dim field As String = item.Field
                For Each _fieldDef As FieldDefinition In Me.GetSearchFields(searchModuleId)
                    If _fieldDef.Id = field AndAlso Not currentDocument.GetField(field) Is Nothing Then
                        Dim myFieldValueString As String = currentDocument.GetField(field).StringValue

                        Dim results As String() = objHighlighter.GetBestFragments(Me.GetDocAnalyzer(currentDocument), _
                                                                                   field, _
                                                                                   HtmlUtils.Clean(myFieldValueString, _
                                                                                                   False), _
                                                                                   Me.LuceneResultsSettings(searchModuleId).HighlighterMaxNbFragments)

                        Dim concatResults As String = ""
                        For Each myResult As String In results
                            concatResults += "..." + myResult
                        Next
                        If Not String.IsNullOrEmpty(concatResults) Then
                            toReturn(field) = concatResults
                        End If
                    End If
                Next
            Next

            Return toReturn


        End Function

        Public Shared Function GetLanguageFilter(ByVal filterQuery As Query) As String
            Dim toReturn As String = ""
            If filterQuery IsNot Nothing Then
                ' je passe par la mthode du business controller car cette dernire effectue 
                ' le rewrite potentiellement ncessaire pour grer correctement les TermRangeQueries ou les Prefixqueries
                
                For Each objterm As Term In BusinessController.ExtractAllTerms(filterQuery)
                    If objterm.Field = LuceneGeneralField.Language Then
                        toReturn = BusinessController.GetTermText(objterm)
                        Exit For
                    End If
                Next

            End If
            Return toReturn
        End Function

        Public Function BuildSort(ByVal sorters As String) As Sort

            Dim objSort As Sort = Nothing
            Dim sorts As String() = sorters.Split(","c)

            Dim objSorts As New List(Of SortField)
            For Each strFieldName As String In sorts

                If strFieldName <> "" Then

                    Dim definition As FieldDefinition = FieldFactory.Instance.GetDefinition(strFieldName.TrimStart("-"c))

                    objSorts.Add(New SortField( _
                                 definition.SortableFieldId _
                                 , DirectCast(IIf(definition.Type > -1, definition.Type, SortField.STRING), Integer) _
                                 , strFieldName.StartsWith("-"c)))
                End If

            Next
            If objSorts.Count > 0 Then
                objSort = New Sort(objSorts.ToArray)
            Else
                objSort = Sort.RELEVANCE
            End If

            Return objSort

        End Function


        Public Function BuildQuery(ByVal moduleId As Integer, ByVal objQuery As LuceneFilterQuery) As Query

            Dim toReturn As New BooleanQuery()

            ' add current portal
            Dim portalQuery As New TermQuery(New Term(DnnSearchItemField.PortalId, Me._PortalId.ToString))
            toReturn.Add(portalQuery, Occur.MUST)

            ' add visibility range and options
            Dim VisibilityQuery As New BooleanQuery()
            VisibilityQuery.Add(New TermQuery(New Term(LuceneGeneralField.CanBeReturnedBySearch, True.ToString())), Occur.MUST)

            Dim BeginningOfTime = DateTools.DateToString(New Date(1, 1, 1), DateTools.Resolution.SECOND)
            Dim RightAboutNow = DateTools.DateToString(Now, DateTools.Resolution.SECOND)
            Dim EndOfTime = DateTools.DateToString(New Date(9999, 12, 31), DateTools.Resolution.SECOND)

            VisibilityQuery.Add(New TermRangeQuery(LuceneGeneralField.ReturnedBySearchWindowStart, BeginningOfTime, RightAboutNow, True, True), Occur.MUST)
            VisibilityQuery.Add(New TermRangeQuery(LuceneGeneralField.ReturnedBySearchWindowEnd, RightAboutNow, EndOfTime, True, True), Occur.MUST)

            toReturn.Add(VisibilityQuery, Occur.MUST)

            ' add current user roles
            If Me.LuceneResultsSettings(moduleId).PermissionFilter = Settings.PermissionFilter.FilterQuery Then

                'Bug Correction : Check If DotNetNuke Context Exist (because of UserController.GetCurrentUserInfo)
                'When Schedule Search Indexation is execute => Context don't exist and it shouldn't try Get Current UserInfo
                'and Build Permission Query
                If HttpContext.Current IsNot Nothing Then
                    Dim currentUser As UserInfo = UserController.GetCurrentUserInfo
                    If Not currentUser.IsSuperUser Then
                        toReturn.Add(Me.GetPermissionQuery(currentUser), Occur.MUST)
                    End If

                End If

            End If


            Dim filterQuery As Query = Nothing
            Dim filterParser As QueryParser = Nothing
            If objQuery.FilterQuery IsNot Nothing Then
                filterQuery = objQuery.FilterQuery
            End If

            If moduleId <> -1 Then
                If Me.LuceneResultsSettings(moduleId).ApplyDefaultLanguageFilter Then
                    filterQuery = BusinessController.GetRefinedQuery(filterQuery, BusinessController.GetCurrentLocaleQuery())
                End If
                If Me.LuceneResultsSettings(moduleId).AdditionalFilter <> "" Then
                    If filterParser Is Nothing Then
                        filterParser = Me.GetDefaultQueryParser(Thread.CurrentThread.CurrentCulture.Name)
                    End If

                    Dim additional As String
                    If Me.LuceneResultsSettings(moduleId).EnableAdditionalFilterTokenReplace Then
                        Dim replace As New AdvancedTokenReplace()
                        additional = replace.ReplaceAllTokens(Me.LuceneResultsSettings(moduleId).AdditionalFilter)
                    Else
                        additional = Me.LuceneResultsSettings(moduleId).AdditionalFilter
                    End If

                    Dim addFilterQuery As Query = filterParser.Parse(additional)
                    filterQuery = BusinessController.GetRefinedQuery(filterQuery, addFilterQuery)
                End If
            End If


            If filterQuery IsNot Nothing Then
                toReturn.Add(filterQuery, Occur.MUST)
            End If


            ' TODO: revenir demander pourquoi on a cet lment ici; il semblerait qu'on rinjecte le langage dans la searchquery, je ne comprends pas l'utilit
            ' en plus cela fait planter les combinaisons de recherche et de filtres
            If objQuery.SearchQuery IsNot Nothing AndAlso objQuery.SearchQuery.Trim <> "" Then
                Dim filterLanguage As String = GetLanguageFilter(filterQuery)
                Dim contentQuery As Query = Me.GetMultiFieldQuery(moduleId, objQuery.SearchQuery, filterLanguage)
                toReturn.Add(contentQuery, Occur.MUST)
            End If


            Return toReturn

        End Function

        Public Function GetMultiFieldQuery(ByVal moduleId As Integer, ByVal criteria As String, _
                                            ByVal filterLanguage As String) As Query
            Dim toReturn As Query = GetGlobal(Of Query)(Me._PortalId.ToString, _
                                                          moduleId.ToString(CultureInfo.InvariantCulture), criteria, _
                                                          filterLanguage)

            If toReturn Is Nothing Then

                toReturn = GetMultiFieldQueryParser(moduleId, filterLanguage).Parse(criteria)
                SetCacheDependant(Of Query)(toReturn, Common.Constants.Cache.DependencyKey, TimeSpan.FromMinutes(10), _
                                              Me._PortalId.ToString(CultureInfo.InvariantCulture), _
                                              moduleId.ToString(CultureInfo.InvariantCulture), criteria, filterLanguage)
            End If

            Return toReturn
        End Function


        Public Function GetDefaultQueryParser(ByVal culture As String) As QueryParser
            Dim parser As New QueryParser(LuceneCurrentVersion, "", Me.GetCompleteAnalyzer(culture))
            parser.AllowLeadingWildcard = True
            parser.LowercaseExpandedTerms = False
            parser.MultiTermRewriteMethod = MultiTermQuery.SCORING_BOOLEAN_QUERY_REWRITE
            Return parser
        End Function


#Region " RunQuery Methods"
        Public Function RunQuery(ByVal portalId As Integer, ByVal objLuceneQuery As LuceneFilterQuery) As LuceneQueryResults
            Return RunQuery(portalId, -1, objLuceneQuery, "")

        End Function

        Public Function RunQuery(ByVal portalId As Integer, ByVal searchModuleId As Integer, ByVal objLuceneQuery As LuceneFilterQuery, ByVal sorters As String) As LuceneQueryResults
            Return RunQuery(portalId, searchModuleId, objLuceneQuery, sorters, -1, 0)
        End Function

        Public Function RunQuery(ByVal portalId As Integer, ByVal searchModuleId As Integer, ByVal objLuceneQuery As LuceneFilterQuery, ByVal sorters As String, _
                                  ByVal resultPageIndex As Integer, ByVal pageSize As Integer) As LuceneQueryResults

            Dim objSort As Sort = Me.BuildSort(sorters)

            Return RunQuery(portalId, searchModuleId, objLuceneQuery, objSort, resultPageIndex, pageSize)

        End Function


        Public Function RunQuery(ByVal portalId As Integer, ByVal searchModuleId As Integer, ByVal objLuceneQuery As LuceneFilterQuery, _
                                  ByVal objSort As Sort, ByVal resultPageIndex As Integer, _
                                  ByVal pageSize As Integer) As LuceneQueryResults

            Dim useCache As Boolean
            Dim toReturn As LuceneQueryResults = Nothing
            Dim queryDriver As PortalQueryingDriver = PortalQueryingDriver.GetByPortalId(portalId)
            Dim objQuery As Query = Me.BuildQuery(searchModuleId, objLuceneQuery)


            If searchModuleId > 0 Then
                useCache = queryDriver.LuceneResultsSettings(searchModuleId).NbSecondsCacheResults > 0
            End If

            If useCache Then
                toReturn = CacheHelper.GetGlobal(Of LuceneQueryResults)(portalId.ToString, searchModuleId.ToString, objQuery.ToString, _
                                                                        objSort.ToString, Thread.CurrentThread.CurrentCulture.Name, resultPageIndex.ToString, pageSize.ToString)

            End If

            If toReturn Is Nothing Then
                toReturn = RunQuery(searchModuleId, objQuery, objSort, resultPageIndex, pageSize)

                If useCache Then
                    SetCacheDependant(Of LuceneQueryResults)(toReturn, Common.Constants.Cache.DependencyKey, _
                                TimeSpan.FromSeconds(queryDriver.LuceneResultsSettings(searchModuleId).NbSecondsCacheResults), portalId.ToString, searchModuleId.ToString, objQuery.ToString, _
                                                                        objSort.ToString, Thread.CurrentThread.CurrentCulture.Name, resultPageIndex.ToString, pageSize.ToString)
                End If
            End If

            Return toReturn

        End Function


        Public Function RunQuery(ByVal searchModuleId As Integer, ByVal objQuery As Query, ByVal objSort As Sort, _
                                  ByVal resultPageIndex As Integer, ByVal pageSize As Integer) As LuceneQueryResults

            System.GC.Collect() 'Free memory (Error Memory Exception when nb results are too big)

            Dim resultIndex As Integer = GetResultIndex(searchModuleId, resultPageIndex, pageSize)

            Dim objSearcher As Searcher = Me.GetSearcher
            Dim nbProcessedResults As Integer
            If resultIndex >= 0 AndAlso pageSize > 0 Then
                nbProcessedResults = resultIndex + pageSize
            Else
                nbProcessedResults = objSearcher.MaxDoc
            End If

            Dim objTopDocs As TopDocs = Me.RunLuceneQuery(objQuery, nbProcessedResults, objSort, objSearcher)


            Dim nbResultsFound As Integer = objTopDocs.TotalHits
            Dim startIdx As Integer = 0
            Dim endIdx As Integer = objTopDocs.ScoreDocs.Length - 1


            If resultIndex >= 0 AndAlso pageSize > 0 Then
                If nbResultsFound > resultIndex Then
                    startIdx = resultIndex
                    If pageSize < nbResultsFound - resultIndex Then
                        endIdx = startIdx + pageSize - 1
                    End If
                Else
                    startIdx = endIdx + 1
                End If
            End If

            Return New LuceneQueryResults(Me, objQuery, objSearcher, objTopDocs, startIdx, endIdx)
        End Function


        Public Function RunLuceneQuery(ByVal objQuery As Query, ByVal nbProcessedResults As Integer, _
                                ByVal objSort As Sort, ByVal objSearcher As Searcher) As TopDocs

            Dim toReturn As TopDocs
            If objSort IsNot Nothing Then
                toReturn = objSearcher.Search(objQuery, Nothing, nbProcessedResults, objSort)
            Else
                toReturn = objSearcher.Search(objQuery, Nothing, nbProcessedResults)
            End If
            Return toReturn
        End Function
#End Region

        Public Function GetUserRoleIds(ByVal portalId As Integer, ByVal userId As Integer) As List(Of Integer)
            Dim toReturn As List(Of Integer) = CacheHelper.GetPersonal(Of List(Of Integer))(userId, "roleIds")
            If toReturn Is Nothing Then
                toReturn = New List(Of Integer)
                toReturn.Add(-1)
                If userId <> -1 Then
                    Dim roles As ArrayList = NukeHelper.RoleController.GetUserRoles(portalId, userId, True)
                    For Each objRole As RoleInfo In roles
                        toReturn.Add(objRole.RoleID)
                    Next

                    'toReturn.Add(NukeHelper.PortalInfo(portalId).RegisteredRoleId)
                Else
                    toReturn.Add(-3)
                End If
                CacheHelper.SetPersonal(Of List(Of Integer))(toReturn, userId, "roleIds")
            End If
            Return toReturn
        End Function

        Public Function GetResultIndex(ByVal moduleId As Integer, ByVal resultPageIndex As Integer, ByVal pageSize As Integer) As Integer
            Dim toReturn As Integer = -1

            If moduleId <= 0 Then
                toReturn = resultPageIndex * pageSize
            Else
                'ModuleId is an ID for AricieLuceneSearch Module
                Dim settings As LuceneResultsSettings = Me.LuceneResultsSettings(moduleId)
                If settings IsNot Nothing AndAlso settings.EnablePaging Then
                    If Not settings.AllowCustomPageSize Then
                        pageSize = settings.DefaultPageSize
                    End If
                    toReturn = resultPageIndex * pageSize
                End If
            End If

            Return toReturn
        End Function



#End Region

#Region "Private methods"

        Private Function GetMultiFieldQueryParser(ByVal moduleId As Integer, ByVal filterLanguage As String) As QueryParser

            Dim toReturn As New MultiFieldQueryParser(LuceneCurrentVersion, GetSearchFields(moduleId).Select(Function(f) f.Id).ToArray(), GetCompleteAnalyzer(filterLanguage))
            toReturn.AllowLeadingWildcard = True
            toReturn.LowercaseExpandedTerms = False
            toReturn.MultiTermRewriteMethod = MultiTermQuery.SCORING_BOOLEAN_QUERY_REWRITE ' useful for highlights
            toReturn.DefaultOperator = QueryParser.Operator.AND

            Return toReturn
        End Function


        Protected Friend Function GetPermissionQuery(ByVal objUser As UserInfo) As Query

            Dim toReturn As New BooleanQuery()

            Dim tabQuery As New BooleanQuery()
            Dim moduleQuery As New BooleanQuery
            Dim itemQuery As New BooleanQuery

            moduleQuery.Add(New TermQuery(New Term(DnnSearchItemField.InheritTabViewPermissions, (True).ToString(CultureInfo.InvariantCulture).ToLower())), Occur.SHOULD)
            itemQuery.Add(New TermQuery(New Term(DnnSearchItemField.InheritModuleViewPermissions, (True).ToString(CultureInfo.InvariantCulture).ToLower())), Occur.SHOULD)

            If objUser.UserID <> -1 Then

                tabQuery.Add(New TermQuery(New Term(DnnSearchItemField.TabViewUserIds, objUser.UserID.ToString(CultureInfo.InvariantCulture))), Occur.SHOULD)
                moduleQuery.Add(New TermQuery(New Term(DnnSearchItemField.ModuleViewUserIds, objUser.UserID.ToString(CultureInfo.InvariantCulture))), Occur.SHOULD)
                itemQuery.Add(New TermQuery(New Term(DnnSearchItemField.ItemViewUserIds, objUser.UserID.ToString(CultureInfo.InvariantCulture))), Occur.SHOULD)

            End If

            Dim roleIds As List(Of Integer) = GetUserRoleIds(objUser.PortalID, objUser.UserID)

            For Each roleId As Integer In roleIds
                'If roleId > 0 Then
                tabQuery.Add(New TermQuery(New Term(DnnSearchItemField.TabViewRoleIds, roleId.ToString(CultureInfo.InvariantCulture))), Occur.SHOULD)
                moduleQuery.Add(New TermQuery(New Term(DnnSearchItemField.ModuleViewRoleIds, roleId.ToString(CultureInfo.InvariantCulture))), Occur.SHOULD)
                itemQuery.Add(New TermQuery(New Term(DnnSearchItemField.ItemViewRoleIds, roleId.ToString(CultureInfo.InvariantCulture))), Occur.SHOULD)
                'Else
                '    tabQuery.Add(New TermQuery(New Term(DnnSearchItemField.TabViewRoleIds, "\" & roleId.ToString(CultureInfo.InvariantCulture))), Occur.SHOULD)
                '    moduleQuery.Add(New TermQuery(New Term(DnnSearchItemField.ModuleViewRoleIds, "\" & roleId.ToString(CultureInfo.InvariantCulture))), Occur.SHOULD)
                '    itemQuery.Add(New TermQuery(New Term(DnnSearchItemField.ItemViewRoleIds, "\" & roleId.ToString(CultureInfo.InvariantCulture))), Occur.SHOULD)
                'End If
            Next

            toReturn.Add(tabQuery, Occur.MUST)
            toReturn.Add(moduleQuery, Occur.MUST)
            toReturn.Add(itemQuery, Occur.MUST)


            Return toReturn


        End Function
#End Region




#Region "Obsoltes Methods"


        <Obsolete("Please use the LuceneFilterQuery structure rather than 'Criteria' string param")> _
        Public Function RunQuery(ByVal portalId As Integer, ByVal criteria As String) As LuceneQueryResults
            Dim objLuceneQuery As New LuceneFilterQuery
            objLuceneQuery.SearchQuery = criteria
            Return Me.RunQuery(portalId, objLuceneQuery)
        End Function



        ''' <summary>
        ''' Search a specified String in the index on a given portal
        ''' </summary>
        ''' <param name="Criteria">Query String to search in the index</param>
        ''' <returns>SearchResultsInfoCollection containing the results of the query</returns>
        <Obsolete("Please use the LuceneFilterQuery structure rather than 'Criteria' and 'AdditionnalFilter' string params")> _
        Public Function RunQuery(ByVal portalId As Integer, ByVal criteria As String, ByVal additionalFilter As String) _
            As LuceneQueryResults
            Dim objLuceneQuery As New LuceneFilterQuery
            objLuceneQuery.SearchQuery = criteria
            If Not String.IsNullOrEmpty(additionalFilter) Then
                objLuceneQuery.FilterQuery = BusinessController.GetQuery(portalId, additionalFilter)
            End If
            Return Me.RunQuery(portalId, objLuceneQuery)
        End Function



        <Obsolete("Please use the LuceneFilterQuery structure rather than 'Criteria' and 'AdditionnalFilter' string params")> _
        Public Function RunQuery(ByVal portalId As Integer, ByVal searchModuleId As Integer, ByVal criteria As String, ByVal additionalFilter As String, ByVal sorters As String) As LuceneQueryResults
            Dim objLuceneQuery As New LuceneFilterQuery
            objLuceneQuery.SearchQuery = criteria
            If Not String.IsNullOrEmpty(additionalFilter) Then
                objLuceneQuery.FilterQuery = BusinessController.GetQuery(portalId, additionalFilter)
            End If
            Return RunQuery(portalId, searchModuleId, objLuceneQuery, sorters)
        End Function

        <Obsolete("Please use the LuceneFilterQuery structure rather than 'Criteria' and 'AdditionnalFilter' string params")> _
        Public Function RunQuery(ByVal portalId As Integer, ByVal searchModuleId As Integer, ByVal criteria As String, ByVal additionalFilter As String, ByVal sorters As String, _
                          ByVal resultPageIndex As Integer, ByVal pageSize As Integer) As LuceneQueryResults

            Dim objLuceneQuery As New LuceneFilterQuery
            objLuceneQuery.SearchQuery = criteria
            If Not String.IsNullOrEmpty(additionalFilter) Then
                objLuceneQuery.FilterQuery = BusinessController.GetQuery(portalId, additionalFilter)
            End If

            Return RunQuery(portalId, searchModuleId, objLuceneQuery, sorters, resultPageIndex, pageSize)

        End Function

        <Obsolete("Please use the LuceneFilterQuery structure rather than 'Criteria' and 'AdditionnalFilter' string params")> _
        Public Function RunQuery(ByVal portalId As Integer, ByVal searchModuleId As Integer, ByVal criteria As String, _
                         ByVal additionalFilter As String, ByVal objSort As Sort, ByVal resultPageIndex As Integer, _
                         ByVal pageSize As Integer) As LuceneQueryResults

            Dim objLuceneQuery As New LuceneFilterQuery
            objLuceneQuery.SearchQuery = criteria
            If Not String.IsNullOrEmpty(additionalFilter) Then
                objLuceneQuery.FilterQuery = BusinessController.GetQuery(portalId, additionalFilter)
            End If

            Return RunQuery(portalId, searchModuleId, objLuceneQuery, objSort, resultPageIndex, pageSize)

        End Function

        <Obsolete("Please use the LuceneFilterQuery structure rather than 'Criteria' and 'AdditionnalFilter' string params")> _
        Public Function BuildQuery(ByVal moduleId As Integer, ByVal criteria As String, _
                                  ByVal additionalFilter As String) As Query
            Dim objQuery As New LuceneFilterQuery
            objQuery.SearchQuery = criteria
            If String.IsNullOrEmpty(additionalFilter) Then
                objQuery.FilterQuery = BusinessController.GetQuery(PortalId, additionalFilter)
            End If
            Return BuildQuery(moduleId, objQuery)
        End Function

#End Region


    End Class
End Namespace

