Imports Aricie.DNN.Modules.LuceneSearch.Business
Imports DotNetNuke.Framework.Providers
Imports Aricie.DNN.Services
Imports System.IO
Imports System.Xml
Imports Aricie.Services
Imports DotNetNuke.Entities.Modules
Imports Aricie.DNN.Modules.LuceneSearch.Settings
Imports System.Linq

Namespace Aricie.DNN.Modules.LuceneSearch.ModuleProviders
    ''' <summary>
    ''' ProviderFactory class used to retrieve the providers that can be used as document content indexer in the Lucene.Net Indexer
    ''' </summary>
    Public Class ProviderFactory

#Region "Private members"

        '''' <summary>
        '''' dictionary containing Module Name/ IModuleProvider
        '''' </summary>
        '''' <remarks>01/06/08: made shared because there is few chances for now that more than one folder need to get parsed.</remarks>
        'Private Shared _modulesSearchProvider As Dictionary(Of String, ILuceneSearchable)

        'Private Shared _standAlonesModulesSearchProvider As Dictionary(Of String, StandAloneLuceneSearchableBase)

        Private _ProvidersByName As New Dictionary(Of String, LuceneProviderConfig)
        Private _Glossaries As New List(Of String)
        Private _ModuleSearchables As New Dictionary(Of String, String)
        Private _Upgraders As New Dictionary(Of String, String)
        Private _StandAlones As New List(Of String)
        Private _ProgressiveStandalones As New List(Of LuceneProviderConfig)
        Private _IUpgradeEventHandlerProviders As New List(Of IUpgradeEventsHandlerProvider)
        Private _IReflexiveProviders As New List(Of ILuceneReflexiveProvider)

#End Region


#Region "cTors"

        Sub New()
            LoadProviders()
        End Sub

        Public Shared ReadOnly Property Instance() As ProviderFactory
            Get
                Return LuceneSearchConfig.Instance.ProviderFactory
            End Get
        End Property

#End Region

        ''' <summary>
        ''' Load in memory the providers incated in the config file
        ''' </summary>
        ''' 
        Private Sub LoadProviders()

            Dim configProviders As List(Of LuceneProviderConfig) = LuceneSearchConfig.Instance.Providers

            If configProviders.Count = 0 Then
                LuceneSearchConfig.Reset()
                configProviders = LuceneSearchConfig.Instance.Providers
            End If

            ' on ne va rajouter que les providers qui sont activs
            For Each objProviderConfig As LuceneProviderConfig In LuceneSearchConfig.Instance.Providers.Where(Function(lpc) lpc.Enabled)
                Try
                    If TypeOf objProviderConfig Is LuceneModuleProviderConfig Then
                        Dim moduleProvider As LuceneModuleProviderConfig = DirectCast(objProviderConfig, LuceneModuleProviderConfig)
                        If Not NukeHelper.IsModuleInstalled(moduleProvider.ModuleName) Then
                            Continue For
                        End If
                    End If
                    _ProvidersByName(objProviderConfig.Name) = objProviderConfig
                    Dim objProvider As Object = objProviderConfig.Provider
                    If TypeOf objProvider Is ILuceneFieldGlossary Then
                        _Glossaries.Add(objProviderConfig.Name)
                    End If
                    If TypeOf objProvider Is ILuceneStandAloneProgressiveSearchable Then
                        _ProgressiveStandalones.Add(objProviderConfig)
                        _Upgraders(objProviderConfig.Name) = objProviderConfig.Name 'weird way around
                    ElseIf TypeOf objProvider Is ILuceneStandAloneSearchable Then
                        _StandAlones.Add(objProviderConfig.Name)
                    End If

                    ' rajoutons l'ensemble des providers qui veulent tre avertis des vnements de l'upgrade
                    If (TypeOf objProvider Is IUpgradeEventsHandlerProvider) Then
                        _IUpgradeEventHandlerProviders.Add(DirectCast(objProvider, IUpgradeEventsHandlerProvider))
                    End If

                    ' rajoutons l'ensemble des providers qui sont reflexifs
                    If (TypeOf objProvider Is ILuceneReflexiveProvider) Then
                        _IReflexiveProviders.Add(DirectCast(objProvider, ILuceneReflexiveProvider))
                    End If

                    If TypeOf objProviderConfig Is LuceneModuleProviderConfig Then
                        Dim moduleProvider As LuceneModuleProviderConfig = DirectCast(objProviderConfig, LuceneModuleProviderConfig)
                        If TypeOf objProvider Is ISearchable Then
                            _ModuleSearchables(moduleProvider.ModuleName) = objProviderConfig.Name
                        End If
                        If TypeOf objProvider Is ILuceneSearchableUpgrade Then
                            _Upgraders(moduleProvider.ModuleName) = objProviderConfig.Name
                        End If
                    End If
                Catch ex As Exception
                    'DotNetNuke.Services.Exceptions.LogException(ex)
                    'SAMY: 16/05/2012: supprim pour viter de logguer des informations trop verbeuses
                    'TODO: revenir faire quelque chose de mieux avec un logger
                    'Jesse: 17/07/2013, je rajoute un test sur l'installation des modules, je rtablis un log ici
                    Aricie.Providers.SystemServiceProvider.Instance.LogException(ex)
                End Try
            Next
        End Sub


        Public Function GetProviderByName(ByVal providerName As String) As LuceneProviderConfig
            Dim toReturn As LuceneProviderConfig = Nothing
            If _ProvidersByName.TryGetValue(providerName, toReturn) Then
                Return toReturn
            End If
            Return Nothing
        End Function



        Public Function GetUpgraderFromModuleName(ByVal moduleName As String) As LuceneModuleProviderConfig
            Dim providerName As String = Nothing
            If _Upgraders.TryGetValue(moduleName, providerName) Then
                Return DirectCast(_ProvidersByName(providerName), LuceneModuleProviderConfig)
            End If
            Return Nothing
        End Function

        Public Function GetSearchableFromModuleName(ByVal moduleName As String) As LuceneModuleProviderConfig
            Dim providerName As String = Nothing
            If _ModuleSearchables.TryGetValue(moduleName, providerName) Then
                Return DirectCast(_ProvidersByName(providerName), LuceneModuleProviderConfig)
            End If
            Return Nothing
        End Function

        Public Function GetSearchables() As List(Of LuceneModuleProviderConfig)
            Dim toReturn As New List(Of LuceneModuleProviderConfig)
            For Each providerName As String In _ModuleSearchables.Values
                toReturn.Add(DirectCast(_ProvidersByName(providerName), LuceneModuleProviderConfig))
            Next
            Return toReturn
        End Function

        Public Function GetProgressiveStandAlonesProviders() As List(Of LuceneProviderConfig)
            Return _ProgressiveStandalones
        End Function

        Public Function GetUpgradeEventHandlerProviders() As List(Of IUpgradeEventsHandlerProvider)
            Return _IUpgradeEventHandlerProviders
        End Function

        Public ReadOnly Property ReflexiveProviders() As IEnumerable(Of ILuceneReflexiveProvider)
            Get
                ' permet de renvoyer un IEnumerable immutable
                Return _IReflexiveProviders.Skip(0)
            End Get
        End Property


        Public Function GetStandAlonesProviders() As List(Of LuceneProviderConfig)
            Dim toReturn As New List(Of LuceneProviderConfig)
            For Each providerName As String In _StandAlones
                toReturn.Add(_ProvidersByName(providerName))
            Next
            Return toReturn
        End Function

        Public Function GetGlossaries() As List(Of LuceneProviderConfig)
            Dim toReturn As New List(Of LuceneProviderConfig)
            For Each providerName As String In _Glossaries
                toReturn.Add(_ProvidersByName(providerName))
            Next
            Return toReturn
        End Function


    End Class
End Namespace


