Imports System.ComponentModel
Imports Aricie.DNN.UI.Attributes
Imports Aricie.DNN.Modules.LuceneSearch.Business
Imports DotNetNuke.UI.WebControls
Imports System.IO
Imports DotNetNuke.Entities.Portals
Imports Aricie.DNN.Modules.LuceneSearch.SiteMap
Imports Aricie.DNN.Settings
Imports Aricie.Collections
Imports Aricie.DNN.UI.WebControls.EditControls
Imports System.Linq

Namespace Aricie.DNN.Modules.LuceneSearch.Settings

    <Serializable()> _
    Public Class LucenePortalSettings

#Region "Private members"

        Private _SynonymeMapPath As String = ""

#End Region

#Region "Public Properties"


#Region "IndexingSettings"

        <ExtendedCategory("IndexingSettings")> _
        <Required(True), _
        Editor(GetType(CustomTextEditControl), GetType(EditControl)), _
        Width(400)> _
        Public Property IndexFolderMapPath() As String = Common.Constants.Index.LUCENEINDEXFILE

        <ExtendedCategory("IndexingSettings")> _
        <Editor(GetType(MultiSelectorEditControl), GetType(EditControl)), MultiSelectorType("Aricie.DNN.Modules.LuceneSearch.UI.Controls.IndexedModuleMultiSelector, Aricie.DNN.Modules.LuceneSearch"), TextField("FriendlyName"), ValueField("ModuleName", TypeCode.String)> _
        Public Property IndexedModuleNames() As New List(Of String)

        <ExtendedCategory("IndexingSettings")> _
       <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property SetPortalLanguageAsDefault() As Boolean = False

        <ExtendedCategory("IndexingSettings")> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property UseSnowBallAnalysers() As Boolean = False

        <ExtendedCategory("IndexingSettings")> _
        <ConditionalVisible("ContentTaxonomyIndexationActivated", False, True, True)> _
        Public Property EnableContentTaxonomyIndexation As Boolean = False

        <ExtendedCategory("IndexingSettings")> _
        <ConditionalVisible("ReflexiveDocumentProviderActivated", False, True, True)> _
        Public Property EnableReflexiveDocumentIndexation As Boolean = False

        <ExtendedCategory("IndexingSettings")> _
        <ConditionalVisible("ReflexiveDocumentsIndexedExtensionsVisible", False, True, True)> _
        <Width(250)> _
        Public Property ReflexiveDocumentsIndexedExtensions() As String = "pdf;doc;docx;xls;xlsx;chm;txt"

        <ExtendedCategory("IndexingSettings")> _
        <ConditionalVisible("ReflexiveDocumentsIndexedExtensionsVisible", False, True, True)> _
        <Width(250)> _
        Public Property ReflexiveDocumentsIndexingTimeWindow() As Integer = 30

        <ExtendedCategory("IndexingSettings")> _
        Public Property EnabledIndexingDocuments() As Boolean = False

        <ExtendedCategory("IndexingSettings")> _
        <ConditionalVisible("EnabledIndexingDocuments", False, True, True)> _
        <Width(250)> _
        Public Property IndexedDocExtensions() As String = "pdf;doc;docx;xls;xlsx;chm;txt"

        <ExtendedCategory("IndexingSettings")> _
        <ConditionalVisible("EnabledIndexingDocuments", False, True, True)> _
        <Editor(GetType(MultiSelectorEditControl), GetType(EditControl)), MultiSelectorType("Aricie.DNN.Modules.LuceneSearch.UI.Controls.IndexedFoldersMultiSelector, Aricie.DNN.Modules.LuceneSearch"), TextField("FolderPath"), ValueField("FolderID", TypeCode.String)> _
        Public Property IndexedFolderIdList() As New List(Of String)

        ''Helpers visibilit

        <System.Xml.Serialization.XmlIgnore()> _
       <Browsable(False)> _
        Public ReadOnly Property ContentTaxonomyIndexationActivated As Boolean
            Get
                Return ModuleProviders.ProviderFactory.Instance.GetProgressiveStandAlonesProviders().Any(Function(ilrp) ilrp.Name = "StandAlone Taxonomy")
            End Get
        End Property

        <System.Xml.Serialization.XmlIgnore()> _
       <Browsable(False)> _
        Public ReadOnly Property ReflexiveDocumentProviderActivated As Boolean
            Get
                Return ModuleProviders.ProviderFactory.Instance.ReflexiveProviders.Any(Function(ilrp) ilrp.GetType.Name = "ReflexiveLinkedDocumentProvider")
            End Get
        End Property

        <System.Xml.Serialization.XmlIgnore()> _
        <Browsable(False)> _
        Public ReadOnly Property ReflexiveDocumentsIndexedExtensionsVisible As Boolean
            Get
                Return ReflexiveDocumentProviderActivated AndAlso EnableReflexiveDocumentIndexation
            End Get
        End Property

        <System.Xml.Serialization.XmlIgnore()> _
        <Browsable(False)> _
        Public ReadOnly Property IndexedDocExtensionsList() As List(Of String)
            Get
                Return New List(Of String)(IndexedDocExtensions.Split(New String() {";"}, StringSplitOptions.RemoveEmptyEntries))
            End Get
        End Property

        <System.Xml.Serialization.XmlIgnore()> _
        <Browsable(False)> _
        Public ReadOnly Property ReflexiveDocumentsIndexedExtensionsList() As IList(Of String)
            Get
                Return ReflexiveDocumentsIndexedExtensions.ToLowerInvariant.Split(New String() {";"}, StringSplitOptions.RemoveEmptyEntries)
            End Get
        End Property

#End Region

#Region "Storage and upgrade settings"
        <ExtendedCategory("StorageAndUpgrade")> _
        <RegularExpressionValidator("\d{1,}")> _
        <Required(True)> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property NbSecondsUpgrading() As Integer = 30

        <ExtendedCategory("StorageAndUpgrade")> _
        <RegularExpressionValidator("\d{1,}")> _
        <Required(True)> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property NbSecondsIndexing() As Integer = 30

        <ExtendedCategory("StorageAndUpgrade")> _
        <RegularExpressionValidator("\d{1,}")> _
        <Required(True)> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property MaxNbDocPerCommit() As Integer = 30

        <ExtendedCategory("StorageAndUpgrade")> _
        <Editor(GetType(PropertyEditorEditControl), GetType(EditControl))> _
        <LabelMode(LabelMode.Top)> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property MaxIndexedAge() As STimeSpan = New STimeSpan(TimeSpan.FromDays(10))

        <ExtendedCategory("StorageAndUpgrade")> _
        <RegularExpressionValidator("\d{1,}")> _
        <Required(True)> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property MinDocTurnOver() As Integer = 50

        <ExtendedCategory("StorageAndUpgrade")> _
        <RegularExpressionValidator("\d{1,}")> _
        <Required(True)> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property OldDocIndexingRatio() As Integer = 5

        <ExtendedCategory("StorageAndUpgrade")> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
Public Property TrustPubDates() As Boolean = False
#End Region



#Region "QuerySettings"

        <ExtendedCategory("QuerySettings")> _
        Public Property UseSharedReader() As Boolean = True

        <ExtendedCategory("QuerySettings")> _
        <Editor(GetType(MultiSelectorEditControl), GetType(EditControl)), MultiSelectorType("Aricie.DNN.Modules.LuceneSearch.UI.Controls.SearchableModuleMultiSelector, Aricie.DNN.Modules.LuceneSearch"), TextField("FriendlyName"), ValueField("ModuleName", TypeCode.String)> _
        Public Property SearchableModuleNames() As New List(Of String)

        <ExtendedCategory("QuerySettings")> _
        <Editor(GetType(MultiSelectorEditControl), GetType(EditControl)), MultiSelectorType("Aricie.DNN.Modules.LuceneSearch.UI.Controls.LuceneFieldMultiSelector, Aricie.DNN.Modules.LuceneSearch"), TextField("FriendlyName"), ValueField("Id", TypeCode.Int32)> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property DefaultSearchFields() As New List(Of String)

        <ExtendedCategory("QuerySettings")> _
        <Required(False), _
        Editor(GetType(CustomTextEditControl), GetType(EditControl)), _
        Width(400)> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property SynonymeMapPath() As String
            Get
                If Not String.IsNullOrEmpty(_SynonymeMapPath) Then
                    Return Me._SynonymeMapPath
                Else
                    Return Me._IndexFolderMapPath + "\Synonyms.xml"
                End If
            End Get
            Set(ByVal value As String)
                Me._SynonymeMapPath = value
            End Set
        End Property

#End Region

#Region "BackupSettings"

        <ExtendedCategory("BackupSettings")> _
        <Required(False), _
        Editor(GetType(CustomTextEditControl), GetType(EditControl)), _
        Width(400)> _
        <Browsable(False)> _
        Public Property BackupsRootFolderMapPath() As String = Common.Constants.Index.LUCENEINDEXBACKUP
            
        <ExtendedCategory("BackupSettings")> _
        <Browsable(False)> _
        Public Property BackupFrequencies() As New SerializableDictionary(Of TimeSpan, Integer)

        <ExtendedCategory("BackupSettings"), RegularExpressionValidator("\d{1,}")> _
        <Browsable(False)> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property AlertDocChangeRate() As Integer = 100
            
        <ExtendedCategory("BackupSettings")> _
        <Browsable(False)> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property AlertEmail() As String = String.Empty

        <ExtendedCategory("BackupSettings")> _
        <Browsable(False)> _
        <RegularExpressionValidator("\d{1,}")> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property RejectDocChangeRate() As Integer = 100

#End Region

#Region "Autocomplete"
        <ExtendedCategory("AutoComplete")> _
        Public Property Autocomplete As New AutoComplete
#End Region

#Region "GoogleSiteMapSettings"

        <ExtendedCategory("GoogleSiteMapSettings"), RegularExpressionValidator("\d{1,}")> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property DefaultPriorityPercentage() As Integer = 70
            
        <ExtendedCategory("GoogleSiteMapSettings")> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property ComputePriority() As Boolean = true
            
        <ExtendedCategory("GoogleSiteMapSettings")> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property ComputeChangeFrequency() As Boolean = True

        <ExtendedCategory("GoogleSiteMapSettings")> _
        <Browsable(False)> _
        Public Property DefaultChangeFrequency() As ChangeFrequency = ChangeFrequency.Daily

        ''Helpers

        <ExtendedCategory("GoogleSiteMapSettings")> _
        <Browsable(False)> _
        Public ReadOnly Property DefaultPriority() As Decimal
            Get
                Return Decimal.Round(Convert.ToDecimal(DefaultPriorityPercentage / 100), 1)
            End Get
        End Property

#End Region

#Region "LogsSettings"

        <ExtendedCategory("LogsSettings")> _
        Public Property LogIndexingProcess() As Boolean = False

        <ExtendedCategory("LogsSettings")> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property EnableFilterLogging() As Boolean = False

        <ExtendedCategory("LogsSettings")> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property PurgeLogNbDays() As Integer = 15

#End Region

#End Region

#Region "Public methods"

        Public Function IsAvailableForIndexing(ByVal moduleName As String) As Boolean
            Return Not Me.IndexedModuleNames.Count = 0 And Me.IndexedModuleNames.Contains(moduleName)
        End Function

        Public Function IsAvailableForSearching(ByVal moduleName As String) As Boolean
            Return Not Me.SearchableModuleNames.Count = 0 And Me.SearchableModuleNames.Contains(moduleName)
        End Function

        Public Function GetNewBackupMapPath(ByVal backupPeriod As TimeSpan) As String
            Return Path.Combine(Me._BackupsRootFolderMapPath, Me.GetNewBackupRelativePath(backupPeriod))
        End Function

        Public Function GetIndexFolderMapPath(ByVal portalId As Integer) As String
            Return GetFullPathFromDirectory(portalId, IndexFolderMapPath)
        End Function

        Public Function GetBackupsRootFolderMapPath(ByVal portalId As Integer) As String
            Return GetFullPathFromDirectory(portalId, BackupsRootFolderMapPath)
        End Function

        Private Function GetFullPathFromDirectory(portalId As Integer, Path As String) As String
            Dim HomeDirectoryMapPath = Aricie.DNN.Services.NukeHelper.PortalInfo(portalId).HomeDirectoryMapPath
            Dim toReturn As New StringBuilder
            If Not String.IsNullOrEmpty(HomeDirectoryMapPath) AndAlso Not String.IsNullOrEmpty(Path) Then
                If Not Path.Contains(":") Then
                    toReturn.Append(HomeDirectoryMapPath)
                End If
                toReturn.Append(Path)
            End If
            Return toReturn.ToString
        End Function

#End Region

#Region "Private methods"

        Friend Shared Function GetCommonBackupRelativePath(ByVal backupPeriod As TimeSpan) As String

            Dim toReturn As String = Common.Constants.Index.LUCENEINDEXBACKUP
            toReturn &= "."c & backupPeriod.Days & "D"
            If backupPeriod.Hours > 0 Then
                toReturn &= backupPeriod.Hours.ToString & "H"c
            End If

            Return toReturn

        End Function

        'Cette fonction cre un dossier de sauvegarde journalier pour la date d'aujourd'hui et en renvoie le chemin
        Private Function GetNewBackupRelativePath(ByVal backupPeriod As TimeSpan) As String

            Dim toReturn As String = GetCommonBackupRelativePath(backupPeriod)
            toReturn &= DateTime.Now.ToString("YYMMDD-HHmmss")

            Return toReturn

        End Function

#End Region

    End Class

End Namespace
