Imports System.ComponentModel
Imports Aricie.DNN.UI.Attributes
Imports Aricie.DNN.UI.WebControls.EditControls
Imports DotNetNuke.UI.WebControls
Imports System.Web.UI.WebControls
Imports System.Xml.Serialization
Imports Lucene.Net.Search.Highlight

Namespace Aricie.DNN.Modules.LuceneSearch.Settings

    Public Enum PermissionFilter
        None
        'FilterResults nevermore
        FilterQuery
    End Enum

    Public Enum FilterControl
        'None
        CommonFilter
        LightFilter
        DateFilter
        AdvancedFilter
        CustomFilter
    End Enum

    Public Enum ResultsControlMode
        TemplateView
        GridView
        CustomView
    End Enum

    Public Enum PagerButtonsStyle
        NextPrevious = 0                'Render : < >
        Numeric = 1                     'Render : 1 2 3 4 5
        NextPreviousFirstLast = 2       'Render : << < > >>
        NumericFirstLast = 3            'Render : << < 1 2 3 4 5 > >>

        TextualNextPrevious = 4                 'Render : Previous Page   Next Page
        TextualNumeric = 5                      'Render : Page 1   Page 2  Page 3
        TextualNextPreviousFirstLast = 6        'Render : First Page   Previous Page    Next Page    Last Page
        TextualNumericFirstLast = 7             'Render : First Page   Page 1  Page 2  Page 3  Last Page
        TextualNumericNextPrevious = 8          'Render : Previous Page  Page 1 Page 2  Next Page
        TextualNumericNextPreviousFirstLast = 9 'Render : First Page  Previous Page  Page 1  Page 2  Page 3  Next Page  Last Page
    End Enum

    <Serializable()> _
    Public Class LuceneResultsSettings

#Region "Private members"

        Private _FilterToLoad As String = String.Empty
        'Private _ResultsControlPath As String = String.Empty

#End Region

#Region "Public properties"

#Region "Interface de la recherche"

        <ExtendedCategory("Searching")> _
        Public Property DisplaySearchPanel As Boolean = True

        <ExtendedCategory("Searching")> _
        <ConditionalVisible("DisplaySearchPanel", False, True, True)> _
        <AutoPostBack()> _
        Public Property ShowAdvancedFilters() As Boolean = True

        <ExtendedCategory("Searching")> _
        <ConditionalVisible("FiltersDisplayed", False, True, True)> _
        Public Property FilterControl() As FilterControl = Settings.FilterControl.CommonFilter


        <ExtendedCategory("Searching")> _
        <ConditionalVisible("FilterControl", False, True, FilterControl.CustomFilter)> _
        <Editor(GetType(CustomTextEditControl), GetType(EditControl))> _
        <Width(350)> _
        Public Property FilterToLoad() As String
            Get
                Select Case FilterControl
                    Case FilterControl.CommonFilter
                        _FilterToLoad = "./Controls/AdvancedFilters/CommonFilter.ascx"
                    Case FilterControl.LightFilter
                        _FilterToLoad = "./Controls/AdvancedFilters/LightFilter.ascx"
                    Case FilterControl.AdvancedFilter
                        _FilterToLoad = "./Controls/AdvancedFilters/AdvancedCriteriaFilter.ascx"
                    Case FilterControl.DateFilter
                        _FilterToLoad = "./Controls/AdvancedFilters/DateFilter.ascx"
                    Case FilterControl.CustomFilter
                        ' on ne change pas la valeur de l'utilisateur
                    Case Else
                        _FilterToLoad = String.Empty
                End Select
                Return _FilterToLoad
            End Get
            Set(ByVal value As String)
                If FilterControl = FilterControl.CustomFilter Then
                    _FilterToLoad = value
                End If
            End Set
        End Property

        <ExtendedCategory("Searching")> _
        <ConditionalVisible("DisplaySearchPanel", False, True)> _
        Public Property ShowSearchButton() As Boolean = True

        <ExtendedCategory("Searching")> _
        <ConditionalVisible("DisplaySearchPanel", False, True)> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property ShowResetButton() As Boolean = False

        <ExtendedCategory("Searching")> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        <ConditionalVisible("DisplaySearchPanel", False, True)> _
        Public Property AllowMinimizeAdvancedSearch() As Boolean = False

#Region "Display Helpers"

        <Browsable(False)> _
        <XmlIgnore()> _
        Public ReadOnly Property SearchPanelDisplayed As Boolean
            Get
                Return DisplaySearchPanel
            End Get
        End Property

        <Browsable(False)> _
        <XmlIgnore()> _
        Public ReadOnly Property FiltersDisplayed As Boolean
            Get
                Return DisplaySearchPanel AndAlso ShowAdvancedFilters
            End Get
        End Property

        <Browsable(False)> _
        <XmlIgnore()> _
        Public ReadOnly Property SearchButtonDisplayed As Boolean
            Get
                Return DisplaySearchPanel AndAlso ShowSearchButton
            End Get
        End Property

        <Browsable(False)> _
        <XmlIgnore()> _
        Public ReadOnly Property ResetButtonDisplayed As Boolean
            Get
                Return DisplaySearchPanel AndAlso ShowResetButton
            End Get
        End Property

        <Browsable(False)> _
        <XmlIgnore()> _
        Public ReadOnly Property SearchMinificationDisplayed As Boolean
            Get
                Return DisplaySearchPanel AndAlso AllowMinimizeAdvancedSearch
            End Get
        End Property

#End Region
#End Region

#Region "Comportement de la recherche"
        <ExtendedCategory("SearchingBehaviour")> _
        Public Property EnableAjax() As Boolean = True

        <ExtendedCategory("SearchingBehaviour")> _
        <Required(True), Editor(GetType(DNNPageEditControl), GetType(EditControl))> _
        <ConditionalVisible("EnableAjax", True, True)> _
        Public Property ResultsTabId() As Integer = -1

        <ExtendedCategory("SearchingBehaviour")> _
        Public Property EnableWarningsFromUrlFiltering As Boolean = True

        <ExtendedCategory("SearchingBehaviour")> _
        Public Property ApplyDefaultLanguageFilter() As Boolean = False

        <ExtendedCategory("SearchingBehaviour")> _
        <Editor(GetType(CustomTextEditControl), GetType(EditControl)), LineCount(3), Width(400)> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property AdditionalFilter() As String = String.Empty

        <ExtendedCategory("SearchingBehaviour")> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property EnableAdditionalFilterTokenReplace() As Boolean = False

        <ExtendedCategory("SearchingBehaviour")> _
        Public Property PermissionFilter() As PermissionFilter = Settings.PermissionFilter.FilterQuery

        ' Modification de l'diteur pour afficher les mmes champs que le portail
        <Editor(GetType(MultiSelectorEditControl), GetType(EditControl)), MultiSelectorType("Aricie.DNN.Modules.LuceneSearch.UI.Controls.LuceneFieldMultiSelector, Aricie.DNN.Modules.LuceneSearch"), TextField("FriendlyName"), ValueField("Id", TypeCode.Int32), Width(500)> _
        <ExtendedCategory("SearchingBehaviour")> _
        Public Property SearchFields() As New List(Of String)

        <Required(True), RegularExpressionValidator("100|\d{1,2}"), ExtendedCategory("SearchingBehaviour")> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property DefaultFuzzyValue() As Integer = 100
#End Region

#Region "CriteriaSettings"

        <ExtendedCategory("CriteriaSettings")> _
        <ConditionalVisible("FilterControl", False, True, FilterControl.AdvancedFilter)> _
        Public Property EnableCustomFieldCriteria() As Boolean = True

        <ExtendedCategory("CriteriaSettings")> _
        <ConditionalVisible("FilterControl", False, True, FilterControl.AdvancedFilter)> _
        Public Property EnableCustomOccurCriteria() As Boolean = True

        <ExtendedCategory("CriteriaSettings")> _
        <ConditionalVisible("FilterControl", False, True, FilterControl.AdvancedFilter)> _
        Public Property EnableCustomFuzzyCriteria() As Boolean = False

        <ExtendedCategory("CriteriaSettings")> _
        <ConditionalVisible("FilterControl", False, True, FilterControl.AdvancedFilter)> _
        Public Property EnableFreeLuceneQuery() As Boolean = False

        <ExtendedCategory("CriteriaSettings")> _
        <ConditionalVisible("FilterControl", False, True, FilterControl.AdvancedFilter)> _
        Public Property ShowPhraseQuery() As Boolean = True

        <ExtendedCategory("CriteriaSettings")> _
        <ConditionalVisible("FilterControl", False, True, FilterControl.AdvancedFilter)> _
        Public Property ShowBooleanShould() As Boolean = True

        <ExtendedCategory("CriteriaSettings")> _
        <ConditionalVisible("FilterControl", False, True, FilterControl.AdvancedFilter)> _
        Public Property ShowBooleanMust() As Boolean = True

        <Required(True), RegularExpressionValidator("\d{1,}"), ExtendedCategory("CriteriaSettings")> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        <ConditionalVisible("FilterControl", False, True, FilterControl.AdvancedFilter)> _
        Public Property DefaultSlopValue() As Integer = 0
#End Region

#Region "Interface rsultats"
        <ExtendedCategory("Results")> _
        Public Property ShowResults() As Boolean = True

        <ExtendedCategory("Results")> _
       <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property DisplayBackButton() As Boolean = False

        <ExtendedCategory("Results")> _
        <Editor(GetType(CustomTextEditControl), GetType(EditControl)), LineCount(3), Width(250)> _
          <ConditionalVisible("DisplayBackButton", False, True, True)> _
        Public Property BackButtonTemplate() As String = "<div class=""LS_BackLink""><a href=""[BACK_LINK_URL]"" >[BACK_LINK_LIB]</a></div>"

        <ExtendedCategory("Results")> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property AllowMinimizeResults() As Boolean = False

        <ExtendedCategory("Results")> _
        Public Property DisplayNbResults() As Boolean = False

        <ExtendedCategory("Results")> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        <Browsable(False)> _
        Public Property ShowCurrentFiltersRow() As Boolean = False

        <ExtendedCategory("Results")> _
        Public Property EnablePaging() As Boolean = True

        <ExtendedCategory("Results")> _
        <ConditionalVisible("EnablePaging", True, True)> _
        <AutoPostBack()> _
        Public Property LimitResults() As Boolean = True

        <Browsable(False)> _
        Public ReadOnly Property NoPagingAndLimited As Boolean
            Get
                Return Not EnablePaging AndAlso LimitResults
            End Get
        End Property

        <ExtendedCategory("Results")> _
        <Required(True), RegularExpressionValidator("\d{1,}")> _
        <ConditionalVisible("NoPagingAndLimited", False, True)> _
        Public Property LimitResultsTo() As Integer = 5

        <ExtendedCategory("Results")> _
        <ConditionalVisible("EnablePaging", False, True)> _
        Public Property DisplayNbPagesResults() As Boolean = False

        <ExtendedCategory("Results")> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        <ConditionalVisible("EnablePaging", False, True)> _
        Public Property PagerButtons() As PagerButtonsStyle = PagerButtonsStyle.NumericFirstLast

        <Required(True), RegularExpressionValidator("\d{1,}"), ExtendedCategory("Results")> _
        <ConditionalVisible("EnablePaging", False, True)> _
        Public Property DefaultPageSize() As Integer = 5

        <ExtendedCategory("Results")> _
        <ConditionalVisible("EnablePaging", False, True)> _
        Public Property AllowCustomPageSize() As Boolean = False

        <ExtendedCategory("Results")> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property AllowCustomGrouping() As Boolean = False

        <ExtendedCategory("Results")> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property AllowCustomSorting() As Boolean = False

        <ExtendedCategory("Results")> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property AllowPrint() As Boolean = False

        <ConditionalVisible("AllowPrint", False, True, True)> _
        <ExtendedCategory("Results")> _
        Public Property DisableSkinForPrintResult() As Boolean = False

        <ConditionalVisible("AllowPrint", False, True, True)> _
        <ExtendedCategory("Results")> _
        Public Property DisablePagingForPrint() As Boolean = False
#End Region

#Region "Comportement rsultats"
        <ExtendedCategory("ResultsBehaviour")> _
        Public Property DisplayResultsOnlyWhereSearch() As Boolean = True

        <ExtendedCategory("ResultsBehaviour")> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property RedirectIfOnlyOneResult() As Boolean = False

        <ExtendedCategory("ResultsBehaviour")> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property DefaultGroupingFieldName() As String = String.Empty

        <ExtendedCategory("ResultsBehaviour")> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property DefaultReverseGroupingOrder() As Boolean = False

        <ExtendedCategory("ResultsBehaviour")> _
        Public Property DefaultSortingFieldName() As String = String.Empty

        <ExtendedCategory("ResultsBehaviour")> _
        Public Property DefaultReverseSortOrder() As Boolean = False

        <Required(True), RegularExpressionValidator("\d{1,}"), ExtendedCategory("ResultsBehaviour")> _
       <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property NbSecondsCacheResults() As Integer = 40

#End Region

#Region "Results Layout"

        <Editor(GetType(UserControlEditControl), GetType(EditControl))> _
        <ExtendedCategory("ResultsLayout")> _
        <UserControl("~/DesktopModules/Aricie.LuceneSearch/Controls/Settings/ModuleSettings/Templates/TemplateEdition.ascx")> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property TemplateConfiguration As New Templates


#End Region

#Region "Highlights Settings"

        <ExtendedCategory("ResultsHighlights")> _
        Public Property EnableHighlighter() As Boolean = True

        <Required(True)> _
        <RegularExpressionValidator("\d{1,}")> _
        <ExtendedCategory("ResultsHighlights")> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property HighlighterMaxNbFragments() As Integer = 4

        <Required(True)> _
        <RegularExpressionValidator("\d{1,}")> _
        <ExtendedCategory("ResultsHighlights")> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property HighlighterFragmentsMaxNbTokens() As Integer = 50

        <Required(True)> _
        <ExtendedCategory("ResultsHighlights")> _
            <Editor(GetType(CustomTextEditControl), GetType(EditControl))> _
            <LineCount(4)> _
            <Width(300)> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property HighlighterFragmentFormat() As String = "&lt;strong &gt;[FieldName]: &lt;/strong &gt;...[Fragment]..."

        <Required(True)> _
        <ExtendedCategory("ResultsHighlights")> _
        <Editor(GetType(CustomTextEditControl), GetType(EditControl))> _
        <LineCount(2)> _
        <Width(200)> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property HighlighterFragmentsSeparator() As String = "&lt;br/&gt;"

        <Required(True)> _
        <ExtendedCategory("ResultsHighlights")> _
        <Editor(GetType(CustomTextEditControl), GetType(EditControl))> _
        <LineCount(2)> _
        <Width(200)> _
        <TrialLimited(DNN.Security.Trial.TrialPropertyMode.Disable)> _
        Public Property HighlighterMatchTokenFormat() As String = "&lt;em &gt;[Content]&lt;/em &gt;"

#End Region

#End Region

#Region "Public methods"

        Public Function GetFormatter() As IFormatter
            Dim format As String() = HighlighterMatchTokenFormat.Split(New String() {"[Content]"}, StringSplitOptions.RemoveEmptyEntries)
            Return New SimpleHTMLFormatter(format(0), format(1))
        End Function

#End Region

    End Class

End Namespace

