Imports Aricie.DNN.Modules.LuceneSearch.Business
Imports DotNetNuke.Services.Localization
Imports Lucene.Net.Index
Imports Lucene.Net.Search
Imports DotNetNuke.Entities.Modules

Namespace Aricie.DNN.Modules.LuceneSearch.UI
    Partial Public Class DesktopModuleFilter
        Inherits LuceneFilterControlBase

#Region "Properties"

        Public Overrides Property Enabled() As Boolean
            Get
                Return Me.chkAllModules.Enabled And Me.ctlDesktopModules.Enabled
            End Get
            Set(ByVal value As Boolean)
                Me.chkAllModules.Enabled = value
                Me.ctlDesktopModules.Enabled = value
            End Set
        End Property

#End Region




#Region "Event Handlers"

        Protected Sub Page_Init(ByVal sender As Object, ByVal e As EventArgs) Handles Me.Init
            If Not Me.IsPostBack Then
                Me.BindLayout()
            End If
        End Sub

        Protected Sub chkAllModules_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) _
            Handles chkAllModules.CheckedChanged

            OnCheckedChanged()

        End Sub


#End Region


#Region "ILuceneFilterControl implements"

        Public Overrides Function GetFilter() As LuceneFilterQuery
            Dim toReturn As New LuceneFilterQuery
            Dim domainFilters As BooleanQuery = Nothing
            If Not Me.chkAllModules.Checked Then
                domainFilters = New BooleanQuery()

                For Each locItem As ListItem In Me.ctlDesktopModules.Items

                    If locItem.Selected Then
                        Dim domainValue As String = BusinessController.FormatSearchCriteria(locItem.Value.ToString)
                        Dim domainTerm As New Term(DnnSearchItemField.ModuleName, domainValue)
                        domainFilters.Add(New TermQuery(domainTerm), Occur.SHOULD)
                    End If

                Next
                toReturn.FilterQuery = domainFilters
            End If
            Return toReturn

        End Function

        Public Overrides Function HasFilter() As Boolean
            Return Not Me.chkAllModules.Checked AndAlso Me.ctlDesktopModules.SelectedIndex <> -1
        End Function

        Public Overrides Function GetFieldNames() As IEnumerable(Of String)
            Return New String() {DnnSearchItemField.ModuleName}

        End Function

        Public Overrides Function ParseFilter(ByVal objFilter As LuceneFilterQuery) As Boolean
            Return ParseTermQuery(objFilter.FilterQuery)
        End Function

        Private Function ParseTermQuery(q As Query, Optional firstCall As Boolean = True) As Boolean
            Dim Parsed As Boolean = False
            If TypeOf q Is TermQuery Then
                Dim CurrentTerm As Term = DirectCast(q, TermQuery).Term
                If CurrentTerm.Field = DnnSearchItemField.ModuleName Then
                    Dim domainValue As String = CurrentTerm.Text
                    For Each locItem As ListItem In Me.ctlDesktopModules.Items
                        If locItem.Value = domainValue Then
                            Parsed = True
                            locItem.Selected = True
                        End If
                    Next
                End If

            ElseIf TypeOf q Is BooleanQuery Then
                For Each c As BooleanClause In DirectCast(q, BooleanQuery).GetClauses()
                    Parsed = ParseTermQuery(c.Query, False) OrElse Parsed
                Next
            End If

            If Parsed AndAlso firstCall Then
                chkAllModules.Checked = False
                OnCheckedChanged()
            End If

            Return Parsed
        End Function

#End Region

#Region "Private methods"


        Private Sub OnCheckedChanged()
            If Me.chkAllModules.Checked Then
                Me.divDomains.Visible = False
            Else
                Me.divDomains.Visible = True
            End If
        End Sub

        Private Sub BindLayout()


            Dim values As ArrayList = New ArrayList()


            'On get tous les desktopmodule Isearchable:
            Dim _
                searchableModules As IList(Of DesktopModuleInfo) = _
                    LuceneIndexer.GetSearchableDesktopModules(Me.PortalSettings.PortalId)
            For Each myDkModule As DesktopModuleInfo In searchableModules
                'seulement ceux qui ont un provider d'indexation lucene
                'If Not (myProviderFactory.getProviderFromModuleName(myDkModule.ModuleName) Is Nothing) Then
                'Seulement ceux qui sont slectionns pour etre searchable
                'If (PortalSettings.GetSiteSetting(Me.PortalId, myDkModule.ModuleName) = Constants.MODULEAVAILABLEFORSEARCH) Then 'Lucene.SearchProvider.
                If PortalIndexingDriver.GetCurrent.LucenePortalSettings.IsAvailableForSearching(myDkModule.ModuleName) _
                    Then

                    Dim locItemtext As String = Localization.GetString(myDkModule.ModuleName, Me.LocalResourceFile)
                    Dim newItem As ListItem = New ListItem()
                    newItem.Value = myDkModule.ModuleName
                    If String.IsNullOrEmpty(locItemtext) OrElse locItemtext.ToLower.StartsWith("resx:") Then
                        newItem.Text = myDkModule.FriendlyName
                    Else
                        newItem.Text = locItemtext
                    End If
                    values.Add(newItem)
                End If
                'End If
            Next

            Me.ctlDesktopModules.DataSource = values
            Me.ctlDesktopModules.DataBind()

            'On efface le choix de domaines s'il n'y en a qu'un        

            If values.Count = 0 Then

                Me.chkAllModules.Checked = True
                Me.divDomains.Visible = False
            ElseIf values.Count = 1 Then
                Me.chkAllModules.Checked = False
                Me.Visible = False
                For Each myItem As ListItem In Me.ctlDesktopModules.Items
                    myItem.Selected = True
                    'noter qu'il n'y en a qu'un!
                Next
            End If

        End Sub

#End Region
    End Class
End Namespace
