Imports Aricie.DNN.Modules.LuceneSearch.Business
Imports Aricie.DNN.Modules.LuceneSearch.ModuleProviders
Imports Aricie.DNN.Modules.LuceneSearch.Settings
Imports Aricie.DNN.UI.Controls
Imports DotNetNuke.Common.Globals
Imports DotNetNuke.Services.Exceptions
Imports DotNetNuke.Services.Localization
Imports DotNetNuke.UI.Skins.Controls
Imports DotNetNuke.UI.Skins
Imports Aricie.DNN.Security.Trial
Imports Aricie.DNN.Modules.LuceneSearch.Security
Imports Aricie.Services

Namespace Aricie.DNN.Modules.LuceneSearch.UI

    Partial Public Class IndexerSettings
        Inherits AriciePortalModuleBase

#Region "Settings courant"
        Private ReadOnly Property SettingsSessionKey As String
            Get
                Return "LuceneSearch.PortalSettings." & PortalId
            End Get
        End Property

        Private Property CurrentSettings As LucenePortalSettings
            Get
                Dim obj As Object = Session(SettingsSessionKey)
                If (obj Is Nothing) Then
                    ' on le clone pour viter que les valeurs non sauvegardes soient conserves
                    obj = ReflectionHelper.CloneObject(Of LucenePortalSettings)(PortalIndexingDriver.GetCurrent.LucenePortalSettings)
                    Session(SettingsSessionKey) = obj
                End If
                Return DirectCast(obj, LucenePortalSettings)
            End Get
            Set(value As LucenePortalSettings)
                Session(SettingsSessionKey) = value
            End Set
        End Property

#End Region

#Region "Event Handlers"

        Private Sub Page_Init(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Init
            IdxSP.DefaultButton = cmdSaveAndContinue.ID
            EnforceFreeVersion()
        End Sub


        Protected Sub Page_Load(ByVal sender As Object, ByVal e As EventArgs)
            Try
                If (Not IsPostBack) Then
                    CurrentSettings = Nothing
                End If
                IdxS.LocalResourceFile = Me.LocalResourceFile
                lblFieldsSelectorError.Visible = False
                BindData()
            Catch ex As Exception
                ProcessModuleLoadException(Me, ex)
            End Try
        End Sub

        Private Sub cmdSaveAndContinue_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmdSaveAndContinue.Click
            Try
                Dim retVal As String = UpdateIndexerSettings()
                If Not retVal <> "" Then
                    lblFieldsSelectorError.Visible = True
                    lblFieldsSelectorError.Text = retVal
                End If
            Catch ex As Exception
                ProcessModuleLoadException(Me, ex)
            End Try

        End Sub

        Private Sub cmdSaveAndReturn_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmdSaveAndReturn.Click

            Try
                Dim retVal As String = UpdateIndexerSettings()
                If retVal <> "" Then
                    lblFieldsSelectorError.Visible = True
                    lblFieldsSelectorError.Text = retVal
                Else
                    Response.Redirect(NavigateURL())
                End If
            Catch ex As Exception
                ProcessModuleLoadException(Me, ex)
            End Try

        End Sub

        Protected Sub cmdClearIndex_Click(ByVal sender As Object, ByVal e As EventArgs) Handles cmdClearIndex.Click
            Try
                If PortalIndexingDriver.GetCurrent().ResetIndex() Then
                    Skin.AddModuleMessage(Me, Localization.GetString("IndexCleared.Message", Me.LocalResourceFile), ModuleMessage.ModuleMessageType.GreenSuccess)
                Else
                    Skin.AddModuleMessage(Me, Localization.GetString("IndexBusy.Message", Me.LocalResourceFile), ModuleMessage.ModuleMessageType.YellowWarning)
                End If
            Catch ex As Exception
                ProcessModuleLoadException(Me, ex)
            End Try
        End Sub

        Private Sub cmdIndex_Click(sender As Object, e As System.EventArgs) Handles cmdIndex.Click
            Dim LSD As New LuceneSearchDataStore()
            LSD.StoreSearchItems(LuceneIndexer.RetrieveSearchIndexItems(PortalId))
            Skin.AddModuleMessage(Me, Localization.GetString("", LocalResourceFile), ModuleMessage.ModuleMessageType.GreenSuccess)
        End Sub

        Private Sub cmdCancel_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmdCancel.Click
            Response.Redirect(NavigateURL())
        End Sub

#End Region

#Region "Public methods"

        Public Function UpdateIndexerSettings() As String

            Dim newPortalSettings As LucenePortalSettings = DirectCast(IdxS.DataSource, LucenePortalSettings)

            'If newPortalSettings.HomeDirectoryMapPath <> Me.PortalSettings.HomeDirectoryMapPath Then
            '    newPortalSettings.HomeDirectoryMapPath = Me.PortalSettings.HomeDirectoryMapPath
            'End If

            If newPortalSettings.IndexedModuleNames.Count = 0 Then
                Return Localization.GetString("lblIndexedModulesError.Text", Me.LocalResourceFile)
            End If

            PortalIndexingDriver.GetCurrent.LucenePortalSettings = newPortalSettings

            Return String.Empty

        End Function

#End Region

#Region "Private methods"


        Private Sub BindData()
            IdxS.DataSource = CurrentSettings
            IdxS.DataBind()

            truc.DataSource = LuceneSearchConfig.Instance
            truc.DataBind()
        End Sub


        Private Sub EnforceFreeVersion()
            Dim tc As TrialController = TrialController.Instance(New TrialProvider())
            Dim objTrialStatus As TrialStatusInfo = tc.LimitModule(Me, True)
            IdxS.TrialStatus = objTrialStatus
        End Sub

#End Region



    End Class

End Namespace
