Imports DotNetNuke.Common.Globals
Imports DotNetNuke.Entities.Modules
Imports DotNetNuke.Services.Exceptions
Imports DotNetNuke.Services.Localization
Imports Aricie.DNN.Modules.LuceneSearch.Business
Imports Aricie.DNN.Modules.LuceneSearch.Security
Imports Aricie.DNN.Modules.LuceneSearch.Settings
Imports Aricie.DNN.Security.Trial
Imports Aricie.DNN.Settings
Imports Aricie.DNN.UI.Controls
Imports Aricie.Services
Imports Configuration.Templating

Namespace Aricie.DNN.Modules.LuceneSearch.UI

    Partial Public Class ResultSettings
        Inherits AriciePortalModuleBase

#Region "Settings methods"

        Protected Overrides Sub OnInit(e As System.EventArgs)
            RSP.DefaultButton = cmdSaveAndContinue.ID
            MyBase.OnInit(e)
        End Sub

        Private ReadOnly Property SettingsSessionKey As String
            Get
                Return "LuceneSearch.ResultSettings." & PortalId & "." & ModuleId
            End Get
        End Property

        Private Property CurrentSettings As LuceneResultsSettings
            Get
                Dim obj As Object = Session(SettingsSessionKey)
                If (obj Is Nothing) Then
                    ' on le clone pour viter que les valeurs non sauvegardes soient conserves
                    obj = ReflectionHelper.CloneObject(Of LuceneResultsSettings)(PortalQueryingDriver.GetCurrent.LuceneResultsSettings(Me.ModuleId))
                    Session(SettingsSessionKey) = obj
                End If
                Return DirectCast(obj, LuceneResultsSettings)
            End Get
            Set(value As LuceneResultsSettings)
                Session(SettingsSessionKey) = value
            End Set
        End Property

#End Region

#Region "Event Handlers"

        Private Sub Page_Init(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Init
            EnforceFreeVersion()
        End Sub

        Protected Sub Page_Load(ByVal sender As Object, ByVal e As EventArgs) Handles Me.Load
            Try
                If Not IsPostBack Then
                    CurrentSettings = Nothing
                End If
                RS.LocalResourceFile = LocalResourceFile
                RS.DataSource = CurrentSettings
                RS.DataBind()

            Catch ex As Exception
                ProcessModuleLoadException(Me, ex)
            End Try

        End Sub

        Private Sub cmdSaveAndContinue_Click(ByVal sender As Object, ByVal e As EventArgs) Handles cmdSaveAndContinue.Click
            Try
                SaveSettings()
            Catch ex As Exception
                ProcessModuleLoadException(Me, ex)
            End Try
        End Sub


        Private Sub cmdSaveAndReturn_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmdSaveAndReturn.Click
            Try
                SaveSettings()
                Response.Redirect(NavigateURL())
            Catch ex As Exception
                ProcessModuleLoadException(Me, ex)
            End Try
        End Sub

        Private Sub cmdCancel_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmdCancel.Click
            Response.Redirect(NavigateURL())
        End Sub
#End Region

#Region "Private methods"

        Private Sub SaveSettings()
            Dim resultSettings As LuceneResultsSettings = DirectCast(RS.DataSource, LuceneResultsSettings)

            ' on force la sauvegarde des diffrentes valeurs
            TemplateProvider.FromSession(SessionStateUtility.GetHttpSessionStateFromContext(HttpContext.Current)).OverrideCurrent()
            TemplateProvider.Usage.CommitAll()

            PortalQueryingDriver.GetCurrent.LuceneResultsSettings(ModuleId) = resultSettings
            Aricie.Services.CacheHelper.RemoveCache(Common.Constants.Cache.DependencyKey)
        End Sub

        Private Sub EnforceFreeVersion()
            Dim tc As TrialController = TrialController.Instance(New TrialProvider())
            Dim objTrialStatus As TrialStatusInfo = tc.LimitModule(Me, True)
            RS.TrialStatus = objTrialStatus
        End Sub

#End Region


    End Class

End Namespace
