Imports Aricie.DNN.Modules.LuceneSearch.Business
Imports DotNetNuke
Imports Lucene.Net.Index
Imports Lucene.Net.Search
Imports DotNetNuke.UI.Utilities
Imports DotNetNuke.UI.WebControls
Imports Aricie.Services

Namespace Aricie.DNN.Modules.LuceneSearch.UI

    Partial Public Class TabsFilter
        Inherits LuceneFilterControlBase


#Region "Event Handlers"

        Protected Sub Page_Init(ByVal sender As Object, ByVal e As EventArgs) Handles Me.Init
            If Not IsPostBack Then
                BindLayout()
            End If
        End Sub

        Protected Sub chkAllModules_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) _
            Handles chkAllTabs.CheckedChanged
            OnCheckedChanged()
        End Sub

        Protected Sub Page_PreRender(ByVal sender As Object, ByVal e As EventArgs) Handles Me.PreRender
            If Me.chkAllTabs.Checked Then
                If Me.IsPostBack Then
                    OnCheckedChanged()
                Else
                    HackSetVisibleFalse()
                End If
                'Else

                '    HackSetVisibleTrue()
            End If
        End Sub

#End Region


#Region "ILuceneFilterControl implements"

        Public Overrides Property Enabled() As Boolean
            Get
                Return Me.tabTree.Enabled
            End Get
            Set(ByVal value As Boolean)
                Me.tabTree.Enabled = value
                Me.chkAllTabs.Enabled = value
            End Set
        End Property

        Public Overrides Function GetFilter() As LuceneFilterQuery
            Dim toReturn As New LuceneFilterQuery
            Dim objQuery As BooleanQuery = Nothing

            If Not Me.chkAllTabs.Checked Then
                objQuery = New BooleanQuery

                For Each tabId As Integer In GetSelectedNodes()
                    Dim tabValue As String = BusinessController.FormatSearchCriteria(tabId.ToString)
                    Dim tabTerm As New Term(DnnSearchItemField.TabId, tabValue)
                    objQuery.Add(New TermQuery(tabTerm), Occur.SHOULD)
                Next
            End If
            toReturn.FilterQuery = objQuery
            Return toReturn

        End Function

        Public Overrides Function HasFilter() As Boolean
            Return Me.chkAllTabs.Checked = False AndAlso Me.GetSelectedNodes.Count > 0
        End Function

        Public Overrides Function GetFieldNames() As IEnumerable(Of String)
            Return New String() {DnnSearchItemField.TabId}
        End Function

#Region "Validation du filtre"

        Private Function IsBooleanQueryValid(q As BooleanQuery) As Boolean
            For Each c As BooleanClause In q.GetClauses()
                If Not TypeOf c.Query() Is TermQuery Then
                    Return False
                End If
            Next
            Return True
        End Function

        Private Function CheckMatchingQuery(q As Query) As Boolean
            Dim Filter = Filtering.FluentFilterRulesManagerFactory.Init() _
                         .Expects(Of TermQuery)("TabId unique") _
                         .ExpectsComplex("Plusieurs TabId", AddressOf IsBooleanQueryValid) _
                         .Create()
            Return Filter.Matches(q)
        End Function

#End Region

        Public Overrides Function ParseFilter(ByVal objFilter As LuceneFilterQuery) As Boolean
            If Not CheckMatchingQuery(objFilter.FilterQuery) Then
                Return False
            End If

            Dim Terms As New HashSet(Of Term)
            objFilter.FilterQuery.ExtractTerms(Terms)

            Dim TabIds As New List(Of String)

            For Each k As Term In Terms
                If k.Field <> DnnSearchItemField.TabId Then
                    Return False ' on a un champ diffrent de tabid, on sort immdiatement
                End If
                TabIds.Add(k.Text)
            Next

            Dim TabIdsWereSelected As Boolean = False
            For Each TabId As String In TabIds
                For Each objNode As System.Web.UI.WebControls.TreeNode In Me.tabTree.Nodes
                    If SelectRecurse(TabId, objNode) Then
                        TabIdsWereSelected = True
                    End If
                Next
            Next

            If TabIdsWereSelected Then
                chkAllTabs.Checked = False
                OnCheckedChanged()
            End If

            Return True
        End Function


#End Region

#Region "Private methods"

        Private Sub BindLayout()
            Me.CreateTree()
        End Sub


        Private Function SelectRecurse(ByVal tabValue As String, ByVal objNode As System.Web.UI.WebControls.TreeNode) As Boolean

            If objNode.Value.Substring(objNode.Value.LastIndexOf("\"c) + 1) = tabValue Then

                objNode.Checked = True
                If objNode.Parent IsNot Nothing Then
                    ExpandRecurse(objNode)
                End If
                Return True
            End If
            For Each objChildNode As System.Web.UI.WebControls.TreeNode In objNode.ChildNodes
                If SelectRecurse(tabValue, objChildNode) Then
                    Return True
                End If
            Next
            Return False
        End Function

        Private Sub ExpandRecurse(ByVal objNode As System.Web.UI.WebControls.TreeNode)
            If objNode.Parent IsNot Nothing Then
                objNode.Parent.Expanded = True
                ExpandRecurse(objNode.Parent)
            End If
        End Sub

        Private Sub OnCheckedChanged()
            Me.trTabs.Visible = Not Me.chkAllTabs.Checked
        End Sub

        ''' <summary>
        ''' Create the tree with all the allowed tab in localized version
        ''' </summary>
        Private Sub CreateTree()
            Dim namespaceDnn As String = "DotNetNuke.NavDataSource"
            Dim namespaceWebControls As String = "DotNetNuke.UI.WebControls.NavDataSource"
            If Version() < 5 Then
                Dim dnn As Object = ReflectionHelper.CreateObject(namespaceDnn)
                tabTree.DataSource = dnn

            Else
                Dim webcontrol As Object = ReflectionHelper.CreateObject(namespaceWebControls)
                tabTree.DataSource = webcontrol
            End If

            tabTree.DataBind()

            For Each node As System.Web.UI.WebControls.TreeNode In tabTree.Nodes
                UpdateRecurse(node)
            Next
        End Sub

        Public Shared Function Version() As Integer
            Return ReflectionHelper.CreateType("DotNetNuke.UI.WebControls.EditControl").Assembly.GetName.Version.Major()
        End Function


        Private Sub UpdateRecurse(ByRef node As System.Web.UI.WebControls.TreeNode)
            node.Checked = False
            node.NavigateUrl = ""
            For Each child As System.Web.UI.WebControls.TreeNode In node.ChildNodes
                UpdateRecurse(child)
            Next

        End Sub

        ''' <summary>
        ''' Get all the selected node of the treeview
        ''' </summary>
        ''' <returns>Arraylist containing TabId</returns>
        Private Function GetSelectedNodes() As List(Of Integer)
            Dim toReturn As New List(Of Integer)
            For Each checkedNode As System.Web.UI.WebControls.TreeNode In tabTree.CheckedNodes
                toReturn.Add(Integer.Parse(checkedNode.Value.Substring(checkedNode.Value.LastIndexOf("\"c) + 1)))
            Next
            Return toReturn
        End Function

        Private Sub HackSetVisibleFalse()
            DNNClientAPI.AddBodyOnloadEventHandler(Me.Page, " if (dnn.dom.getById('" & Me.trTabs.ClientID & "') != null){dnn.dom.getById('" & Me.trTabs.ClientID & "').style.display = 'none';};")
        End Sub

#End Region
    End Class
End Namespace
