Imports Aricie.DNN.Modules.LuceneSearch.Business
Imports Aricie.DNN.Services
Imports System.Globalization
Imports DotNetNuke.Services.Search
Imports Lucene.Net.Documents
Imports System.Reflection

Imports DotNetNuke.Entities.Portals
Imports DotNetNuke.Entities.Users
Imports System.Web
Imports System.Linq



Namespace Aricie.DNN.Modules.LuceneSearch.ModuleProviders
    Public Class DocumentExchangeProvider
        Implements DotNetNuke.Entities.Modules.ISearchable
        Implements Aricie.DNN.Modules.LuceneSearch.Business.ILuceneFieldGlossary

#Region "Fields"

        Public Class Fields
            Public Shared DMXEntryType As String = "DMXEntryType"
            Public Shared DMXCollectionId As String = "DMXCollectionId"
            Public Shared DMXSharedDoc As String = "DMXSharedDoc"
        End Class

#End Region



#Region "Implement ILuceneFieldGlossary"
        Public Overridable Function GetFieldDefinitions() As IList(Of FieldDefinition) _
            Implements ILuceneFieldGlossary.GetFieldDefinitions
            Dim toReturn As New List(Of FieldDefinition)
            toReturn.Add(New FieldDefinition(Fields.DMXEntryType, Field.Store.YES, Field.Index.NOT_ANALYZED, Field.TermVector.NO, False))
            toReturn.Add(New FieldDefinition(Fields.DMXCollectionId, Field.Store.YES, Field.Index.NOT_ANALYZED, Field.TermVector.NO, False))
            toReturn.Add(New FieldDefinition(Fields.DMXSharedDoc, Field.Store.YES, Field.Index.NOT_ANALYZED, Field.TermVector.NO, False))
            Return toReturn
        End Function

#End Region

        Private Property DMXAssembly As Assembly
        Private Property DMXVersion As System.Version

        Private _EntryInfoType As Type
        Private ReadOnly Property EntryInfoType As Type
            Get
                Return _EntryInfoType
            End Get
        End Property

        Private _CategoryInfoType As Type
        Private ReadOnly Property CategoryInfoType As Type
            Get
                Return _CategoryInfoType
            End Get
        End Property

        Private _ExtensionInfoType As Type
        Private ReadOnly Property ExtensionInfoType As Type
            Get
                Return _ExtensionInfoType
            End Get
        End Property

        Public Sub New()
            ' prenons les informations ncessaires sur le numro de version
            DMXAssembly = Assembly.Load("Bring2mind.DNN.Modules.DMX.Core")
            DMXVersion = DMXAssembly.GetName.Version

            If (DMXVersion.Major >= 6) Then
                _EntryInfoType = DMXAssembly.GetType("Bring2mind.DNN.Modules.DMX.Entities.Entries.EntryInfo")
                _CategoryInfoType = DMXAssembly.GetType("Bring2mind.DNN.Modules.DMX.Entities.Categories.CategoryInfo")
                _ExtensionInfoType = DMXAssembly.GetType("Bring2mind.DNN.Modules.DMX.Framework.ExtensionInfo")
            Else
                _EntryInfoType = DMXAssembly.GetType("Bring2mind.DNN.Modules.DMX.Business.EntryInfo")
                _CategoryInfoType = DMXAssembly.GetType("Bring2mind.DNN.Modules.DMX.Business.CategoryInfo")
                _ExtensionInfoType = DMXAssembly.GetType("Bring2mind.DNN.Modules.DMX.Business.ExtensionInfo")
            End If

        End Sub

        Private Function GetEntryInfos(PortalId As Integer, Optional ModuleId As Integer = 0) As IEnumerable
            Dim entries As IEnumerable = Nothing
            'TODO: remplacer cette dtermination du type du controller par la valeur fournie dans le moduleInfo
            If (DMXVersion.Major >= 6) Then

                ' retrieve root path for module
                Dim ModuleSettingsType As Type = DMXAssembly.GetType("Bring2mind.DNN.Modules.DMX.Common.Settings.ModuleSettings")
                Dim GetModuleSettingsMethod As MethodInfo = ModuleSettingsType.GetMethod("GetModuleSettings") 'entry = Bring2mind.DNN.Modules.DMX.Common.Settings.ModuleSettings.GetModuleSettings(ModuleID)
                Dim ModuleSettingsValue = GetModuleSettingsMethod.Invoke(Nothing, New Object() {ModuleId})
                Dim RootId = GetPropertyAs(Of String)("RootId", ModuleSettingsValue)

                entries = CallGetEntriesByPortal(DMXAssembly.GetType("Bring2mind.DNN.Modules.DMX.Entities.Entries.EntriesController"), PortalId)

                Dim FilteredEntries = New List(Of Object)

                For Each entry As Object In entries
                    Dim Path As String = GetPropertyAs(Of String)("Path", entry)
                    Dim PathElements As String() = Path.Split(New String() {";"}, StringSplitOptions.RemoveEmptyEntries)
                    If (PathElements.Contains(RootId)) Then
                        FilteredEntries.add(entry)
                    End If
                Next
                entries = FilteredEntries
            Else
                If (DMXVersion.Major = 5 And DMXVersion.Minor >= 3) Then
                    entries = CallGetEntriesByPortal(DMXAssembly.GetType("Bring2mind.DNN.Modules.DMX.Business.EntriesController"), PortalId)
                Else
                    entries = CallGetEntriesByPortalOlderVersions(DMXAssembly.GetType("Bring2mind.DNN.Modules.DMX.Business.EntriesController"), PortalId)
                End If
            End If

            Return entries
        End Function

        Private Function CallGetEntriesByPortal(t As Type, currentPortal As Integer) As IEnumerable
            Dim getEntriesByPortal As MethodInfo = t.GetMethod("GetEntriesByPortal", New Type() {GetType(Integer), GetType(Boolean), GetType(Integer), GetType(Integer)})
            Return getEntriesByPortal.Invoke(Nothing, New Object() {currentPortal, False, 0, 0})
        End Function

        Private Function CallGetEntriesByPortalOlderVersions(t As Type, currentPortal As Integer) As IEnumerable
            Dim getEntriesByPortal As MethodInfo = t.GetMethod("GetEntriesByPortal", New Type() {GetType(Integer), GetType(Boolean)})
            Return getEntriesByPortal.Invoke(Nothing, New Object() {currentPortal, False})
        End Function

        Private Function GetPropertyAs(Of T)(PropertyName As String, InnerObject As Object) As T
            Return Aricie.Services.ReflectionHelper.GetProperty(InnerObject.GetType, PropertyName, InnerObject)
        End Function

        Private Function GetIndexedPropertyAs(Of T)(PropertyName As String, InnerObject As Object, key As String) As T
            Dim pi As PropertyInfo = InnerObject.GetType().GetProperty(PropertyName)
            Return pi.GetValue(InnerObject, New Object() {key})
        End Function

#Region "Implement DotNetNuke ISearchable"
        Public Function GetSearchItems(ByVal modInfo As DotNetNuke.Entities.Modules.ModuleInfo) As DotNetNuke.Services.Search.SearchItemInfoCollection Implements DotNetNuke.Entities.Modules.ISearchable.GetSearchItems
            Dim toReturn As New SearchItemInfoCollection
            Dim entries As IEnumerable = GetEntryInfos(modInfo.PortalID, modInfo.ModuleID)

            For Each entry As Object In entries
                Try
                    If GetPropertyAs(Of Boolean)("IsApproved", entry) Then
                        Dim item As New LuceneSearchItemInfo(modInfo.PortalID) With {.ModuleId = modInfo.ModuleID}
                        Me.FillSearchItem(item, entry)
                        toReturn.Add(item)
                    End If
                Catch ex As Exception
                    DotNetNuke.Services.Exceptions.Exceptions.LogException(ex)
                End Try
            Next

            Return toReturn
        End Function
#End Region

#Region "Private Methods"
        Protected Overridable Sub FillSearchItem(ByRef item As LuceneSearchItemInfo, ByVal entry As Object)

            Dim factory As FieldFactory = FieldFactory.Instance
            item.Title = GetPropertyAs(Of String)("Title", entry)
            item.PubDate = GetPropertyAs(Of Date)("LastModified", entry)
            item.Description = GetPropertyAs(Of String)("Remarks", entry)
            item.Author = GetPropertyAs(Of String)("Owner", entry)

            Dim Version6OuPlus As Func(Of System.Version, Boolean) = Function(v) v.Major >= 6

            Dim categories As IList = GetIndexedPropertyAs(Of IList)("Categories", entry, Aricie.DNN.Services.NukeHelper.PortalInfo(item.PortalId).DefaultLanguage)
            If categories.Count > 0 Then
                Dim CategoryNameProperty As String = ValueIf(Version6OuPlus, "CategoryName", "Name")
                Dim CategoryIdProperty As String = ValueIf(Version6OuPlus, "CategoryId", "id")
                
                item.Category = GetPropertyAs(Of String)(CategoryNameProperty, categories(0))
                item.CategoryId = GetPropertyAs(Of String)(CategoryIdProperty, categories(0))
                For Each cat As Object In categories
                    item.AdditionalFields.Add(factory.CreateField(LuceneGeneralField.SubCategories, GetPropertyAs(Of String)(CategoryNameProperty, cat)))
                Next
            End If

            item.ItemId = GetPropertyAs(Of Integer)("EntryId", entry)
            item.SearchKey = String.Format("Entry-{0}", item.ItemId)
            item.GUID = String.Format("EntryId={0}", item.ItemId)

            item.AdditionalFields.Add(factory.CreateField(Fields.DMXEntryType, GetPropertyAs(Of String)("EntryType", entry)))
            item.AdditionalFields.Add(factory.CreateField(Fields.DMXCollectionId, GetPropertyAs(Of Integer)("CollectionId", entry)))
            item.AdditionalFields.Add(factory.CreateField(LuceneGeneralField.ParentItemId, GetPropertyAs(Of Integer)("CollectionId", entry)))
            item.AdditionalFields.Add(factory.CreateField(LuceneGeneralField.Path, GetPropertyAs(Of String)("Path", entry)))
            item.AdditionalFields.Add(factory.CreateField(LuceneGeneralField.FileName, GetPropertyAs(Of String)("OriginalFileName", entry)))
            item.AdditionalFields.Add(factory.CreateField(LuceneGeneralField.Version, GetPropertyAs(Of Integer)("Version", entry).ToString(CultureInfo.InvariantCulture)))
            item.AdditionalFields.Add(factory.CreateField(LuceneGeneralField.FileSize, GetPropertyAs(Of Integer)("FileSize", entry).ToString(CultureInfo.InvariantCulture)))
            item.AdditionalFields.Add(factory.CreateField(LuceneGeneralField.KeyWords, GetPropertyAs(Of String)("Keywords", entry)))

            Dim extension As Object = GetPropertyAs(Of Object)("Extension", entry)
            item.AdditionalFields.Add(factory.CreateField(LuceneGeneralField.Extension, GetPropertyAs(Of String)("ExtensionKey", extension)))
            item.AdditionalFields.Add(factory.CreateField(LuceneGeneralField.IconPath, "DesktopModules/Bring2mind/DMX/images/" & Replace(GetPropertyAs(Of String)("Icon16", extension), "-", ".")))

            item.AdditionalFields.Add(factory.CreateField(LuceneGeneralField.IsDeleted, GetPropertyAs(Of Boolean)("Deleted", entry)))
            item.AdditionalFields.Add(factory.CreateField(LuceneGeneralField.IsApproved, GetPropertyAs(Of Boolean)("IsApproved", entry)))
            item.AdditionalFields.Add(factory.CreateField(LuceneGeneralField.LastVersionId, GetPropertyAs(Of Integer)("LastVersionId", entry)))
            item.AdditionalFields.Add(factory.CreateField(LuceneGeneralField.State, GetPropertyAs(Of String)("State", entry)))
            item.AdditionalFields.Add(factory.CreateField(LuceneGeneralField.AuthorName, GetPropertyAs(Of String)("Author", entry)))

            'todo: deal with content

            item.InheritModuleViewPermissions = False
            Dim objUser As UserInfo = UserController.GetUser(item.PortalId, entry.Owner, True)
            Dim isShared As Boolean
            For Each permissionEntry As Object In GetPropertyAs(Of IEnumerable)("Permissions", entry)
                If GetPropertyAs(Of String)("RoleName", permissionEntry) <> "Administrators" Then
                    If GetPropertyAs(Of Boolean)("AllowAccess", permissionEntry) AndAlso GetPropertyAs(Of String)("PermissionKey", permissionEntry) = "VIEW" Then
                        If String.IsNullOrEmpty(GetPropertyAs(Of String)("RoleName", permissionEntry)) Then
                            Dim UserId As Integer = GetPropertyAs(Of Integer)(ValueIf(Version6OuPlus, "UserId", "UserID"), permissionEntry)
                            item.ItemUserRoleIds.Add(UserId)
                            If UserId <> entry.Owner Then
                                isShared = True
                            End If
                        Else
                            item.ItemViewRoleIds.Add(GetPropertyAs(Of Integer)(ValueIf(Version6OuPlus, "RoleId", "RoleID"), permissionEntry))
                            isShared = True
                        End If
                    End If
                End If
            Next
            If isShared Then
                item.AdditionalFields.Add(factory.CreateField(Fields.DMXSharedDoc, (True).ToString(CultureInfo.InvariantCulture)))
            End If
        End Sub

        Private Function ValueIf(f As Func(Of System.Version, Boolean), Y As String, N As String) As String
            If f(DMXVersion) Then
                Return Y
            Else
                Return N
            End If
        End Function
#End Region

    End Class
End Namespace

