Imports Aricie.DNN.Modules.LuceneSearch.Business
Imports Aricie.DNN.Modules.ModuleWorkflows.Business
Imports Aricie.DNN.Modules.ModuleWorkflows.Settings
Imports Lucene.Net.Documents
Imports Aricie.DNN.Settings
Imports Aricie.DNN.Services
Imports DotNetNuke.Services.Exceptions
Imports DotNetNuke.Entities.Modules.Definitions
Imports DotNetNuke.Entities.Modules
Imports Aricie.DNN.Services.Filtering
Imports DotNetNuke.Services.Search
Imports Aricie.DNN.Modules.LuceneSearch.Settings

Namespace Aricie.DNN.Modules.LuceneSearch.ModuleProviders
    Public Class ModuleWorkFlowsProvider
        Implements ISearchable
        Implements ILuceneFieldGlossary
        Implements ILuceneSearchableUpgrade

#Region "Private Members"
        Protected _WorkflowSettings As Aricie.DNN.Modules.ModuleWorkflows.Settings.WorkflowSettings
#End Region

#Region "Properties"
        Public Property WorkflowSettings() As Aricie.DNN.Modules.ModuleWorkflows.Settings.WorkflowSettings
            Get
                If Me._WorkflowSettings Is Nothing Then
                    Me._WorkflowSettings = SettingsController.GetModuleSettings(Of WorkflowSettings)(SettingsScope.PortalSettings, NukeHelper.PortalId)
                End If
                Return Me._WorkflowSettings
            End Get
            Set(ByVal Value As Aricie.DNN.Modules.ModuleWorkflows.Settings.WorkflowSettings)
                Me._WorkflowSettings = Value
            End Set
        End Property
#End Region

#Region "Implement ISearchable"
        Public Function GetSearchItems(ByVal modInfo As DotNetNuke.Entities.Modules.ModuleInfo) As DotNetNuke.Services.Search.SearchItemInfoCollection Implements DotNetNuke.Entities.Modules.ISearchable.GetSearchItems
            Dim toReturn As New SearchItemInfoCollection
            Dim wrapperSettings As WrapperSettings = SettingsController.GetModuleSettings(Of WrapperSettings)(SettingsScope.ModuleSettings, modInfo.ModuleID)


            ' on appelle GetAdditionalSearchItems pour le module encapsul
            Dim mdc As New ModuleDefinitionController
            Dim dmc As New DesktopModuleController

            Dim moduleName As String = dmc.GetDesktopModule(mdc.GetModuleDefinition(wrapperSettings.ModuleDefId).DesktopModuleID).ModuleName

            Dim searchProvider As LuceneModuleProviderConfig = ProviderFactory.Instance.GetSearchableFromModuleName(moduleName)
            If searchProvider IsNot Nothing Then
                Try
                    toReturn.AddRange(searchProvider.AsSearchable.GetSearchItems(modInfo))
                Catch ex As Exception
                    LogException(ex)
                End Try
            End If


            Return toReturn
        End Function
#End Region

#Region "Implement ILuceneSearchableUpgrade"
        Public Sub UpgradeSearchItem(ByVal portalId As Integer, ByRef searchItem As LuceneSearchItemInfo) Implements ILuceneSearchableUpgrade.UpgradeSearchItem

            Dim ContentController As New ContentController()
            Dim ContentInf As ContentInfo
            Dim wrapperSettings As WrapperSettings = GetModuleSettings(Of WrapperSettings)(SettingsScope.ModuleSettings, searchItem.ModuleId)
            Dim factory As FieldFactory = FieldFactory.Instance

            Dim sk As String = searchItem.SearchKey
            If sk.LastIndexOf("-") > 0 Then
                sk = sk.Substring(sk.LastIndexOf("-"c) + 1)
            End If

            ' On ne peut faire appelle  AdvancedTokenReplace  cause de l'indexation automatique. 
            'La fonction DNN ReplaceToken plante lors de l'indexation automatique

            'If wrapperSettings.ApplyTokenReplace Then
            '    Dim atp As AdvancedTokenReplace = GetAdvancedTokenReplace(portalId, searchItem.TabId, searchItem.ModuleId)

            '    searchItem.Description = atp.ReplaceAllTokens(searchItem.Description)
            '    searchItem.Content = atp.ReplaceAllTokens(searchItem.Content)
            'End If


            Dim itemId As Integer
            If Integer.TryParse(sk, itemId) Then
                ContentInf = ContentController.GetContent(searchItem.ModuleId, itemId)

                If wrapperSettings.IsLocalized Then
                    searchItem.Language = wrapperSettings.LocaleCode
                End If

                searchItem.SiteMapFrequency = ContentInf.MetaData.Frequency
                searchItem.SiteMapPriority = ContentInf.MetaData.Priority

                If ContentInf IsNot Nothing Then

                    searchItem.AdditionalFields.Add(factory.CreateField("Status", ContentInf.Status))
                    searchItem.AdditionalFields.Add(factory.CreateField("Version", ContentInf.Version))
                End If
            End If


            ' on appelle UpgradeSearchItem pour le module encapsul
            Dim mdc As New ModuleDefinitionController
            Dim dmc As New DesktopModuleController

            Dim moduleName As String = dmc.GetDesktopModule(mdc.GetModuleDefinition(wrapperSettings.ModuleDefId).DesktopModuleID).ModuleName
            searchItem.AdditionalFields.Add(factory.CreateField("WrappedModuleName", moduleName))
            Dim searchProvider As LuceneModuleProviderConfig = ProviderFactory.Instance.GetUpgraderFromModuleName(moduleName)
            If searchProvider IsNot Nothing Then
                Try
                    searchProvider.AsUpgrader.UpgradeSearchItem(portalId, searchItem)
                Catch ex As Exception
                    LogException(ex)
                End Try
            End If


        End Sub
#End Region

#Region "Implement ILuceneFieldGlossary"
        Public Function GetFieldDefinitions() As IList(Of FieldDefinition) _
            Implements ILuceneFieldGlossary.GetFieldDefinitions
            Dim toReturn As New List(Of FieldDefinition)
            toReturn.Add(New FieldDefinition("Status", Field.Store.YES, Field.Index.NOT_ANALYZED, _
                                               Field.TermVector.NO, False, 1))
            toReturn.Add(New FieldDefinition("WrappedModuleName", Field.Store.YES, Field.Index.NOT_ANALYZED, _
                                               Field.TermVector.NO, False, 1))
            Return toReturn
        End Function
#End Region

#Region "Methods"
        Public Function GetAdvancedTokenReplace(ByVal portalId As Integer, ByVal tabId As Integer, ByVal moduleId As Integer) As AdvancedTokenReplace
            Dim toReturn As AdvancedTokenReplace
            Dim mc As New ModuleController
            Dim mInfo As ModuleInfo = mc.GetModule(moduleId, tabId)

            If DnnContext.Current.AdvancedTokenReplace Is Nothing Then
                toReturn = New AdvancedTokenReplace(mInfo)
                Dim tokenSource As TokenSourceInfo = Me.WorkflowSettings.FilteringSettings.TokenSource
                tokenSource.SetTokens(toReturn)
                DnnContext.Current.AdvancedTokenReplace = toReturn
            Else
                toReturn = DnnContext.Current.AdvancedTokenReplace
                toReturn.SetModule(mInfo)
                'toReturn.SetObjectReplace(DnnContext.Instance, "Context")
                'toReturn.ResourceFile = Me.SharedResourceFile
            End If


            Return toReturn
        End Function
#End Region


    End Class
End Namespace

