Imports Aricie.Services
Imports Aricie.DNN.Settings
Imports Aricie.DNN.Services
Imports Aricie.DNN.Modules.LuceneSearch.Business
Imports Aricie.DNN.Modules.ModuleWorkflows
Imports Aricie.DNN.Modules.ModuleWorkflows.Business
Imports Aricie.DNN.Modules.ModuleWorkflows.Settings
Imports Lucene.Net.Documents
Imports Ventrian.PropertyAgent
Imports DotNetNuke.Entities.Modules

Namespace Aricie.DNN.Modules.LuceneSearch.ModuleProviders
    Public Class PropertyAgentProvider
        Implements ILuceneFieldGlossary
        Implements ILuceneSearchableUpgrade

#Region "Private Constants"
        Private Const glbDesktopModuleName As String = "PropertyAgent"
        Private Const glbCustomFieldPrefix As String = "PACustom"
        Private Const glbCustomFieldUntokenizedPrefix As String = "PAUCustom"
        Private Const glbFieldUntokenizedPrefix As String = "PAU"
        Private Const PropertyTypeFields As String = "Type"
        Private Const glbThematiquePrefix As String = "Thematique"
#End Region

#Region "Private Shared Members"
        Private Shared _PropertyController As PropertyController
     
#End Region

#Region "Private Properties"
        Private ReadOnly Property PropController() As PropertyController
            Get
                If _PropertyController Is Nothing Then
                    _PropertyController = New PropertyController
                End If
                Return _PropertyController
            End Get
        End Property

#End Region

#Region "Implement ILuceneSearchableUpgrade"
        Public Sub UpgradeSearchItem(ByVal portalId As Integer, ByRef searchItem As LuceneSearchItemInfo) _
            Implements ILuceneSearchableUpgrade.UpgradeSearchItem

            Dim propId As Integer = Integer.Parse(searchItem.SearchKey)
            Dim prop As PropertyInfo = PropController.Get(propId)
            Dim value As String
            Dim factory As FieldFactory = FieldFactory.Instance
            Dim cFields As Dictionary(Of Integer, CustomFieldInfo) = Me.GetCustomFields
            If prop IsNot Nothing AndAlso prop.PropertyID <> -1 Then
                searchItem.AdditionalFields.Add(factory.CreateField(glbFieldUntokenizedPrefix & PropertyTypeFields, prop.PropertyTypeName))
                For Each customFieldId As Integer In prop.PropertyList.Keys
                    If cFields.ContainsKey(customFieldId) Then
                        Dim customField As CustomFieldInfo = cFields(customFieldId)
                        If customField.IsSearchable Then
                            If prop.PropertyList.ContainsKey(customFieldId) Then
                                value = prop.PropertyList(customFieldId).ToString
                                searchItem.AdditionalFields.Add(factory.CreateField(glbCustomFieldPrefix & customField.Name, value))
                                searchItem.AdditionalFields.Add(factory.CreateField(glbCustomFieldUntokenizedPrefix & customField.Name, value))
                            End If
                        End If

                    End If
                Next
                searchItem.PubDate = prop.DatePublished
                Dim thematique As String = GetCategory(prop)
                If Not String.IsNullOrEmpty(thematique) Then

                    searchItem.AdditionalFields.Add(factory.CreateField(glbThematiquePrefix, thematique))
                End If
                'Dim propValue As PropertyValueInfo = _PropertyValueController.GetByCustomField(propertyId, customFieldId)
                'toreturn = propValue.CustomValue
            End If

        End Sub
#End Region

#Region "Implement ILuceneFieldGlossary"
        Public Function GetFieldDefinitions() As IList(Of FieldDefinition) _
            Implements ILuceneFieldGlossary.GetFieldDefinitions
            Dim toReturn As List(Of FieldDefinition)
            Dim tempDico As New Dictionary(Of String, FieldDefinition)
            Dim customFields As Dictionary(Of Integer, CustomFieldInfo) = GetCustomFields()
            For Each objCustomField As CustomFieldInfo In customFields.Values
                If objCustomField.IsSearchable Then
                    tempDico(glbCustomFieldPrefix & objCustomField.Name) = New FieldDefinition(glbCustomFieldPrefix & objCustomField.Name, Field.Store.YES, Field.Index.ANALYZED, Field.TermVector.NO, False, 1)
                    tempDico(glbCustomFieldUntokenizedPrefix & objCustomField.Name) = New FieldDefinition(glbCustomFieldUntokenizedPrefix & objCustomField.Name, Field.Store.YES, Field.Index.NOT_ANALYZED, Field.TermVector.NO, False, 1)
                End If
            Next
            toReturn = New List(Of FieldDefinition)(tempDico.Values)
            'On rajoute dans l'index le type de catgorie de la proprit
            toReturn.Add(New FieldDefinition(glbFieldUntokenizedPrefix & PropertyTypeFields, Field.Store.YES, Field.Index.NOT_ANALYZED, Field.TermVector.NO, False, 1))
            Return toReturn
        End Function
#End Region

#Region "Methods"
        'Public Function GetFieldDefinitions(ByVal modID As Integer) As IList(Of FieldDefinition)

        '    Dim toReturn As List(Of FieldDefinition)
        '    Dim tempDico As New Dictionary(Of String, FieldDefinition)
        '    Dim customFields As Dictionary(Of Integer, CustomFieldInfo) = GetCustomFields(modID)
        '    For Each objCustomField As CustomFieldInfo In customFields.Values
        '        If objCustomField.IsSearchable Then
        '            tempDico(glbCustomFieldPrefix & objCustomField.Name) = New FieldDefinition(glbCustomFieldPrefix & objCustomField.Name, Field.Store.YES, Field.Index.ANALYZED, Field.TermVector.NO, False, 1)
        '            tempDico(glbCustomFieldUntokenizedPrefix & objCustomField.Name) = New FieldDefinition(glbCustomFieldUntokenizedPrefix & objCustomField.Name, Field.Store.YES, Field.Index.NOT_ANALYZED, Field.TermVector.NO, False, 1)
        '        End If
        '    Next
        '    toReturn = New List(Of FieldDefinition)(tempDico.Values)
        '    Return toReturn
        'End Function


        Private Function GetCustomFields() As Dictionary(Of Integer, CustomFieldInfo)
            Dim toReturn As Dictionary(Of Integer, CustomFieldInfo) = CacheHelper.GetGlobal(Of Dictionary(Of Integer, CustomFieldInfo))()
            If toReturn Is Nothing Then
                toReturn = New Dictionary(Of Integer, CustomFieldInfo)
                Dim mDefId As Integer = NukeHelper.GetModuleDefIdByModuleName(glbDesktopModuleName)
                If mDefId <> -1 Then

                    For Each myPortalID As Integer In NukeHelper.PortalIds()

                        Dim moduleDefFriendlyName As String = NukeHelper.ModuleDefinitionController.GetModuleDefinition(mDefId).FriendlyName
                        Dim propModules As ArrayList = NukeHelper.ModuleController.GetModulesByDefinition(myPortalID, moduleDefFriendlyName)
                        Dim customFields As List(Of CustomFieldInfo)
                        Dim objCustomFieldController As New CustomFieldController
                        For Each propModule As ModuleInfo In propModules
                            customFields = objCustomFieldController.List(propModule.ModuleID, True)
                            For Each customField As CustomFieldInfo In customFields
                                toReturn(customField.CustomFieldID) = customField
                            Next
                        Next
                        Dim propWrappedModules As List(Of ModuleInfo) = GetWrappedModules(myPortalID)
                        For Each propWrappedModule As ModuleInfo In propWrappedModules
                            Dim wrapperSettings As WrapperSettings = GetModuleSettings(Of WrapperSettings)(SettingsScope.ModuleSettings, propWrappedModule.ModuleID)
                            If mDefId = wrapperSettings.ModuleDefId Then
                                customFields = objCustomFieldController.List(propWrappedModule.ModuleID, True)
                                For Each customField As CustomFieldInfo In customFields
                                    toReturn(customField.CustomFieldID) = customField
                                Next
                            End If
                        Next

                    Next

                End If
                CacheHelper.SetGlobal(Of Dictionary(Of Integer, CustomFieldInfo))(toReturn)
            End If
            Return toReturn

        End Function

        'Private Function GetCustomFields(ByVal modID As Integer) As Dictionary(Of Integer, CustomFieldInfo)
        '    Dim toReturn As New Dictionary(Of Integer, CustomFieldInfo)
        '    'Dim mDefId As Integer = NukeHelper.GetModuleDefIdByModuleName(glbDesktopModuleName)
        '    If modID <> -1 Then
        '        Dim customFields As List(Of CustomFieldInfo)
        '        Dim objCustomFieldController As New CustomFieldController
        '        customFields = objCustomFieldController.List(modID, True)
        '        For Each customField As CustomFieldInfo In customFields
        '            If customField.IsPublished Then
        '                toReturn(customField.CustomFieldID) = customField
        '            End If
        '        Next
        '    End If
        '    Return toReturn
        'End Function

        Public Function GetCategory(ByVal prop As PropertyInfo) As String

            If prop IsNot Nothing Then
                Dim ContentCtrl As New ContentController()
                Dim CatList As List(Of CategoryInfo) = ContentCtrl.GetContentCategories(prop.ModuleID, prop.PropertyID)
                If CatList IsNot Nothing AndAlso CatList.Count > 0 Then
                    Return CatList(0).Name
                End If
            End If
            Return ""
        End Function
#End Region

    End Class
End Namespace

