Imports Aricie.DNN.Modules.LuceneSearch.Business
Imports Lucene.Net.Documents
Imports DotNetNuke.Modules.Repository
Imports System.IO
Imports DotNetNuke.Entities.Portals
Imports DotNetNuke.Entities.Users

Namespace Aricie.DNN.Modules.LuceneSearch.ModuleProviders
    Public Class RepositoryProvider
        Implements ILuceneFieldGlossary
        Implements ILuceneSearchableUpgrade

#Region "Private Members"
        Private _RepositoryController As RepositoryController

        Private _RepositoryAttributeValuesController As RepositoryAttributeValuesController
        Private _RepositoryAttributesController As RepositoryAttributesController
        Private _RepositoryObjectValuesController As RepositoryObjectValuesController
#End Region

#Region "Private Properties"
        Private ReadOnly Property RepositoryCtlr() As RepositoryController
            Get
                If _RepositoryController Is Nothing Then
                    _RepositoryController = New RepositoryController
                End If
                Return _RepositoryController
            End Get
        End Property

        Private ReadOnly Property RepositoryAttributeValuesCtlr() As RepositoryAttributeValuesController
            Get
                If _RepositoryAttributeValuesController Is Nothing Then
                    _RepositoryAttributeValuesController = New RepositoryAttributeValuesController
                End If
                Return _RepositoryAttributeValuesController
            End Get
        End Property

        Private ReadOnly Property RepositoryAttributesCtlr() As RepositoryAttributesController
            Get
                If _RepositoryAttributesController Is Nothing Then
                    _RepositoryAttributesController = New RepositoryAttributesController
                End If
                Return _RepositoryAttributesController
            End Get
        End Property

        Private ReadOnly Property RepositoryObjectValuesCtlr() As RepositoryObjectValuesController
            Get
                If _RepositoryObjectValuesController Is Nothing Then
                    _RepositoryObjectValuesController = New RepositoryObjectValuesController
                End If
                Return _RepositoryObjectValuesController
            End Get
        End Property
#End Region

#Region "Implement ILuceneSearchableUpgrade"
        Public Sub UpgradeSearchItem(ByVal portalId As Integer, ByRef searchItem As LuceneSearchItemInfo) _
            Implements ILuceneSearchableUpgrade.UpgradeSearchItem


            Dim repositoryId As Integer = Integer.Parse(searchItem.SearchKey)

            Dim repInfo As RepositoryInfo = Me.RepositoryCtlr.GetSingleRepositoryObject(repositoryId)

            If repInfo.FileName <> "" Then


                Dim folder = GetTargetFolder(searchItem.ModuleId, repInfo)

                Dim filePath As String = Path.Combine(folder, repInfo.FileName)

                searchItem.AdditionalFields = DocumentController.GetDocumentFromFile(filePath, False)


            End If

            searchItem.AdditionalFields.Add(New Field(LuceneGeneralField.ItemId, repInfo.ItemId.ToString, Field.Store.YES, Field.Index.NOT_ANALYZED))

            Dim attributes As ArrayList = Me.RepositoryAttributesCtlr.GetRepositoryAttributes(searchItem.ModuleId)
            Dim objectValue As RepositoryObjectValuesInfo
            Dim attributeStringToAdd As String = ""
            For Each objAttribute As RepositoryAttributesInfo In attributes

                For Each value As RepositoryAttributeValuesInfo In _
                    Me.RepositoryAttributeValuesCtlr.GetRepositoryAttributeValues(objAttribute.ItemID)
                    objectValue = _
                        Me.RepositoryObjectValuesCtlr.GetSingleRepositoryObjectValues(repInfo.ItemId, _
                                                                                              value.ItemID)
                    If Not IsNothing(objectValue) Then
                        'concatenation des mots cls
                        attributeStringToAdd &= value.ValueName & " , "
                    End If
                Next
                If attributeStringToAdd.Length > 0 Then
                    'Ajout des mots cls  la description
                    Dim factory As FieldFactory = FieldFactory.Instance
                    searchItem.AdditionalFields.Add(factory.CreateField(LuceneGeneralField.KeyWords, attributeStringToAdd))

                End If

            Next

        End Sub
#End Region

#Region "Implement ILuceneFieldGlossary"
        Public Function GetFieldDefinitions() As IList(Of FieldDefinition) _
         Implements ILuceneFieldGlossary.GetFieldDefinitions
            'all fields used by this provider are already defined in the DefaultGlossary
            Return Nothing
        End Function
#End Region

#Region "Methods"
        ''' <summary>
        ''' Get the Target Folder for the files for a given ModuleId and RepositoryInfo
        ''' </summary>
        ''' <param name="moduleid">ModuleID of the Repository</param>
        ''' <param name="pRepository">RepositoryInfo representing the repository</param>
        Private Function GetTargetFolder(ByVal moduleid As Integer, ByVal pRepository As RepositoryInfo) As String
            Dim path As String = ""
            Dim moduleSettings As Hashtable = PortalSettings.GetModuleSettings(moduleid)
            Dim currentUserInfo As UserInfo = UserController.GetCurrentUserInfo
            Dim rbl As New RepositoryBL

            If (pRepository.CreatedByUser <> "") Then
                If moduleSettings.Item("userfolders") Then
                    path = (moduleSettings.Item("folderlocation") & "\" & pRepository.CreatedByUser.ToString & "\")
                Else
                    path = (moduleSettings.Item("folderlocation") & "\")
                End If
            ElseIf moduleSettings.Item("userfolders") Then
                path = (moduleSettings.Item("folderlocation") & "\" & currentUserInfo.UserID.ToString & "\")
            Else
                path = (moduleSettings.Item("folderlocation") & "\")
            End If
            If Not Directory.Exists(path) Then
                Directory.CreateDirectory(path)
            End If
            Return path
        End Function
#End Region

    End Class
End Namespace

